// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/ringbuf/win32/rx_proc.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*
*
*	This module performs the work associated with acquiring data. This could
*	mean anything from reading data from a file or device to generating data
*	on the fly. This module is expected to start one or more threads to
*	perform this work rather than performing the work inline within the call
*	to get things started.
*
*/

#include <stddef.h>
#include <stdio.h>

#include "rx_proc.h"
#include "hpdi32_util.h"
#include "gsc_util.h"



/* variables	*************************************************************/

static FILE*	_file;



/*****************************************************************************
*
*	Function:	_acquire_data
*
*	Purpose:
*
*		Implement the operations that acquire data.
*		Any and all errors are reported via the "data" structure.
*
*	Arguments:
*
*		data	The process data we to use.
*
*		bm		The data transfer buffer we're to use.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _acquire_data(proc_data_t* data, gsc_util_buf_man_t* bm)
{
	long	l;
	U32*	ptr;
	U32		size;
	size_t	st;
	U32		status;

	switch (data->operation)
	{
		default:
		case RX_OP_INCREMENT:

			ptr		= (U32*) bm->buffer;
			size	= (U32) bm->size / 4;

			for (l = 0; l < (long) size; l++)
				ptr[l]	= l;

			bm->offset	= 0;
			bm->count	= size * 4;
			data->bytes	+= bm->count;
			break;

		case RX_OP_LOAD:

			st	= fread(bm->buffer, 1, bm->size, _file);

			if (st == bm->size)
			{
				bm->offset	= 0;
				bm->count	= (U32) st;
				data->bytes	+= st;
			}
			else if (feof(_file))
			{
				bm->offset	= 0;
				bm->count	= (U32) st;
				data->bytes	+= st;
				bm->eof		= 1;
				data->done	= 1;
				strcpy(data->text, "end of file");
			}
			else if (ferror(_file))
			{
				data->err	= 1;
				sprintf(data->text,
						"fread(0x%lX, 0x%lX, 0x%lX, 0x%lX) did not complet as expected",
						(long) bm->buffer,
						(long) 1,
						(long) bm->size,
						(long) _file);
				gsc_buf_man_stop(TRUE, NULL);
			}
			else
			{
				bm->offset	= 0;
				bm->count	= (U32) st;
				data->bytes	+= st;
			}

			break;

		case RX_OP_RECEIVE:

			status	= hpdi32_read(data->dev, bm->buffer, (U32) bm->size, &bm->count);

			if ((status != GSC_SUCCESS) || (bm->count != bm->size))
			{
				data->err	= 1;
				sprintf(data->text,
						"hpdi32_read(0x%lX, 0x%lX, %ld, %ld): status = 0x%lX",
						(long) data->dev,
						(long) bm->buffer,
						(long) bm->size,
						(long) bm->count,
						(long) status);
				gsc_buf_man_stop(TRUE, NULL);
			}
			else
			{
				bm->offset	= 0;
				data->bytes	+= bm->count;
			}

			break;
	}
}



/*****************************************************************************
*
*	Function:	_rx_process
*
*	Purpose:
*
*		Start the process/thread that acquires data.
*
*	Arguments:
*
*		arg		A pointer to the process data we to use.
*
*	Returned:
*
*		0		We don't use this.
*
*****************************************************************************/

static  U32 _rx_process(void* arg)
{
	gsc_util_buf_man_t	bm;
	proc_data_t*		data	= (proc_data_t*) arg;
	int					errs;

	for (;;)
	{
		data->bytes		= 0;
		data->ms_begin	= gsc_os_time_delta_ms();

		if (data->operation == RX_OP_LOAD)
		{
			_file	= fopen(data->file_name, "rb");

			if (_file == NULL)
			{
				data->err	= TRUE;
				strcpy(data->text, "fopen() failed");
				break;
			}
		}
		else
		{
			_file	= NULL;
		}

		errs	= process_thread_sync(data);

		if (errs)
		{
			data->err	= TRUE;
			sprintf(data->text, "process_thread_sync() returned %d", errs);
			break;
		}

		for (;;)
		{
			if (data->stop)
			{
				strcpy(data->text, "stop requested");
				break;
			}

			if (data->done)
				break;

			errs	= gsc_buf_man_request_empty(&bm);

			if (errs)
			{
				data->err	= TRUE;
				strcpy(data->text, "gsc_buf_man_request_empty() failed");
				break;
			}

			if (bm.eof)
			{
				strcpy(data->text, "end of file");
				break;
			}

			if (bm.buffer == NULL)
			{
				strcpy(data->text, "stop requested");
				break;
			}

			_acquire_data(data, &bm);

			if (data->err)
			{
				gsc_buf_man_release_buffer(&bm);
				break;
			}

			errs	= gsc_buf_man_release_buffer(&bm);

			if (errs)
			{
				data->err	= TRUE;
				strcpy(data->text, "gsc_buf_man_release_buffer() failed");
				break;
			}
		}

		break;
	}

	if (_file)
		fclose(_file);

	data->ms_end	= gsc_os_time_delta_ms();
	data->done		= TRUE;
	return(0);
}



/*****************************************************************************
*
*	Function:	rx_process_start
*
*	Purpose:
*
*		Start the process/thread that acquires data.
*
*	Arguments:
*
*		data	Data for beginning and ending our processing.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

int rx_process_start(proc_data_t* data)
{
	int				errs;
	gsc_os_thread_t	thread;

	gsc_label_text(data->name);

	if (data == NULL)
	{
		errs	= gsc_label_fail_text(1, "rx_process_start(): NULL argument");
	}
	else if (data->dev == NULL)
	{
		errs	= gsc_label_fail_text(1, "rx_process_start(): device handle is NULL");
	}
	else
	{
		switch (data->operation)
		{
			default:

				data->operation	= RX_OP_INCREMENT;

			case RX_OP_INCREMENT:

				strcpy(data->op_title, "Incrementing 32-bit pattern.");
				break;

			case RX_OP_LOAD:

				sprintf(data->op_title, "Loading data from disk: %s", data->file_name);
				break;

			case RX_OP_RECEIVE:

				strcpy(data->op_title, "Receive data from device.");
				break;

		}

		printf("%s\n", data->op_title);
		gsc_label_level_inc();
		gsc_label_text("Starting");
		errs	= gsc_os_thread_create(&thread, _rx_process, data);

		if (errs)
			gsc_label_fail_text(1, "gsc_os_thread_create() failed");
		else
			printf("PASS\n");

		gsc_label_level_dec();
	}

	return(errs);
}


