// HPDI32 - Application - Any OS - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/sbtest/pio.c $
// $Rev: 3630 $
// $Date: 2009-10-12 10:21:12 -0500 (Mon, 12 Oct 2009) $

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "hpdi32.h"
#include "main.h"
#include "gsc_util.h"



/*****************************************************************************
*
*	Function:	_pio_throughput_write_test
*
*	Purpose:
*
*		Perform a single threaded PIO based throughput test using the data
*		buffer provided.
*
*	Arguments:
*
*		handle		The handle of the board of interest.
*
*		seconds		The number of seconds over which to run the test.
*
*		rate		The resulting rate is recorded here in bytes per second
*					assuming 32-bit samples
*
*		buffer		The data buffer to use.
*
*		size		The size of the above data buffer in bytes.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _pio_throughput_write_test(	void*	handle,
										U32		seconds,
										U32*	rate,
										void*	buffer,
										size_t	size)
{
	unsigned long	begin;
	U32				bytes;
	unsigned long	end;
	unsigned long	feature;
	long			l;
	long			ms;
	long			sec_ms	= 1000L * (long) seconds;
	U32				sent;
	U32				status;
	U32				tfsr	= 0;

	/*
	*	Fill the Tx FIFO with data before we begin. We do this so that the
	*	transfer rate isn't artifically high due an initially empty FIFO.
	*/

	status	= HPDI32_MISC_FEATURES__FIFO_SIZE(handle, &feature);

	if (status == GSC_SUCCESS)
		status	= hpdi32_reg_read(handle, HPDI32_TFSR, &tfsr);

	if ((status == GSC_SUCCESS) && (tfsr))
	{
		// The best PIO performance is generally achieved with the Almost
		// Full or Alomst Empty level set to 1/2 the FIFO size.
		status	= HPDI32_FIFO_ALMOST_LEVEL__TX_AF_SET(handle, tfsr / 2);
	}

	if (feature == HPDI32_MISC_FEATURES_ABSENT)
		tfsr	= 128L * 1024;

	if (status == GSC_SUCCESS)
	{
		printf(".");
		fflush(stdout);
	}

	for (l = 0; (status == GSC_SUCCESS) && (l < (long) tfsr); l++)
		status	= hpdi32_reg_write(handle, HPDI32_FDR, 0);

	if (status == GSC_SUCCESS)
	{
		printf("\b \b");
		fflush(stdout);
		status	= HPDI32_TX_OVERRUN__CLEAR(handle);
	}

	/*
	*	Begin.
	*/

	begin	= gsc_os_time_delta_ms();

	for (bytes = 0; status == GSC_SUCCESS;)
	{
		status	= hpdi32_write(handle, buffer, (U32) size, &sent);

		if (status != GSC_SUCCESS)
		{
			printf(	"(hpdi32_write(handle, buf, %lu, %lu) error = 0x%lX) ",
					(long) size,
					(long) sent,
					(long) status);
		}

		bytes	+= sent;
		end		= gsc_os_time_delta_ms();
		ms		= (long) (end - begin);

		if (ms >= sec_ms)
		{
			rate[0]	= (U32) (1000.0 * bytes / ms);
			break;
		}
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_pio_throughput_read_test
*
*	Purpose:
*
*		Perform a single threaded PIO based read throughput test using the
*		data buffer provided.
*
*	Arguments:
*
*		handle		The handle of the board of interest.
*
*		seconds		The number of seconds over which to run the test.
*
*		rate		The resulting rate is recorded here in bytes per second
*					assuming 32-bit samples
*
*		buffer		The data buffer to use.
*
*		size		The size of the above data buffer in bytes.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _pio_throughput_read_test(	void*	handle,
										U32		seconds,
										U32*	rate,
										void*	buffer,
										size_t	size)
{
	unsigned long	begin;
	U32				bytes;
	unsigned long	end;
	long			ms;
	long			sec_ms	= 1000L * (long) seconds;
	U32				status;
	U32				xfer	= GSC_SUCCESS;

	begin	= gsc_os_time_delta_ms();

	for (bytes = 0;;)
	{
		status	= hpdi32_read(handle, buffer, (U32) size, &xfer);

		if (status != GSC_SUCCESS)
		{
			printf(	"(hpdi32_read(handle, buf, %lu, %lu) error = 0x%lX) ",
					(long) size,
					(long) xfer,
					(long) status);
			break;
		}

		bytes	+= xfer;
		end		= gsc_os_time_delta_ms();
		ms		= (long) (end - begin);

		if (ms >= sec_ms)
		{
			rate[0]	= (U32) (1000.0 * bytes / ms);
			break;
		}
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	pio_buf_api_throughput_write_test
*
*	Purpose:
*
*		Perform a single threaded PIO based throughput test where the data
*		buffer is supplied by the API.
*
*	Arguments:
*
*		handle		The handle of the board of interest.
*
*		seconds		The number of seconds over which to run the test.
*
*		size		The size of the buffer used, in bytes.
*
*		rate		The resulting rate is recorded here in bytes per second
*					assuming 32-bit samples
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 pio_buf_api_throughput_write_test(
	void*			handle,
	U32				seconds,
	unsigned long*	size,
	U32*			rate)
{
	U32		status;
	void*	vp;

	size[0]	= 1L * 1024 * 1024 * 4;
	status	= HPDI32_IO_BUFFER_SIZE__TX_SET(handle, size[0], size);
	size[0]	-= size[0] % 4;

	if ((status == GSC_SUCCESS) && (size[0] == 0))
		status	= GSC_FAILED;

	if (status == GSC_SUCCESS)
		status	= HPDI32_IO_BUFFER_POINTER__TX_GET(handle, (void*) &vp);

	if ((status == GSC_SUCCESS) && (vp == NULL))
		status	= GSC_FAILED;

	if (status == GSC_SUCCESS)
		status	= _pio_throughput_write_test(handle, seconds, rate, vp, size[0]);

	HPDI32_IO_BUFFER_SIZE__TX_FREE(handle);
	return(status);
}



/*****************************************************************************
*
*	Function:	pio_buf_api_throughput_read_test
*
*	Purpose:
*
*		Perform a single threaded PIO based read throughput test where the
*		data buffer is supplied by the API.
*
*	Arguments:
*
*		handle		The handle of the board of interest.
*
*		seconds		The number of seconds over which to run the test.
*
*		size		The size of the buffer used, in bytes.
*
*		rate		The resulting rate is recorded here in bytes per second
*					assuming 32-bit samples
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 pio_buf_api_throughput_read_test(
	void*			handle,
	U32				seconds,
	unsigned long*	size,
	U32*			rate)
{
	U32		status;
	void*	vp;

	size[0]	= 1L * 1024 * 1024 * 4;
	status	= HPDI32_IO_BUFFER_SIZE__RX_SET(handle, size[0], size);
	size[0]	-= size[0] % 4;

	if ((status == GSC_SUCCESS) && (size[0] == 0))
		status	= GSC_FAILED;

	if (status == GSC_SUCCESS)
		status	= HPDI32_IO_BUFFER_POINTER__RX_GET(handle, (void*) &vp);

	if ((status == GSC_SUCCESS) && (vp == NULL))
		status	= GSC_FAILED;

	if (status == GSC_SUCCESS)
		status	= _pio_throughput_read_test(handle, seconds, rate, vp, size[0]);

	HPDI32_IO_BUFFER_SIZE__RX_FREE(handle);
	return(status);
}



/*****************************************************************************
*
*	Function:	pio_buf_user_throughput_write_test
*
*	Purpose:
*
*		Perform a single threaded PIO based throughput test where the data
*		buffer is in application/user space.
*
*	Arguments:
*
*		handle		The handle of the board of interest.
*
*		seconds		The number of seconds over which to run the test.
*
*		size		The size of the buffer used, in bytes.
*
*		rate		The resulting rate is recorded here in bytes per second
*					assuming 32-bit samples
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 pio_buf_user_throughput_write_test(
	void*			handle,
	U32				seconds,
	unsigned long*	size,
	U32*			rate)
{
	void*	buf;
	U32		status;

	size[0]	= 1L * 1024 * 1024 * 4;
	buf		= malloc(size[0]);

	if (buf)
	{
		status	= _pio_throughput_write_test(handle, seconds, rate, buf, size[0]);
		free(buf);
	}
	else
	{
		status	= GSC_FAILED;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	pio_buf_user_throughput_read_test
*
*	Purpose:
*
*		Perform a single threaded PIO based read throughput test where the
*		data buffer is in application/user space.
*
*	Arguments:
*
*		handle		The handle of the board of interest.
*
*		seconds		The number of seconds over which to run the test.
*
*		size		The size of the buffer used, in bytes.
*
*		rate		The resulting rate is recorded here in bytes per second
*					assuming 32-bit samples
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 pio_buf_user_throughput_read_test(
	void*			handle,
	U32				seconds,
	unsigned long*	size,
	U32*			rate)
{
	void*	buf;
	U32		status;

	size[0]	= 1L * 1024 * 1024 * 4;
	buf		= malloc(size[0]);

	if (buf)
	{
		status	= _pio_throughput_read_test(handle, seconds, rate, buf, size[0]);
		free(buf);
	}
	else
	{
		status	= GSC_FAILED;
	}

	return(status);
}


