// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/button.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement support for custom button control.
*
*****************************************************************************/

#include <windows.h>

#include "button.h"
#include "gui.h"



/*****************************************************************************
*
*	Function:	_arrow_paint
*
*	Purpose:
*
*		Paint an arrowhead per the given structure.
*
*	Arguments:
*
*		dc		The device context to use.
*
*		data	The polyline data to use.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _arrow_paint(HDC dc, const button_arrow_t* data)
{
	HBRUSH	brush;
	HPEN	pen;

	if (data->enable)
	{
		brush	= SelectObject(dc, data->brush);
		pen		= SelectObject(dc, data->pen);

		Polygon(dc, data->list, data->count);

		SelectObject(dc, brush);
		SelectObject(dc, pen);
	}
}



/*****************************************************************************
*
*	Function:	_polyline_paint
*
*	Purpose:
*
*		Paint a polyline per the given structure.
*
*	Arguments:
*
*		dc		The device context to use.
*
*		data	The polyline data to use.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _polyline_paint(HDC dc, const button_polyline_t* data)
{
	HPEN	pen;

	if (data->enable)
	{
		pen	= SelectObject(dc, data->pen);

		Polyline(dc, data->list, data->count);

		SelectObject(dc, pen);
	}
}



/*****************************************************************************
*
*	Function:	_rect_paint
*
*	Purpose:
*
*		Paint a rectangle per the given structure.
*
*	Arguments:
*
*		dc		The device context to use.
*
*		data	The rectangle data to use.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _rect_paint(HDC dc, const button_rect_t* data)
{
	HBRUSH	brush;
	HPEN	pen;

	if (data->enable)
	{
		brush	= SelectObject(dc, data->brush);
		pen		= SelectObject(dc, data->pen);

		Rectangle(	dc,
					data->rect.left,
					data->rect.top,
					data->rect.right,
					data->rect.bottom);

		SelectObject(dc, brush);
		SelectObject(dc, pen);
	}
}



/*****************************************************************************
*
*	Function:	_text_align
*
*	Purpose:
*
*		Compute the text alignment for one line of text.
*
*	Arguments:
*
*		data		The control specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _text_align(button_t* data)
{
	HFONT	font;
	RECT	rect;
	int		width;

	font	= SelectObject(data->dc, data->text1.font);
	memset(&rect, 0, sizeof(rect));
	DrawText(data->dc, data->text1.text, -1, &rect, DT_CALCRECT);
	width	= RECT_WIDTH(rect);
	SelectObject(data->dc, font);

	switch (data->style & BS_CENTER)
	{
		default:
		case BS_CENTER:

			data->text2.x	= (RECT_WIDTH(data->client) - width) / 2;
			data->text1.x	= data->text2.x + 1;
			break;

		case BS_LEFT:

			data->text2.x	= data->text2.rect.left;
			data->text1.x	= data->text2.x + 1;
			break;

		case BS_RIGHT:

			data->text2.x	= data->text2.rect.right - width;
			data->text1.x	= data->text2.x - 1;
			break;
	}

	switch (data->style & BS_VCENTER)
	{
		default:
		case BS_VCENTER:

			data->text2.y	= (RECT_HEIGHT(data->client) - RECT_HEIGHT(rect)) / 2;
			data->text1.y	= data->text2.y + 1;
			break;

		case BS_TOP:

			data->text2.y	= data->text2.rect.top;
			data->text1.y	= data->text2.y + 1;
			break;

		case BS_BOTTOM:

			data->text2.y	= data->text2.rect.bottom - 1 - RECT_HEIGHT(rect);
			data->text1.y	= data->text2.y - 1;
			break;
	}
}



/*****************************************************************************
*
*	Function:	_text_paint
*
*	Purpose:
*
*		Paint text per the given structure.
*
*	Arguments:
*
*		dc		The device context to use.
*
*		data	The text data to use.
*
*	Returned:
*
*		The width of the painted text in pixels.
*
*****************************************************************************/

static int _text_paint(HDC dc, const button_text_t* data)
{
	UINT		align;
	COLORREF	color_bg;
	COLORREF	color_fg;
	HFONT		font;
	int			mode;
	RECT		rect;
	int			width	= 0;

	if (data->length)
	{
		font		= SelectObject(dc, data->font);

		if (data->enable)
		{
			align		= SetTextAlign(dc, TA_LEFT | TA_TOP);
			color_bg	= SetBkColor(dc, data->color_bg);
			color_fg	= SetTextColor(dc, data->color_fg);
			mode		= SetBkMode(dc, TRANSPARENT);

			ExtTextOut(	dc,
						data->x,
						data->y,
						ETO_CLIPPED,
						&data->rect,
						data->text,
						data->length,
						NULL);

			SetTextAlign(dc, align);
			SetBkColor(dc, color_bg);
			SetTextColor(dc, color_fg);
			SetBkMode(dc, mode);
		}

		DrawText(dc, data->text, -1, &rect, DT_CALCRECT);
		width	= RECT_WIDTH(rect);

		SelectObject(dc, font);
	}

	return(width);
}



/*****************************************************************************
*
*	Function:	_wave_paint
*
*	Purpose:
*
*		Paint a portion of a waveform per the given data.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*		x		The starting X coordinate.
*
*		y		The starting Y coordinate.
*
*		wave	The info for the waveform portion we're to process.
*
*		dx		The unit width in pixels.
*
*		dy		The waveform height in pixels.
*
*	Returned:
*
*		The ending X coordinate.
*
*****************************************************************************/

static int _wave_paint(
	button_t*		data,
	int				x,
	int				y,
	const wave_t*	wave,
	int				dx,
	int				dy)
{
	int		count	= wave->count;
	int		y2		= y + dy;

	x	+= wave->skip * dx;
	MoveToEx(data->dc, x, y, NULL);

	for (; count--;)
	{
		if (wave->w1)
		{
			x	+= dx * wave->w1;
			LineTo(data->dc, x, y);
		}

		if (wave->w2)
		{
			LineTo(data->dc, x, y2);
			x	+= dx * wave->w2;
			LineTo(data->dc, x, y2);
			LineTo(data->dc, x, y);
		}

		if (count == 0)
			LineTo(data->dc, x, y);
	}

	return(x);
}



/*****************************************************************************
*
*	Function:	_control_paint
*
*	Purpose:
*
*		Paint the given button.
*
*	Arguments:
*
*		control		The control handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _control_paint(HWND control, button_t* data)
{
	GetWindowText(control, data->text1.text, sizeof(data->text1.text));
	data->text1.length	= (int) strlen(data->text1.text);
	strcpy(data->text2.text, data->text1.text);
	data->text2.length	= data->text1.length;
	_text_align(data);
	_polyline_paint(data->dc, &data->border.frame1);
	_polyline_paint(data->dc, &data->border.frame2);
	_polyline_paint(data->dc, &data->border.ul_out);
	_polyline_paint(data->dc, &data->border.ul_mid);
	_polyline_paint(data->dc, &data->border.ul_in);
	_polyline_paint(data->dc, &data->border.lr_out);
	_polyline_paint(data->dc, &data->border.lr_mid);
	_polyline_paint(data->dc, &data->border.lr_in);
	_rect_paint(data->dc, &data->inside1);
	_rect_paint(data->dc, &data->inside2);
	_text_paint(data->dc, &data->text1);
	_text_paint(data->dc, &data->text2);
	_arrow_paint(data->dc, &data->arrow1);
	_arrow_paint(data->dc, &data->arrow2);

	if (data->paint)
		(data->paint)(control, data);
}



/*****************************************************************************
*
*	Function:	button_init
*
*	Purpose:
*
*		Initialize the custom button control.
*
*	Arguments:
*
*		dlg			The parent dialog handle.
*
*		device		The device structure to access.
*
*		id			The control id.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void button_init(HWND dlg, device_t* device, int id)
{
	HWND		control;
	button_t*	data;
	TEXTMETRIC	tm;

	for (;;)	// A convenience loop.
	{
		control	= GetDlgItem(dlg, id);

		if (control == NULL)
			break;

		data	= (button_t*) malloc(sizeof(button_t));

		if (data == NULL)
			break;

		memset(data, 0, sizeof(button_t));
		SET_WINDOW_USER_DATA(control, data);
		data->dc	= GetDC(dlg);
		GetClientRect(control, &data->client);

		data->device	= device;
		data->style		= (DWORD) GetWindowLong(control, GWL_STYLE);

		GetTextMetrics(data->dc, &tm);
		data->tmAscent	= tm.tmAscent;
		data->tmDescent	= tm.tmDescent;

		// First outter border.
		data->border.frame1.count		= 5;
		data->border.frame1.enable		= TRUE;
		data->border.frame1.list[0].x	= data->client.left;
		data->border.frame1.list[0].y	= data->client.top;
		data->border.frame1.list[1].x	= data->client.right - 1;
		data->border.frame1.list[1].y	= data->client.top;
		data->border.frame1.list[2].x	= data->client.right - 1;
		data->border.frame1.list[2].y	= data->client.bottom - 1;
		data->border.frame1.list[3].x	= data->client.left;
		data->border.frame1.list[3].y	= data->client.bottom - 1;
		data->border.frame1.list[4].x	= data->client.left;
		data->border.frame1.list[4].y	= data->client.top;
		data->border.frame1.pen			= data->device->paint.black.pen;

		// Second outter border.
		data->border.frame2.count		= 5;
		data->border.frame2.enable		= TRUE;
		data->border.frame2.list[0].x	= data->client.left + 1;
		data->border.frame2.list[0].y	= data->client.top + 1;
		data->border.frame2.list[1].x	= data->client.right - 2;
		data->border.frame2.list[1].y	= data->client.top + 1;
		data->border.frame2.list[2].x	= data->client.right - 2;
		data->border.frame2.list[2].y	= data->client.bottom - 2;
		data->border.frame2.list[3].x	= data->client.left + 1;
		data->border.frame2.list[3].y	= data->client.bottom - 2;
		data->border.frame2.list[4].x	= data->client.left + 1;
		data->border.frame2.list[4].y	= data->client.top + 1;
		data->border.frame2.pen			= data->device->paint.black.pen;

		// First upper left edge.
		data->border.ul_out.count		= 3;
		data->border.ul_out.enable		= TRUE;
		data->border.ul_out.list[0].x	= data->client.left + 1;
		data->border.ul_out.list[0].y	= data->client.bottom - 3;
		data->border.ul_out.list[1].x	= data->client.left + 1;
		data->border.ul_out.list[1].y	= data->client.top + 1;
		data->border.ul_out.list[2].x	= data->client.right - 2;
		data->border.ul_out.list[2].y	= data->client.top + 1;
		data->border.ul_out.pen			= data->device->paint.white.pen;

		// Second upper left edge.
		data->border.ul_mid.count		= 3;
		data->border.ul_mid.enable		= TRUE;
		data->border.ul_mid.list[0].x	= data->client.left + 2;
		data->border.ul_mid.list[0].y	= data->client.bottom - 4;
		data->border.ul_mid.list[1].x	= data->client.left + 2;
		data->border.ul_mid.list[1].y	= data->client.top + 2;
		data->border.ul_mid.list[2].x	= data->client.right - 3;
		data->border.ul_mid.list[2].y	= data->client.top + 2;
		data->border.ul_mid.pen			= data->device->paint.white.pen;

		// Third upper left edge.
		data->border.ul_in.count		= 3;
		data->border.ul_in.enable		= TRUE;
		data->border.ul_in.list[0].x	= data->client.left + 3;
		data->border.ul_in.list[0].y	= data->client.bottom - 5;
		data->border.ul_in.list[1].x	= data->client.left + 3;
		data->border.ul_in.list[1].y	= data->client.top + 3;
		data->border.ul_in.list[2].x	= data->client.right - 4;
		data->border.ul_in.list[2].y	= data->client.top + 3;
		data->border.ul_in.pen			= data->device->paint.white.pen;

		// First lower right edge.
		data->border.lr_out.count		= 3;
		data->border.lr_out.enable		= TRUE;
		data->border.lr_out.list[0].x	= data->client.left + 1;
		data->border.lr_out.list[0].y	= data->client.bottom - 2;
		data->border.lr_out.list[1].x	= data->client.right - 2;
		data->border.lr_out.list[1].y	= data->client.bottom - 2;
		data->border.lr_out.list[2].x	= data->client.right - 2;
		data->border.lr_out.list[2].y	= data->client.top;
		data->border.lr_out.pen			= data->device->paint.white.pen;

		// Second lower right edge.
		data->border.lr_mid.count		= 3;
		data->border.lr_mid.enable		= TRUE;
		data->border.lr_mid.list[0].x	= data->client.left + 2;
		data->border.lr_mid.list[0].y	= data->client.bottom - 3;
		data->border.lr_mid.list[1].x	= data->client.right - 3;
		data->border.lr_mid.list[1].y	= data->client.bottom - 3;
		data->border.lr_mid.list[2].x	= data->client.right - 3;
		data->border.lr_mid.list[2].y	= data->client.top + 1;
		data->border.lr_mid.pen			= data->device->paint.white.pen;

		// Third lower right edge.
		data->border.lr_in.count		= 3;
		data->border.lr_in.enable		= TRUE;
		data->border.lr_in.list[0].x	= data->client.left + 3;
		data->border.lr_in.list[0].y	= data->client.bottom - 4;
		data->border.lr_in.list[1].x	= data->client.right - 4;
		data->border.lr_in.list[1].y	= data->client.bottom - 4;
		data->border.lr_in.list[2].x	= data->client.right - 4;
		data->border.lr_in.list[2].y	= data->client.top + 2;
		data->border.lr_in.pen			= data->device->paint.white.pen;

		// First inner rectangle.
		data->inside1.brush			= data->device->paint.btnface.brush;
		data->inside1.enable		= TRUE;
		data->inside1.pen			= data->device->paint.btnhighlt.pen;
		data->inside1.rect.left		= data->client.left + 3;
		data->inside1.rect.top		= data->client.top + 3;
		data->inside1.rect.right	= data->client.right - 3;
		data->inside1.rect.bottom	= data->client.bottom - 3;

		// Second inner rectangle.
		data->inside2.brush			= data->device->paint.btnface.brush;
		data->inside2.enable		= TRUE;
		data->inside2.pen			= data->device->paint.btntext_dot.pen;
		data->inside2.rect.left		= data->client.left + 4;
		data->inside2.rect.top		= data->client.top + 4;
		data->inside2.rect.right	= data->client.right - 4;
		data->inside2.rect.bottom	= data->client.bottom - 4;

		// The arrow heads.
		data->arrow1.brush			= data->device->paint.btnhighlt.brush;
		data->arrow1.count			= 5;
		data->arrow1.direction		= 0;
		data->arrow1.enable			= FALSE;
		data->arrow1.height			= (GetSystemMetrics(SM_CYSMICON) * 3) / 5;
		data->arrow1.offset			= 0;
		data->arrow1.pen			= data->device->paint.gray_lt.pen;
		data->arrow1.width			= (GetSystemMetrics(SM_CXSMICON) * 3) / 5;

		data->arrow2				= data->arrow1;

		// The text images.
		data->text1.color_bg	= device->paint.btnface.color;
		data->text1.color_fg	= device->paint.btnhighlt.color;
		data->text1.enable		= TRUE;
		data->text1.font		= device->font.defalt;
		data->text1.rect.left	= data->client.left + 5;
		data->text1.rect.top	= data->client.top + 5;
		data->text1.rect.right	= data->client.right - 5;
		data->text1.rect.bottom	= data->client.bottom - 5;
		GetWindowText(control, data->text1.text, sizeof(data->text1.text));
		data->text1.length		= (int) strlen(data->text1.text);

		data->text2				= data->text1;
		data->text2.color_fg	= device->paint.btntext.color;

		_text_align(data);
		ReleaseDC(dlg, data->dc);
		break;
	}
}



/*****************************************************************************
*
*	Function:	button_arrow_direction_set
*
*	Purpose:
*
*		Set the control's arrow direction.
*
*	Arguments:
*
*		dlg			The dialog containing the control to access.
*
*		id			The id of the control to access.
*
*		direction	The arrow's direction: - = left, 0 = none, + = right
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void button_arrow_direction_set(HWND dlg, int id, int direction)
{
	HWND		control	= GetDlgItem(dlg, id);
	button_t*	data	= (button_t*) GET_WINDOW_USER_DATA(control);
	int			dy		= data->arrow1.height / 2;
	int			i;
	int			x		= data->inside2.rect.left + data->arrow1.offset;
	int			y		= data->inside2.rect.top + (data->tmAscent / 2);

	data->arrow1.direction	= direction;

	if (direction > 0)
	{
		// The arrow points right.
		data->arrow1.enable		= TRUE;
		data->arrow1.list[0].x	= x;
		data->arrow1.list[0].y	= y;
		data->arrow1.list[1].x	= x;
		data->arrow1.list[1].y	= y - dy;
		data->arrow1.list[2].x	= x + data->arrow1.width;
		data->arrow1.list[2].y	= y;
		data->arrow1.list[3].x	= x;
		data->arrow1.list[3].y	= y + dy;
		data->arrow1.list[4].x	= x;
		data->arrow1.list[4].y	= y;
	}
	else if (direction < 0)
	{
		// The arrow points left.
		data->arrow1.enable		= TRUE;
		data->arrow1.list[0].x	= x + data->arrow1.width;
		data->arrow1.list[0].y	= y;
		data->arrow1.list[1].x	= x + data->arrow1.width;
		data->arrow1.list[1].y	= y - dy;
		data->arrow1.list[2].x	= x;
		data->arrow1.list[2].y	= y;
		data->arrow1.list[3].x	= x + data->arrow1.width;
		data->arrow1.list[3].y	= y + dy;
		data->arrow1.list[4].x	= x + data->arrow1.width;
		data->arrow1.list[4].y	= y;
	}
	else
	{
		// No arrow is utilized.
		data->arrow1.enable	= FALSE;
	}

	data->arrow2	= data->arrow1;

	for (i = 0; i < (int) ARRAY_ELEMENTS(data->arrow1.list); i++)
	{
		data->arrow1.list[i].x++;
		data->arrow1.list[i].y++;
	}
}



/*****************************************************************************
*
*	Function:	button_arrow_offset_set
*
*	Purpose:
*
*		Set the control's arrow direction.
*
*	Arguments:
*
*		dlg			The dialog containing the control to access.
*
*		id			The id of the control to access.
*
*		offset		The offset from the left most text area.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void button_arrow_offset_set(HWND dlg, int id, int offset)
{
	HWND		control	= GetDlgItem(dlg, id);
	button_t*	data	= (button_t*) GET_WINDOW_USER_DATA(control);

	data->arrow1.offset	= offset;
	button_arrow_direction_set(dlg, id, data->arrow1.direction);
}



/*****************************************************************************
*
*	Function:	button_circle_paint
*
*	Purpose:
*
*		Paint a circle given the button's current state.
*
*	Arguments:
*
*		control		The handle to the control.
*
*		data		The control specific data we track.
*
*		x			The X coordinate of the left edge.
*
*		y			The Y coordinate of the top edge.
*
*		brush		The brush to fill the circle with. Is is the defult used
*					if the device is opened.
*
*		cross		Draw a cross over the circle.
*
*	Returned:
*
*		The width of the output.
*
*****************************************************************************/

int button_circle_paint(
	HWND		control,
	button_t*	data,
	int			x,
	int			y,
	HBRUSH		brush,
	int			cross)
{
	HDC		dc	= GetDC(control);
	HPEN	pen;
	int		h	= ((data->arrow1.height / 2) * 2) + 1;
	int		w	= data->arrow1.width + 2;

	if (data->device->handle == NULL)
		brush	= data->device->paint.btnshadow.brush;

	if (data->arrow1.enable)
		pen	= SelectObject(dc, data->arrow1.pen);
	else
		pen	= SelectObject(dc, data->arrow2.pen);

	brush	= SelectObject(dc, brush);
	y		= (y + (data->tmAscent / 2)) - (h / 2);

	Ellipse(dc, x, y, x + w, y + h);

	if (cross)
	{
		MoveToEx(dc, x, y, NULL);
		LineTo(dc, x + w, y + h);
		MoveToEx(dc, x, y + h - 1, NULL);
		LineTo(dc, x + w, y - 1);
	}

	SelectObject(dc, brush);
	SelectObject(dc, pen);
	ReleaseDC(control, dc);
	return(w);
}



/*****************************************************************************
*
*	Function:	button_paint_function_set
*
*	Purpose:
*
*		Call the given function when the control needs painting. The function
*		is called after the common portion is painted.
*
*	Arguments:
*
*		dlg			The dialog containing the control to access.
*
*		id			The id of the control to access.
*
*		paint		The function to call.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void button_paint_function_set(
	HWND	dlg,
	int		id,
	void	(*paint)(HWND control, button_t* data))
{
	HWND		control	= GetDlgItem(dlg, id);
	button_t*	data	= (button_t*) GET_WINDOW_USER_DATA(control);

	data->paint	= paint;
}



/*****************************************************************************
*
*	Function:	button_text_paint
*
*	Purpose:
*
*		Paint text per the given structure.
*
*	Arguments:
*
*		dlg		The control to access.
*
*		data	The control specific datw e track.
*
*		x		The X coordinate.
*
*		y		The Y coordinate.
*
*		text	The text to paint.
*
*	Returned:
*
*		The width of the text painted.
*
*****************************************************************************/

int button_text_paint(HWND control, button_t* data, int x, int y, const char* text)
{
	HDC				dc		= GetDC(control);
	button_text_t	t;
	int				w;

	t	= data->text1;
	strncpy(t.text, text, sizeof(t.text));
	t.length	= (int) strlen(t.text);
	t.x			= x;
	t.y			= y;
	_text_paint(dc, &t);

	t	= data->text2;
	strncpy(t.text, text, sizeof(t.text));
	t.length	= (int) strlen(t.text);
	t.x			= x;
	t.y			= y;
	w			= _text_paint(dc, &t);

	ReleaseDC(control, dc);
	return(w);
}



/*****************************************************************************
*
*	Function:	button_text_width_get
*
*	Purpose:
*
*		Return the width, in pixels, of the control's text.
*
*	Arguments:
*
*		dlg		The dialog containing the control to access.
*
*		id		The id of the control to access.
*
*	Returned:
*
*		The computed width in pixels.
*
*****************************************************************************/

int button_text_width_get(HWND dlg, int id)
{
	HWND		control	= GetDlgItem(dlg, id);
	button_t*	data	= (button_t*) GET_WINDOW_USER_DATA(control);
	HDC			dc;
	HFONT		font_old;
	RECT		rect;
	int			width;

	dc			= GetDC(control);
	font_old	= SelectObject(dc, data->text1.font);
	memset(&rect, 0, sizeof(rect));
	DrawText(dc, data->text1.text, -1, &rect, DT_CALCRECT);
	width	= RECT_WIDTH(rect);
	SelectObject(dc, font_old);
	ReleaseDC(control, dc);
	return(width);
}



/*****************************************************************************
*
*	Function:	button_wave_paint
*
*	Purpose:
*
*		Paint the given waveform using the control's current state.
*
*	Arguments:
*
*		data		The control specific data we track.
*
*		x			The starting X coordinate.
*
*		y			The starting Y coordinate. This is the top edge, but we
*					modify this so the centerline aligns with that of the
*					arrow, if one is present.
*
*		wave		The data set to process.
*
*	Returned:
*
*		The width of the waveform.
*
*****************************************************************************/

int button_wave_paint(
	button_t*		data,
	int				x,
	int				y,
	const wave_t*	list)
{
	button_arrow_t*	arrow;
	int				dx;
	int				dy;
	HPEN			pen_old;
	int				width;
	int				x1;
	int				x2;

	if (data->arrow1.enable)
		arrow	= &data->arrow1;
	else
		arrow	= &data->arrow2;

	dx		= arrow->width / 3;
	dy		= (arrow->height / 2) * 2;
	y		= (y + (data->tmAscent / 2)) - (dy / 2);
	pen_old	= SelectObject(data->dc, arrow->pen);
	x1		= x;
	x2		= x;

	for (; list->count > 0; list++)
	{
		if (list->flags & WAVE_TOP)
			x2	= _wave_paint(data, x, y, list, dx, dy);

		if (list->flags & WAVE_BOTTOM)
			x2	= _wave_paint(data, x, y + dy, list, dx, -dy);

		x	= x2;
	}

	SelectObject(data->dc, pen_old);
	width	= x - x1;
	return(width);
}



/*****************************************************************************
*
*	Function:	button_wm_drawitem
*
*	Purpose:
*
*		Process a WM_DRAWITEM for one of the configured controls.
*
*	Arguments:
*
*		dis		The structure describing the action to take.
*
*	Returned:
*
*		TRUE	We processed the message.
*
*****************************************************************************/

LRESULT button_wm_drawitem(DRAWITEMSTRUCT* dis)
{
	button_t*	data;

	data	= (button_t*) GET_WINDOW_USER_DATA(dis->hwndItem);

	if (data)
	{
		if (dis->itemState & ODS_DISABLED)
		{
			// The item is disabled.
			data->border.frame2.enable	= FALSE;
			data->border.ul_out.enable	= TRUE;
			data->border.ul_out.pen		= data->device->paint.btnhighlt.pen;
			data->border.ul_mid.enable	= TRUE;
			data->border.ul_mid.pen		= data->device->paint.btnhighlt.pen;
			data->border.ul_in.enable	= FALSE;
			data->border.lr_out.enable	= TRUE;
			data->border.lr_out.pen		= data->device->paint.btnshadow.pen;
			data->border.lr_mid.enable	= TRUE;
			data->border.lr_mid.pen		= data->device->paint.btnshadow.pen;
			data->border.lr_in.enable	= FALSE;
			data->inside1.enable		= TRUE;
			data->inside1.pen			= data->device->paint.btnface.pen;
			data->inside2.enable		= FALSE;
			data->arrow1.enable			= FALSE;
			data->arrow2.enable			= TRUE;
			data->arrow2.brush			= data->device->paint.btnshadow.brush;
			data->arrow2.pen			= data->device->paint.btnshadow.pen;

			data->offset				= 0;
			data->text1.color_fg		= data->device->paint.btnhighlt.color;
			data->text1.enable			= TRUE;
			data->text2.color_fg		= data->device->paint.btnshadow.color;
			data->text2.enable			= TRUE;
		}
		else if ((dis->itemState & ODS_FOCUS) == 0)
		{
			// The item is enabled but does not have the focus.
			data->border.frame2.enable	= FALSE;
			data->border.ul_out.enable	= TRUE;
			data->border.ul_out.pen		= data->device->paint.btnhighlt.pen;
			data->border.ul_mid.enable	= TRUE;
			data->border.ul_mid.pen		= data->device->paint.btnhighlt.pen;
			data->border.ul_in.enable	= FALSE;
			data->border.lr_out.enable	= TRUE;
			data->border.lr_out.pen		= data->device->paint.btnshadow.pen;
			data->border.lr_mid.enable	= TRUE;
			data->border.lr_mid.pen		= data->device->paint.btnshadow.pen;
			data->border.lr_in.enable	= FALSE;
			data->inside1.enable		= TRUE;
			data->inside1.pen			= data->device->paint.btnface.pen;
			data->inside2.enable		= FALSE;
			data->arrow1.enable			= FALSE;
			data->arrow2.enable			= TRUE;
			data->arrow2.brush			= data->device->paint.btntext.brush;
			data->arrow2.pen			= data->device->paint.btntext.pen;

			data->offset				= 0;
			data->text1.enable			= FALSE;
			data->text2.enable			= TRUE;
			data->text2.color_fg		= data->device->paint.btntext.color;
		}
		else if (dis->itemState & ODS_SELECTED)
		{
			// The item is enabled, has the focus and is selected
			data->border.frame2.enable	= TRUE;
			data->border.ul_out.enable	= FALSE;
			data->border.ul_mid.enable	= TRUE;
			data->border.ul_mid.pen		= data->device->paint.btnshadow.pen;
			data->border.ul_in.enable	= TRUE;
			data->border.ul_in.pen		= data->device->paint.btnshadow.pen;
			data->border.lr_out.enable	= FALSE;
			data->border.lr_mid.enable	= TRUE;
			data->border.lr_mid.pen		= data->device->paint.btnhighlt.pen;
			data->border.lr_in.enable	= TRUE;
			data->border.lr_in.pen		= data->device->paint.btnhighlt.pen;
			data->inside1.enable		= FALSE;
			data->inside2.enable		= TRUE;
			data->inside2.pen			= data->device->paint.btntext_dot.pen;
			data->arrow1.enable			= TRUE;
			data->arrow1.brush			= data->device->paint.btntext.brush;
			data->arrow1.pen			= data->device->paint.btntext.pen;
			data->arrow2.enable			= FALSE;

			data->offset				= 1;
			data->text1.color_fg		= data->device->paint.btntext.color;
			data->text1.enable			= TRUE;
			data->text2.enable			= FALSE;
		}
		else
		{
			// The item is enabled and has the focus, but is not selected.
			data->border.frame2.enable	= TRUE;
			data->border.ul_out.enable	= FALSE;
			data->border.ul_mid.enable	= TRUE;
			data->border.ul_mid.pen		= data->device->paint.btnhighlt.pen;
			data->border.ul_in.enable	= TRUE;
			data->border.ul_in.pen		= data->device->paint.btnhighlt.pen;
			data->border.lr_out.enable	= FALSE;
			data->border.lr_mid.enable	= TRUE;
			data->border.lr_mid.pen		= data->device->paint.btnshadow.pen;
			data->border.lr_in.enable	= TRUE;
			data->border.lr_in.pen		= data->device->paint.btnshadow.pen;
			data->inside1.enable		= FALSE;
			data->inside2.enable		= TRUE;
			data->inside2.pen			= data->device->paint.btntext_dot.pen;
			data->arrow1.enable			= FALSE;
			data->arrow2.enable			= TRUE;
			data->arrow2.brush			= data->device->paint.btntext.brush;
			data->arrow2.pen			= data->device->paint.btntext.pen;

			data->offset				= 0;
			data->text1.enable			= FALSE;
			data->text2.enable			= TRUE;
			data->text2.color_fg		= data->device->paint.btntext.color;
		}

		data->dc	= dis->hDC;
		_control_paint(dis->hwndItem, data);
	}

	return(TRUE);
}


