// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/gpio_xfer.c $
// $Rev: 31050 $
// $Date: 2014-10-27 17:51:19 -0500 (Mon, 27 Oct 2014) $

/*****************************************************************************
*
*	Implement common support for the GPIO read/write window.
*
*****************************************************************************/

#include <windows.h>
#include <process.h>
#include <stdio.h>

#include "gpio.h"
#include "align.h"
#include "cable.h"
#include "gui.h"
#include "ini.h"
#include "resource.h"
#include "util_ctrl.h"



/* #defines	*****************************************************************/

#define	KEY_X_ENABLED		"x.enabled"
#define	KEY_X_INDEX			"x.index"
#define	KEY_X_SELECTED		"x.selected"
#define	KEY_Y_ENABLED		"y.enabled"
#define	KEY_Y_INDEX			"y.index"
#define	KEY_Y_SELECTED		"y.selected"

#define	VALUES_MIN			5

#define	INDEX_TO_PIXEL(i,d)	((((i) - (d)->offset) * (d)->scale) + (d)->ruler.rect.left)
#define	PIXEL_TO_INDEX(x,d)	((((float) ((x) - (d)->graph.rect.left + 1)) / (d)->scale) + (d)->offset)



/* prototypes	*************************************************************/

static void		_cursor_update(gpio_xfer_t* data);
static void		_data_scroll_pos_update(HWND dlg, gpio_xfer_t* data);
static void		_data_track_update(HWND dlg, gpio_xfer_t* data, int x);
static void		_set_window_status(HWND dlg, const char* msg);
static LRESULT	_wm_paint(HWND dlg, gpio_xfer_t* data, int invalidate);
static LRESULT	_wm_vscroll(HWND dlg, WPARAM wParam);
static void		_x_y_settings_load(gpio_xfer_t* data);
static void		_x_y_settings_save(gpio_xfer_t* data);



/* variables	*************************************************************/

static const struct
{
	int			interval;
	const char*	text;
} _intervals[]	=
{
	{ 1,	"1 ms"	},
	{ 2,	"2 ms"	},
	{ 3,	"3 ms"	},
	{ 4,	"4 ms"	},
	{ 5,	"5 ms"	},
	{ 10,	"10 ms"	},
	{ 15,	"15 ms"	},
	{ 20,	"20 ms"	},
	{ 25,	"25 ms"	}
};

static const struct
{
	int			ctrl_id;
	U32			which;
	const char*	fc;
} _command_list[]	=
{
	/* cntrl_id					which					fc				*/
	{ IDC_GPIO_XFER_COMMAND_1,	HPDI32_WHICH_GPIO_0_,	"Line Valid"	},
	{ IDC_GPIO_XFER_COMMAND_2,	HPDI32_WHICH_GPIO_1_,	"Status Valid"	},
	{ IDC_GPIO_XFER_COMMAND_3,	HPDI32_WHICH_GPIO_2_,	"Rx Ready"		},
	{ IDC_GPIO_XFER_COMMAND_4,	HPDI32_WHICH_GPIO_3_,	"Tx Ready"		},
	{ IDC_GPIO_XFER_COMMAND_5,	HPDI32_WHICH_GPIO_4_,	"Tx Enable"		},
	{ IDC_GPIO_XFER_COMMAND_6,	HPDI32_WHICH_GPIO_5_,	"Rx Enable"		},
	{ IDC_GPIO_XFER_COMMAND_0,	HPDI32_WHICH_GPIO_6_,	"Frame Valid"	}
};



/*****************************************************************************
*
*	Function:	_align_controls
*
*	Purpose:
*
*		Align the controls within the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The dialog specific data we track.
*
*		resize		Resize the window after moving the controls?
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _align_controls(HWND dlg, gpio_xfer_t* data, int resize)
{
	static	align_t list1[]	=
	{
		// The "Start" button.
		{// 0
		/* id		*/	IDC_GPIO_XFER_START,
		/* ref		*/	0,
		/* flags	*/	ALIGN_L | ALIGN_W | ALIGN_T | ALIGN_H,
		/* value	*/	-999	// see below
		},

		// The "Stop" button.
		{// 1
		/* id		*/	IDC_GPIO_XFER_STOP,
		/* ref		*/	IDC_GPIO_XFER_START,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{// 2
		/* id		*/	IDC_GPIO_XFER_STOP,
		/* ref		*/	IDC_GPIO_XFER_START,
		/* flags	*/	ALIGN_L_R | ALIGN_W,
		/* value	*/	-999	// see below
		},

		// The "File" button.
		{// 3
		/* id		*/	IDC_GPIO_XFER_FILE,
		/* ref		*/	IDC_GPIO_XFER_STOP,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{// 4
		/* id		*/	IDC_GPIO_XFER_FILE,
		/* ref		*/	IDC_GPIO_XFER_STOP,
		/* flags	*/	ALIGN_L_R | ALIGN_W,
		/* value	*/	-999	// see below
		},

		// The "Close" button.
		{// 5
		/* id		*/	IDCANCEL,
		/* ref		*/	IDC_GPIO_XFER_FILE,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{// 6
		/* id		*/	IDCANCEL,
		/* ref		*/	IDC_GPIO_XFER_FILE,
		/* flags	*/	ALIGN_L_R | ALIGN_W,
		/* value	*/	-999	// see below
		},

		// The "Interval" static text.
		{// 7
		/* id		*/	IDC_GPIO_XFER_INTERVAL_TEXT,
		/* ref		*/	IDCANCEL,
		/* flags	*/	ALIGN_L_R | ALIGN_W,
		/* value	*/	-999	// see below
		},

		// The "Interval" list box.
		{// 8
		/* id		*/	IDC_GPIO_XFER_INTERVAL_LIST,
		/* ref		*/	IDC_GPIO_XFER_INTERVAL_TEXT,
		/* flags	*/	ALIGN_L_R | ALIGN_W,
		/* value	*/	MARGIN
		},

		{ 0, 0, 0, 0 }
	};

	static	align_t list2[]	=
	{
		// The "Scale" static text.
		{// 0
		/* id		*/	IDC_GPIO_XFER_SCALE_TEXT,
		/* ref		*/	IDC_GPIO_XFER_INTERVAL_LIST,
		/* flags	*/	ALIGN_L_R | ALIGN_W,
		/* value	*/	-999	// see below
		},

		// The scale UP/DOWN scroll bar.
		{// 1
		/* id		*/	IDC_GPIO_XFER_SCALE_SCROLL,
		/* ref		*/	IDC_GPIO_XFER_SCALE_TEXT,
		/* flags	*/	ALIGN_L_R | ALIGN_W,
		/* value	*/	MARGIN
		},
		{// 2
		/* id		*/	IDC_GPIO_XFER_SCALE_SCROLL,
		/* ref		*/	IDC_GPIO_XFER_START,
		/* flags	*/	ALIGN_T_T | ALIGN_B_B,
		/* value	*/	0
		},

		// The ruler static text.
		{// 3
		/* id		*/	IDC_GPIO_XFER_RULER_TITLE,
		/* ref		*/	IDC_GPIO_XFER_START,
		/* flags	*/	ALIGN_L_L | ALIGN_R_R,
		/* value	*/	0
		},
		{// 4
		/* id		*/	IDC_GPIO_XFER_RULER_TITLE,
		/* ref		*/	0,
		/* flags	*/	ALIGN_L | ALIGN_W,
		/* value	*/	-999	// see below
		},
		{// 5
		/* id		*/	IDC_GPIO_XFER_RULER_TITLE,
		/* ref		*/	IDC_GPIO_XFER_START,
		/* flags	*/	ALIGN_T_B | ALIGN_W,
		/* value	*/	-999	// see below
		},

		// The ruler rectangle.
		{// 6
		/* id		*/	IDC_GPIO_XFER_RULER,
		/* ref		*/	IDC_GPIO_XFER_RULER_TITLE,
		/* flags	*/	ALIGN_T_T | ALIGN_B_B,
		/* value	*/	0
		},
		{// 7
		/* id		*/	IDC_GPIO_XFER_RULER,
		/* ref		*/	IDC_GPIO_XFER_RULER_TITLE,
		/* flags	*/	ALIGN_L_R,
		/* value	*/	MARGIN
		},
		{// 8
		/* id		*/	IDC_GPIO_XFER_RULER,
		/* ref		*/	0,
		/* flags	*/	ALIGN_R_R,
		/* value	*/	-999	// see below
		},

		// The "Cable Command 1: GPIO 0/Line Valid" button.
		{// 9
		/* id		*/	IDC_GPIO_XFER_COMMAND_1,
		/* ref		*/	IDC_GPIO_XFER_RULER_TITLE,
		/* flags	*/	ALIGN_L_L | ALIGN_R_R,
		/* value	*/	0
		},
		{// 10
		/* id		*/	IDC_GPIO_XFER_COMMAND_1,
		/* ref		*/	IDC_GPIO_XFER_RULER_TITLE,
		/* flags	*/	ALIGN_T_B | ALIGN_H,
		/* value	*/	MARGIN
		},

		// The "Cable Command 2: GPIO 1/Status Valid" button.
		{// 11
		/* id		*/	IDC_GPIO_XFER_COMMAND_2,
		/* ref		*/	IDC_GPIO_XFER_COMMAND_1,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{// 12
		/* id		*/	IDC_GPIO_XFER_COMMAND_2,
		/* ref		*/	IDC_GPIO_XFER_COMMAND_1,
		/* flags	*/	ALIGN_T_B | ALIGN_H,
		/* value	*/	MARGIN * 2
		},

		// The "Cable Command 3: GPIO 2/Rx Ready" button.
		{// 13
		/* id		*/	IDC_GPIO_XFER_COMMAND_3,
		/* ref		*/	IDC_GPIO_XFER_COMMAND_2,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{// 14
		/* id		*/	IDC_GPIO_XFER_COMMAND_3,
		/* ref		*/	IDC_GPIO_XFER_COMMAND_2,
		/* flags	*/	ALIGN_T_B | ALIGN_H,
		/* value	*/	MARGIN * 2
		},

		// The "Cable Command 4: GPIO 3/Tx Data Ready" button.
		{// 15
		/* id		*/	IDC_GPIO_XFER_COMMAND_4,
		/* ref		*/	IDC_GPIO_XFER_COMMAND_3,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{// 16
		/* id		*/	IDC_GPIO_XFER_COMMAND_4,
		/* ref		*/	IDC_GPIO_XFER_COMMAND_3,
		/* flags	*/	ALIGN_T_B | ALIGN_H,
		/* value	*/	MARGIN * 2
		},

		// The "Cable Command 5: GPIO 4/Tx Enabled" button.
		{// 17
		/* id		*/	IDC_GPIO_XFER_COMMAND_5,
		/* ref		*/	IDC_GPIO_XFER_COMMAND_4,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{// 18
		/* id		*/	IDC_GPIO_XFER_COMMAND_5,
		/* ref		*/	IDC_GPIO_XFER_COMMAND_4,
		/* flags	*/	ALIGN_T_B | ALIGN_H,
		/* value	*/	MARGIN * 2
		},

		// The "Cable Command 6: GPIO 5/Rx Enabled" button.
		{// 19
		/* id		*/	IDC_GPIO_XFER_COMMAND_6,
		/* ref		*/	IDC_GPIO_XFER_COMMAND_5,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{// 20
		/* id		*/	IDC_GPIO_XFER_COMMAND_6,
		/* ref		*/	IDC_GPIO_XFER_COMMAND_5,
		/* flags	*/	ALIGN_T_B | ALIGN_H,
		/* value	*/	MARGIN * 2
		},

		// The "Cable Command 0: GPIO 6/Frame Valid" button.
		{// 21
		/* id		*/	IDC_GPIO_XFER_COMMAND_0,
		/* ref		*/	IDC_GPIO_XFER_COMMAND_6,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{// 22
		/* id		*/	IDC_GPIO_XFER_COMMAND_0,
		/* ref		*/	IDC_GPIO_XFER_COMMAND_6,
		/* flags	*/	ALIGN_T_B | ALIGN_H,
		/* value	*/	MARGIN * 2
		},

		// The data rectangle.
		{// 23
		/* id		*/	IDC_GPIO_XFER_DATA,
		/* ref		*/	IDC_GPIO_XFER_RULER,
		/* flags	*/	ALIGN_L_L | ALIGN_R_R,
		/* value	*/	0
		},
		{// 24
		/* id		*/	IDC_GPIO_XFER_DATA,
		/* ref		*/	IDC_GPIO_XFER_RULER,
		/* flags	*/	ALIGN_T_B,
		/* value	*/	-1
		},
		{// 25
		/* id		*/	IDC_GPIO_XFER_DATA,
		/* ref		*/	IDC_GPIO_XFER_COMMAND_0,
		/* flags	*/	ALIGN_B_B,
		/* value	*/	MARGIN + 2
		},

		// The data scroll bar.
		{// 26
		/* id		*/	IDC_GPIO_XFER_POSITION,
		/* ref		*/	IDC_GPIO_XFER_DATA,
		/* flags	*/	ALIGN_L_L | ALIGN_R_R | ALIGN_T_B | ALIGN_H,
		/* value	*/	0
		},

		// The "X" check box.
		{// 27
		/* id		*/	IDC_GPIO_XFER_X_CHECK,
		/* ref		*/	IDC_GPIO_XFER_COMMAND_0,
		/* flags	*/	ALIGN_L_L | ALIGN_W,
		/* value	*/	0
		},
		{// 28
		/* id		*/	IDC_GPIO_XFER_X_CHECK,
		/* ref		*/	IDC_GPIO_XFER_POSITION,
		/* flags	*/	ALIGN_T_T | ALIGN_B_B,
		/* value	*/	0
		},

		// The "X" button.
		{// 29
		/* id		*/	IDC_GPIO_XFER_X_BUTTON,
		/* ref		*/	IDC_GPIO_XFER_X_CHECK,
		/* flags	*/	ALIGN_L_R | ALIGN_W | ALIGN_T_T | ALIGN_H,
		/* value	*/	0
		},
		{// 30
		/* id		*/	IDC_GPIO_XFER_X_BUTTON,
		/* ref		*/	IDC_GPIO_XFER_X_CHECK,
		/* flags	*/	ALIGN_T_T | ALIGN_B_B,
		/* value	*/	0
		},

		// The "Y" button.
		{// 31
		/* id		*/	IDC_GPIO_XFER_Y_BUTTON,
		/* ref		*/	IDC_GPIO_XFER_X_BUTTON,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{// 32
		/* id		*/	IDC_GPIO_XFER_Y_BUTTON,
		/* ref		*/	IDC_GPIO_XFER_COMMAND_0,
		/* flags	*/	ALIGN_R_R | ALIGN_W,
		/* value	*/	0
		},

		// The "Y" check box.
		{// 33
		/* id		*/	IDC_GPIO_XFER_Y_CHECK,
		/* ref		*/	IDC_GPIO_XFER_X_CHECK,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{// 34
		/* id		*/	IDC_GPIO_XFER_Y_CHECK,
		/* ref		*/	IDC_GPIO_XFER_Y_BUTTON,
		/* flags	*/	ALIGN_R_L | ALIGN_W,
		/* value	*/	0
		},

		// The status text.
		{// 35
		/* id		*/	IDC_GPIO_XFER_STATUS,
		/* ref		*/	0,
		/* flags	*/	ALIGN_L,
		/* value	*/	-999	// see below
		},
		{// 36
		/* id		*/	IDC_GPIO_XFER_STATUS,
		/* ref		*/	IDC_GPIO_XFER_POSITION,
		/* flags	*/	ALIGN_T_B | ALIGN_H,
		/* value	*/	MARGIN
		},
		{// 37
		/* id		*/	IDC_GPIO_XFER_STATUS,
		/* ref		*/	0,
		/* flags	*/	ALIGN_R_R,
		/* value	*/	-999	// see below
		},

		// The status frame.
		{// 38
		/* id		*/	IDC_GPIO_XFER_FRAME,
		/* ref		*/	0,
		/* flags	*/	ALIGN_L_L,
		/* value	*/	-1
		},
		{// 39
		/* id		*/	IDC_GPIO_XFER_FRAME,
		/* ref		*/	0,
		/* flags	*/	ALIGN_R_R,
		/* value	*/	1
		},
		{// 40
		/* id		*/	IDC_GPIO_XFER_FRAME,
		/* ref		*/	IDC_GPIO_XFER_POSITION,
		/* flags	*/	ALIGN_T_B,
		/* value	*/	1
		},
		{// 41
		/* id		*/	IDC_GPIO_XFER_FRAME,
		/* ref		*/	IDC_GPIO_XFER_STATUS,
		/* flags	*/	ALIGN_B_B,
		/* value	*/	MARGIN + 1
		},

		{ 0, 0, 0, 0 }
	};

	int		bottom;
	HWND	control;
	int		h;
	RECT	rect;

	list1[0].value	= data->device->margin;
	list1[2].value	= data->device->margin;
	list1[4].value	= data->device->margin;
	list1[6].value	= data->device->margin;
	list1[7].value	= data->device->margin;

	list2[0].value	= data->device->margin;
	list2[4].value	= data->device->margin;
	list2[5].value	= data->device->margin;
	list2[8].value	= - data->device->margin;
	list2[35].value	= data->device->margin;
	list2[37].value	= - data->device->margin;

	if (resize)
		align_control_list(dlg, list1);

	align_control_list(dlg, list2);

	if (resize)
	{
		control	= GetDlgItem(dlg, IDC_GPIO_XFER_FRAME);
		GetWindowRect(control, &rect);
		bottom	= rect.bottom;
		GetWindowRect(dlg, &rect);
		h	= GetSystemMetrics(SM_CYSIZEFRAME);
		rect.bottom	= bottom + h - 1;
		MoveWindow(dlg, rect.left, rect.top, RECT_WIDTH(rect), RECT_HEIGHT(rect), TRUE);
	}
}



/*****************************************************************************
*
*	Function:	_buttons_update
*
*	Purpose:
*
*		Update the cable command button text.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The dialog specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static U32 _buttons_update(HWND dlg, gpio_xfer_t* data)
{
	char	buf[32];
	int		i;
	U32		mode;
	U32		stat;
	U32		status	= GSC_SUCCESS;

	for (i = 0; i < (int) ARRAY_ELEMENTS(_command_list); i++)
	{
		if (data->device->handle)
		{
			stat	= HPDI32_CABLE_COMMAND_MODE__GET(	data->device->handle,
														_command_list[i].which,
														&mode);
			status	= (status == GSC_SUCCESS) ? stat : status;
		}
		else
		{
			mode	= HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL;
		}

		switch (mode)
		{
			default:
			case HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL:

				strcpy(buf, _command_list[i].fc);
				break;

			case HPDI32_CABLE_COMMAND_MODE_GPIO_IN:

				sprintf(buf, "GPIO %d in", i);
				break;

			case HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_LOW:
			case HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_HI:

				sprintf(buf, "GPIO %d out", i);
				break;
		}

		ctrl_menu_set_text(dlg, _command_list[i].ctrl_id, buf);
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_compute_scale
*
*	Purpose:
*
*		Compute the scaling factor needed to display the given number of data
*		values.
*
*	Arguments:
*
*		data	The structure where the data needed is recorded.
*
*		values	The number of values to display.
*
*	Returned:
*
*		The computed scaling factor.
*
*****************************************************************************/

static float _compute_scale(gpio_xfer_t* data, int values)
{
	float	frac;
	float	scale;

	frac	= ((float) 1) / data->graph.width;
	scale	= ((float) data->graph.width) / (frac + values);
	return(scale);
}



/*****************************************************************************
*
*	Function:	_cursor_begin
*
*	Purpose:
*
*		Begin use of the cursor.
*
*	Arguments:
*
*		dlg		The dialog we're working with.
*
*		data	The structure where the data needed is recorded.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _cursor_begin(HWND dlg, gpio_xfer_t* data)
{
	data->offset				= 0;
	data->cursor.x_right		= data->graph.rect.left - 1;
	data->cursor.x				= data->graph.rect.left - 1;
	data->cursor.values_last	= 0;
}



/*****************************************************************************
*
*	Function:	_cursor_end
*
*	Purpose:
*
*		End cursor positioning while data is being transferred.
*
*	Arguments:
*
*		data		The structure where the data needed is recorded.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _cursor_end(gpio_xfer_t* data)
{
	HDC		dc;
	int		i;
	HPEN	pen_old;

	_cursor_update(data);

	if (data->cursor.x < data->graph.rect.right)
	{
		dc		= GetDC(data->graph.control);

		// Remove the cursor.
		pen_old	= SelectObject(dc, data->device->paint.window.pen);
		MoveToEx(dc, data->cursor.x, data->graph.rect.top, NULL);
		LineTo(dc, data->cursor.x, data->graph.rect.bottom);

		// Restore the guide lines.
		SelectObject(dc, data->device->paint.gray_lt.pen);

		for (i = 0; i < (int) ARRAY_ELEMENTS(data->signals); i++)
		{
			MoveToEx(dc, data->cursor.x, data->signals[i].y_top, NULL);
			LineTo(dc, data->cursor.x + 1, data->signals[i].y_top);
			MoveToEx(dc, data->cursor.x, data->signals[i].y_bot, NULL);
			LineTo(dc, data->cursor.x + 1, data->signals[i].y_bot);
		}

		SelectObject(dc, pen_old);
		ReleaseDC(data->graph.control, dc);
	}
}



/*****************************************************************************
*
*	Function:	_cursor_update
*
*	Purpose:
*
*		Update the cursor position while data is being transferred.
*
*	Arguments:
*
*		data		The structure where the data needed is recorded.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _cursor_update(gpio_xfer_t* data)
{
	HDC		dc;
	int		i;
	int		j;
	U8		mask;
	HPEN	pen_old;
	U8		v_last;
	U8		value;
	int		values;
	int		x_begin;	// Beginning of cursor update area.
	int		x_end;		// End of cursor update area.
	int		x_right;	// Right side of current data item.

	for (;;)
	{
		if (data->cursor.x_right > data->graph.rect.right)
			break;

		values	= data->values;
		x_begin	= (int) INDEX_TO_PIXEL(data->cursor.values_last, data);
		x_end	= (int) INDEX_TO_PIXEL(values, data);

		if (x_end > (data->graph.rect.right - 1))
			x_end	= data->graph.rect.right - 1;

		if (data->cursor.values_last >= data->capacity)
		{
			break;
		}
		else if (x_end == (data->graph.rect.right - 1))
		{
			if (values < data->capacity)
				break;
		}
		else if (x_end <= x_begin)
		{
			break;
		}

		data->cursor.x	= x_end + 1;
		dc	= GetDC(data->graph.control);

		// Clear the area just traversed.
		pen_old	= SelectObject(dc, data->device->paint.window.pen);

		if (x_begin <= data->cursor.x_right)
			i	= data->cursor.x_right + 1;
		else
			i	= x_begin;

		for (; i <= x_end; i++)
		{
			MoveToEx(dc, i, data->graph.rect.top, NULL);
			LineTo(dc, i, data->graph.rect.bottom);
		}

		// Restore the horizontal guide lines.
		SelectObject(dc, data->device->paint.gray_lt.pen);

		for (i = 0; i < (int) ARRAY_ELEMENTS(data->signals); i++)
		{
			MoveToEx(dc, x_begin, data->signals[i].y_top, NULL);
			LineTo(dc, x_end + 1, data->signals[i].y_top);
			MoveToEx(dc, x_begin, data->signals[i].y_bot, NULL);
			LineTo(dc, x_end + 1, data->signals[i].y_bot);
		}

		// Draw the data values.
		SelectObject(dc, data->device->paint.black.pen);

		for (i = data->cursor.values_last; i < values; i++)
		{
				x_right	= (int) INDEX_TO_PIXEL(i + 1, data);

				if (x_right > (data->graph.rect.right - 1))
					x_right	= data->graph.rect.right - 1;

				for (j = 0; j < (int) ARRAY_ELEMENTS(data->signals); j++)
				{
					mask	= 1 << j;
					value	= data->buffer[i] & mask;

					if (i <= 0)
						v_last	= data->buffer[0] & mask;
					else
						v_last	= data->buffer[i - 1] & mask;

					if (v_last)
						MoveToEx(dc, x_begin, data->signals[j].y_top, NULL);
					else
						MoveToEx(dc, x_begin, data->signals[j].y_bot, NULL);

					if (value)
					{
						if (v_last == 0)
							LineTo(dc, x_begin, data->signals[j].y_top);

						LineTo(dc, x_right + 1, data->signals[j].y_top);
					}
					else
					{
						if (v_last)
							LineTo(dc, x_begin, data->signals[j].y_bot);

						LineTo(dc, x_right + 1, data->signals[j].y_bot);
					}
				}

				x_begin	= x_right;
		}

		// Dray the cursor.

		if (data->cursor.x < data->graph.rect.right)
		{
			MoveToEx(dc, data->cursor.x, data->graph.rect.top, NULL);
			LineTo(dc, data->cursor.x, data->graph.rect.bottom);
		}

		data->cursor.x_right		= x_end;
		data->cursor.values_last	= values;
		SelectObject(dc, pen_old);
		ReleaseDC(data->graph.control, dc);

		break;
	}
}



/*****************************************************************************
*
*	Function:	_data_offset_adjust
*
*	Purpose:
*
*		Compute an offset value per the given changess to the given offset.
*
*	Arguments:
*
*		data		The data we're to display.
*
*		wParam		The wParam from a WM_HSCROLL message, or another value
*					if called for other than a WM_HSCROLL message.
*
*		offset		The offset's starting value.
*
*	Returned:
*
*		int			The resulting offset.
*
*****************************************************************************/

static int _data_offset_adjust(gpio_xfer_t* data, WPARAM wParam, int offset)
{
	int	code	= LOWORD(wParam);
	int	line;
	int	page;

	page	= (int) (((data->graph.width + 1) / data->scale) + 0.5);
	line	= page / 20;

	if (line < 1)
		line	= 1;

	switch (code)
	{
		default:											break;
		case SB_BOTTOM:			offset	= data->capacity;	break;
		case SB_LINEDOWN:		offset	+= line;			break;
		case SB_LINEUP:			offset	-= line;			break;
		case SB_PAGEDOWN:		offset	+= page;			break;
		case SB_PAGEUP:			offset	-= page;			break;
		case SB_THUMBPOSITION:	offset	= HIWORD(wParam);	break;
		case SB_THUMBTRACK:		offset	= HIWORD(wParam);	break;
		case SB_TOP:			offset	= 0;				break;
	}

	if (offset >= (data->capacity - page))
		offset	= data->capacity - page;
	else if (offset < 0)
		offset	= 0;

	return(offset);
}



/*****************************************************************************
*
*	Function:	_data_scroll_pos_update
*
*	Purpose:
*
*		Update the position scroll bar.
*
*	Arguments:
*
*		dlg			The dialog in use.
*
*		data		The data for the work we're to do.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _data_scroll_pos_update(HWND dlg, gpio_xfer_t* data)
{
	HWND	control;
	int		page;

	// Update the horizontal scroll bar.
	page	= (int) (((data->graph.width + 1) / data->scale) + 0.5);

	if (page >= data->capacity)
	{
		data->graph.trackable	= FALSE;
		ctrl_menu_set(dlg, IDC_GPIO_XFER_POSITION, CTRL_DISABLE);
	}
	else
	{
		data->graph.trackable	= TRUE;
		control	= GetDlgItem(dlg, IDC_GPIO_XFER_POSITION);
		EnableWindow(control, TRUE);
		SetScrollRange(control, SB_CTL, 0, data->capacity - page, FALSE);
		SetScrollPos(control, SB_CTL, data->offset, TRUE);
	}
}



/*****************************************************************************
*
*	Function:	_data_track_begin
*
*	Purpose:
*
*		Initialize data to begin horizontal tracking (dragging).
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The data we're to display.
*
*		x			The new X coordinate (relative to the ruler control).
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _data_track_begin(HWND dlg, gpio_xfer_t* data, int x)
{
	data->ruler.x_min	= data->ruler.rect.left;
	data->ruler.x_max	= data->ruler.rect.right;
	data->ruler.index	= (int) PIXEL_TO_INDEX(x, data);
	_data_track_update(dlg, data, x);
}



/*****************************************************************************
*
*	Function:	_data_track_update
*
*	Purpose:
*
*		Respond to an updated horizontal tracking (dragging) position.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The data we're to display.
*
*		x			The new X coordinate (relative to the ruler control).
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _data_track_update(HWND dlg, gpio_xfer_t* data, int x)
{
	int		delta;
	int		offset;

	delta				= data->ruler.index;
	data->ruler.index	= (int) PIXEL_TO_INDEX(x, data);
	delta				-= data->ruler.index;

	if (delta)
	{
		offset	= _data_offset_adjust(data, (WPARAM) -1, data->offset + delta);

		if (offset != data->offset)
		{
			data->offset		= offset;
			data->ruler.index	= (int) PIXEL_TO_INDEX(x, data);
			_wm_paint(dlg, data, TRUE);
			_data_scroll_pos_update(dlg, data);
		}
	}
}



/*****************************************************************************
*
*	Function:	_graph_show_status
*
*	Purpose:
*
*		Display status information.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*		index		The current data index.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _graph_show_status(HWND dlg, gpio_xfer_t* data, int index)
{
	char	buf[128]	= "";
	long	l;
	long	x			= 0;
	long	y			= 0;
	char	z[64];

	if (index >= 0)
	{
		l	= (long) data->interval * index;
		sprintf(buf, "%ld ms", l);
	}

	if (data->x.enabled)
	{
		if (index >= 0)
			strcat(buf, ", ");

		x	= (long) data->interval * data->x.index;
		sprintf(z, "X = %ld ms", x);
		strcat(buf, z);
	}

	if (data->y.enabled)
	{
		if (index >= 0)
			strcat(buf, ", ");

		y	= (long) data->interval * data->y.index;
		sprintf(z, "Y = %ld ms", y);
		strcat(buf, z);
	}

	if ((data->x.enabled) && (data->y.enabled))
	{
		l	= y - x;
		sprintf(z, ", X->Y = %ld", l);
		strcat(buf, z);
	}

	ctrl_menu_set_text(dlg, IDC_GPIO_XFER_STATUS, buf);
}



/*****************************************************************************
*
*	Function:	_graph_wm_lbuttondown
*
*	Purpose:
*
*		Process a WM_LBUTTONDOWN message for the GRAPH area.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*		point		The coordinates of the mouse (dialog relative).
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _graph_wm_lbuttondown(HWND dlg, gpio_xfer_t* data, POINT point)
{
	HCURSOR	cursor;
	int		index;

	if (data->values == 0)
	{
	}
	else if (data->graph.inside)
	{
		if (data->x.selected)
			cursor	= data->device->cursor.cross_x;
		else if (data->y.selected)
			cursor	= data->device->cursor.cross_y;
		else
			cursor	= data->device->cursor.cross;

		SetCursor(cursor);
		SetFocus(dlg);
		ClientToScreen(dlg, &point);
		ScreenToClient(data->graph.control, &point);
		index	= (int) PIXEL_TO_INDEX(point.x, data);

		if (data->x.selected)
			data->x.index	= index;
		else if (data->y.selected)
			data->y.index	= index;

		_graph_show_status(dlg, data, index);

		if ((data->x.selected) || (data->y.selected))
			_wm_paint(dlg, data, TRUE);

		_x_y_settings_save(data);
	}
	else
	{
		data->graph.wait_til_up	= TRUE;
	}
}



/*****************************************************************************
*
*	Function:	_graph_wm_lbuttonup
*
*	Purpose:
*
*		Process a WM_LBUTTONUP message for the GRAPH area.
*
*	Arguments:
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _graph_wm_lbuttonup(gpio_xfer_t* data)
{
	HCURSOR	cursor;

	if (data->values == 0)
	{
	}
	else
	{
		data->graph.wait_til_up	= FALSE;

		if (data->graph.inside)
		{
			if (data->x.selected)
				cursor	= data->device->cursor.cross_x;
			else if (data->y.selected)
				cursor	= data->device->cursor.cross_y;
			else
				cursor	= data->device->cursor.cross;

			SetCursor(cursor);
		}
	}
}



/*****************************************************************************
*
*	Function:	_graph_wm_mousemove
*
*	Purpose:
*
*		Process a WM_MOUSEMOVE message for the GRAPH area.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*		point		The coordinates of the mouse (dialog relative).
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _graph_wm_mousemove(HWND dlg, gpio_xfer_t* data, POINT point)
{
	HCURSOR	cursor;
	int		index;

	if (data->values == 0)
	{
	}
	else if (data->ruler.captured)
	{
	}
	else if (data->graph.wait_til_up)
	{
	}
	else
	{
		ClientToScreen(dlg, &point);
		ScreenToClient(data->graph.control, &point);

		if ((point.x >= data->graph.rect.left) &&
			(point.x < data->graph.rect.right) &&
			(point.y > data->graph.rect.top) &&
			(point.y < data->graph.rect.bottom))
		{
			if (data->x.selected)
				cursor	= data->device->cursor.cross_x;
			else if (data->y.selected)
				cursor	= data->device->cursor.cross_y;
			else
				cursor	= data->device->cursor.cross;

			if (data->graph.inside)
				SetCursor(cursor);
			else
				data->graph.cursor_last	= SetCursor(cursor);

			data->graph.inside	= TRUE;
			index	= (int) PIXEL_TO_INDEX(point.x, data);
			_graph_show_status(dlg, data, index);
		}
		else
		{
			data->graph.inside	= FALSE;
		}
	}
}



/*****************************************************************************
*
*	Function:	_graph_wm_paint
*
*	Purpose:
*
*		Paint the data (usually in response to a WM_PAINT message).
*
*	Arguments:
*
*		data		The data we're to display.
*
*		invalidate	Invalidate the entire data area?
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _graph_wm_paint(gpio_xfer_t* data, int invalidate)
{
	HDC		dc;
	HBRUSH	brush_old;
	int		i;
	int		index;
	int		j;
	U8		mask;
	HPEN	pen_old;
	int		quit	= FALSE;
	U8		v_last;
	U8		value;
	int		x_left;
	int		x_right;

	dc	= GetDC(data->graph.control);

	if (invalidate)
	{
		InvalidateRect(data->graph.control, NULL, TRUE);
		UpdateWindow(data->graph.control);
	}

	// Paint the background.
	pen_old		= SelectObject(dc, data->device->paint.window.pen);
	brush_old	= SelectObject(dc, data->device->paint.window.brush);

	Rectangle(	dc,
				data->graph.rect.left,
				data->graph.rect.top,
				data->graph.rect.right,
				data->graph.rect.bottom);

	// Paint the X cursor.

	if (data->x.enabled)
	{
		SelectObject(dc, data->device->paint.x_cursor.pen);
		x_left	= (int) INDEX_TO_PIXEL(data->x.index, data);

		if ((x_left >= data->graph.rect.left) &&
			(x_left < data->graph.rect.right))
		{
			MoveToEx(dc, x_left, data->graph.rect.top, NULL);
			LineTo(dc, x_left, data->graph.rect.bottom);
		}
	}

	// Paint the Y cursor.

	if (data->y.enabled)
	{
		SelectObject(dc, data->device->paint.y_cursor.pen);
		x_left	= (int) INDEX_TO_PIXEL(data->y.index, data);

		if ((x_left >= data->graph.rect.left) &&
			(x_left < data->graph.rect.right))
		{
			MoveToEx(dc, x_left, data->graph.rect.top, NULL);
			LineTo(dc, x_left, data->graph.rect.bottom);
		}
	}

	// Paint the guide lines.
	SelectObject(dc, data->device->paint.gray_lt.pen);

	for (i = 0; i < (int) ARRAY_ELEMENTS(data->signals); i++)
	{
		MoveToEx(dc, data->graph.rect.left, data->signals[i].y_top, NULL);
		LineTo(dc, data->graph.rect.right, data->signals[i].y_top);
		MoveToEx(dc, data->graph.rect.left, data->signals[i].y_bot, NULL);
		LineTo(dc, data->graph.rect.right, data->signals[i].y_bot);
	}

	// Display the data.
	SelectObject(dc, data->device->paint.black.pen);
	x_left	= (int) INDEX_TO_PIXEL(data->offset, data);

	for (i = 0;; i++)
	{
		index	= i + data->offset;

		if (index >= data->values)
			break;

		x_right	= (int) INDEX_TO_PIXEL(index + 1, data);

		if (x_right >= data->graph.rect.right)
		{
			quit	= TRUE;
			x_right	= data->graph.rect.right - 1;
		}

		for (j = 0; j < (int) ARRAY_ELEMENTS(data->signals); j++)
		{
			mask	= 1 << j;
			value	= data->buffer[index] & mask;

			if (index <= data->offset)
				v_last	= data->buffer[data->offset] & mask;
			else
				v_last	= data->buffer[index - 1] & mask;

			if (v_last)
				MoveToEx(dc, x_left, data->signals[j].y_top, NULL);
			else
				MoveToEx(dc, x_left, data->signals[j].y_bot, NULL);

			if (value)
			{
				if (v_last == 0)
					LineTo(dc, x_left, data->signals[j].y_top);

				LineTo(dc, x_right + 1, data->signals[j].y_top);
			}
			else
			{
				if (v_last)
					LineTo(dc, x_left, data->signals[j].y_bot);

				LineTo(dc, x_right + 1, data->signals[j].y_bot);
			}
		}

		if (quit)
			break;

		x_left	= x_right;

		if (x_left >= data->graph.rect.right)
			break;
	}

	SelectObject(dc, brush_old);
	SelectObject(dc, pen_old);
	ReleaseDC(data->graph.control, dc);
}



/*****************************************************************************
*
*	Function:	_graph_wm_rbuttondown
*
*	Purpose:
*
*		Process a WM_RBUTTONDOWN message for the GRAPH area.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*		point		The coordinates of the mouse (dialog relative).
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _graph_wm_rbuttondown(HWND dlg, gpio_xfer_t* data, POINT point)
{
	HCURSOR	cursor;
	int		index;
	int		paint;

	if (data->values == 0)
	{
	}
	else if (data->graph.inside)
	{
		if (data->x.selected)
			cursor	= data->device->cursor.cross_x;
		else if (data->y.selected)
			cursor	= data->device->cursor.cross_y;
		else
			cursor	= data->device->cursor.cross;

		SetCursor(cursor);
		SetFocus(dlg);
		ClientToScreen(dlg, &point);
		ScreenToClient(data->graph.control, &point);
		index	= (int) PIXEL_TO_INDEX(point.x, data);

		if ((data->x.selected) && (data->y.enabled))
		{
			data->y.index	= index;
			paint			= TRUE;
		}
		else if ((data->y.selected) && (data->x.enabled))
		{
			data->x.index	= index;
			paint			= TRUE;
		}
		else
		{
			paint	= FALSE;
		}

		_graph_show_status(dlg, data, index);

		if (paint)
			_wm_paint(dlg, data, TRUE);

		_x_y_settings_save(data);
	}
}



/*****************************************************************************
*
*	Function:	_graph_wm_rbuttonup
*
*	Purpose:
*
*		Process a WM_RBUTTONUP message for the GRAPH area.
*
*	Arguments:
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _graph_wm_rbuttonup(gpio_xfer_t* data)
{
	HCURSOR	cursor;

	if (data->values == 0)
	{
	}
	else if (data->graph.inside)
	{
		if (data->x.selected)
			cursor	= data->device->cursor.cross_x;
		else if (data->y.selected)
			cursor	= data->device->cursor.cross_y;
		else
			cursor	= data->device->cursor.cross;

		SetCursor(cursor);
	}
}



/*****************************************************************************
*
*	Function:	_interval_get
*
*	Purpose:
*
*		Retrieve the interval setting in ms.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The interval in ms is recorded here.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _interval_get(HWND dlg, gpio_xfer_t* data)
{
	HWND	control;
	LRESULT	i;

	control	= GetDlgItem(dlg, IDC_GPIO_XFER_INTERVAL_LIST);
	i	=	 SendMessage(control, CB_GETCURSEL, 0, 0);
	data->interval	= _intervals[i].interval;
	ini_int_set(NULL, data->section, "interval", (int) i);
}



/*****************************************************************************
*
*	Function:	_interval_init
*
*	Purpose:
*
*		Initialize the INTERVAL list box.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The dialog specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _interval_init(HWND dlg, gpio_xfer_t* data)
{
	HWND	control;
	int		i;

	control	= GetDlgItem(dlg, IDC_GPIO_XFER_INTERVAL_LIST);
	SendMessage(control, CB_RESETCONTENT, 0, 0);

	for (i = 0; i < (int) ARRAY_ELEMENTS(_intervals); i++)
		SendMessage(control, CB_ADDSTRING, 0, (LPARAM) _intervals[i].text);

	ini_int_get(NULL, data->section, "interval", &i, 4);

	if (i < 0)
		i	= 0;
	else if (i >= (int) ARRAY_ELEMENTS(_intervals))
		i	= ARRAY_ELEMENTS(_intervals) - 1;

	SendMessage(control, CB_SETCURSEL, (UINT) i, 0);
}



/*****************************************************************************
*
*	Function:	_refresh_window
*
*	Purpose:
*
*		Update the controls based on the current state.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		device		The device to access.
*
*		msg			A status message to apply.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _refresh_window(HWND dlg, device_t* device, const char* msg)
{
	BOOL			access;
	UINT			checked;
	HWND			control;
	gpio_xfer_t*	data		= (gpio_xfer_t*) GET_WINDOW_USER_DATA(dlg);
	BOOL			idle_data	= (data->values) && (data->active == FALSE);
	U32				status;

	set_win_title(dlg, device, FALSE, TRUE, data->title);

	if (device->handle)
	{
		access	= TRUE;
	}
	else
	{
		access	= FALSE;
		SendMessage(dlg, IDC_GPIO_XFER_STOP, 0, 0);
		_wm_paint(dlg, data, TRUE);
		_set_window_status(dlg, "");
	}

	control	= GetDlgItem(dlg, IDC_GPIO_XFER_STOP);
	EnableWindow(control, (BOOL) (data->active));
	control	= GetDlgItem(dlg, IDCANCEL);
	EnableWindow(control, (BOOL) (!data->active));

	control	= GetDlgItem(dlg, IDC_GPIO_XFER_RULER_TITLE);
	EnableWindow(control, (BOOL) ((data->values) || (data->active)));

	control	= GetDlgItem(dlg, IDC_GPIO_XFER_COMMAND_0);
	EnableWindow(control, access);
	control	= GetDlgItem(dlg, IDC_GPIO_XFER_COMMAND_1);
	EnableWindow(control, access);
	control	= GetDlgItem(dlg, IDC_GPIO_XFER_COMMAND_2);
	EnableWindow(control, access);
	control	= GetDlgItem(dlg, IDC_GPIO_XFER_COMMAND_3);
	EnableWindow(control, access);
	control	= GetDlgItem(dlg, IDC_GPIO_XFER_COMMAND_4);
	EnableWindow(control, access);
	control	= GetDlgItem(dlg, IDC_GPIO_XFER_COMMAND_5);
	EnableWindow(control, access);
	control	= GetDlgItem(dlg, IDC_GPIO_XFER_COMMAND_6);
	EnableWindow(control, access);

	_x_y_settings_load(data);

	control	= GetDlgItem(dlg, IDC_GPIO_XFER_X_CHECK);
	EnableWindow(control, idle_data);
	checked	= data->x.enabled ? BST_CHECKED : BST_UNCHECKED;
	SendMessage(control, BM_SETCHECK, checked, 0);
	control	= GetDlgItem(dlg, IDC_GPIO_XFER_X_BUTTON);
	EnableWindow(control, (BOOL) ((idle_data) && (data->x.enabled)));
	control	= GetDlgItem(dlg, IDC_GPIO_XFER_Y_CHECK);
	EnableWindow(control, idle_data);
	checked	= data->y.enabled ? BST_CHECKED : BST_UNCHECKED;
	SendMessage(control, BM_SETCHECK, checked, 0);
	control	= GetDlgItem(dlg, IDC_GPIO_XFER_Y_BUTTON);
	EnableWindow(control, (BOOL) ((idle_data) && (data->y.enabled)));

	status	= _buttons_update(dlg, data);
	error_status(dlg, IDC_GPIO_XFER_STATUS, status);
	(data->refresh_window)(dlg, data);

	if ((device->handle) && (msg == NULL))
			msg	= "";

	if (msg)
		_set_window_status(dlg, msg);

	if (device->handle == NULL)
		ctrl_menu_set(dlg, IDCANCEL, CTRL_FOCUS);
}



/*****************************************************************************
*
*	Function:	_ruler_wm_lbuttondown
*
*	Purpose:
*
*		Process a WM_LBUTTONDOWN message for the RULER area.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*		point		The coordinates of the mouse (dialog relative).
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _ruler_wm_lbuttondown(HWND dlg, gpio_xfer_t* data, POINT point)
{
	if (data->values == 0)
	{
	}
	else if (!data->graph.trackable)
	{
	}
	else if (data->ruler.inside)
	{
		SetCapture(dlg);
		data->ruler.captured	= TRUE;
		SetCursor(data->device->cursor.arrows);
		SetFocus(dlg);
		ClientToScreen(dlg, &point);
		ScreenToClient(data->ruler.control, &point);
		_data_track_begin(dlg, data, point.x);
	}
	else
	{
		data->ruler.wait_til_up	= TRUE;
	}
}



/*****************************************************************************
*
*	Function:	_ruler_wm_lbuttonup
*
*	Purpose:
*
*		Process a WM_LBUTTONUP message for the RULER area.
*
*	Arguments:
*
*		data		The window specific data we track.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static void _ruler_wm_lbuttonup(gpio_xfer_t* data)
{
	if (data->values == 0)
	{
	}
	else if (!data->graph.trackable)
	{
	}
	else
	{
		data->ruler.wait_til_up	= FALSE;

		if (data->ruler.captured)
		{
			ReleaseCapture();
			data->ruler.captured	= FALSE;
		}

		if (data->ruler.inside)
		{
			if (data->ruler.inside)
				SetCursor(data->device->cursor.arrows);
			else
				data->ruler.cursor_last	= SetCursor(data->device->cursor.arrows);

			data->ruler.inside	= TRUE;
		}
	}
}



/*****************************************************************************
*
*	Function:	_ruler_wm_mousemove
*
*	Purpose:
*
*		Process a WM_MOUSEMOVE message for the RULER area.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*		point		The coordinates of the mouse (dialog relative).
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _ruler_wm_mousemove(HWND dlg, gpio_xfer_t* data, POINT point)
{
	int	inside;

	if (data->values == 0)
	{
	}
	else if (!data->graph.trackable)
	{
	}
	else if (data->ruler.wait_til_up)
	{
	}
	else
	{
		ClientToScreen(dlg, &point);
		ScreenToClient(data->ruler.control, &point);

		if ((point.x >= data->ruler.rect.left) &&
			(point.x < data->ruler.rect.right) &&
			(point.y >= data->ruler.rect.top) &&
			(point.y < data->ruler.rect.bottom))
		{
			inside	= TRUE;
		}
		else
		{
			inside	= FALSE;
		}

		if (data->ruler.captured)
		{
			SetCursor(data->device->cursor.arrows);

			if ((point.x >= data->ruler.x_min) && (point.x < data->ruler.x_max))
				_data_track_update(dlg, data, point.x);
		}
		else if (inside)
		{
			if (data->ruler.inside)
				SetCursor(data->device->cursor.arrows);
			else
				data->ruler.cursor_last	= SetCursor(data->device->cursor.arrows);

			data->ruler.inside	= TRUE;
		}
		else
		{
			data->ruler.inside	= FALSE;
		}
	}
}



/*****************************************************************************
*
*	Function:	_ruler_wm_paint
*
*	Purpose:
*
*		Paint the ruler (usually per a WM_PAINT message).
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The dialog specific data we track.
*
*		invalidate	Invalidate the entire data area?
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _ruler_wm_paint(HWND dlg, gpio_xfer_t* data, int invalidate)
{
	static const int	list[]	=
	{ 1, 2, 5, 10, 25, 50, 100, 200, 250, 500 };

	UINT		align;
	int			bkmode;
	HBRUSH		brush_old;
	char		buf[16];
	float		chars;
	COLORREF	color;
	HDC			dc;
	int			delta;
	HFONT		font;
	HFONT		font_old;
	int			i;
	int			j;
	int			offset;
	HPEN		pen_old;
	TEXTMETRIC	tm;
	int			x;

	dc	= GetDC(dlg);
	GetTextMetrics(dc, &tm);
	ReleaseDC(dlg, dc);

	dc			= GetDC(data->ruler.control);
	bkmode		= SetBkMode(dc, OPAQUE);
	color		= SetTextColor(dc, data->device->paint.black.color);
	align		= SetTextAlign(dc, TA_CENTER | TA_TOP);
	font_old	= (HFONT) SendMessage(data->ruler.control, WM_GETFONT, 0, 0);
	font		= (HFONT) SendMessage(dlg, WM_GETFONT, 0, 0);
	SelectObject(dc, font);
	SetBkColor(dc, data->device->paint.btnface.color);
	pen_old		= SelectObject(dc, data->device->paint.btnface.pen);
	brush_old	= SelectObject(dc, data->device->paint.btnface.brush);

	if (invalidate)
	{
		InvalidateRect(data->ruler.control, NULL, TRUE);
		UpdateWindow(data->ruler.control);
	}

	// Paint the background.
	Rectangle(	dc,
				data->ruler.rect.left,
				data->ruler.rect.top,
				data->ruler.rect.right,
				data->ruler.rect.bottom);

	if ((data->active) || (data->values))
	{
		// Get the metrics for the ruler.

		for (i = 0; i < ((int) ARRAY_ELEMENTS(list) - 1); i++)
		{
			chars	= (float) ((data->scale * list[i]) / tm.tmAveCharWidth);

			if (chars > 7.0)
				break;
		}

		delta	= list[i];

		// Paint the ruler text and marks.
		SelectObject(dc, data->device->paint.black.pen);
		offset	= (data->offset / delta) * delta;

		for (i = offset;; i += delta)
		{
			x	= (int) INDEX_TO_PIXEL(i, data);

			sprintf(buf, "%d", i * data->interval);
			j	= (int) strlen(buf);
			ExtTextOut(	dc,
						x,
						data->ruler.rect.top,
						ETO_CLIPPED,
						&data->ruler.rect,
						buf,
						(UINT) j,
						NULL);

			if (x >= data->ruler.rect.right)
				break;

			if (x >= data->ruler.rect.left)
			{
				MoveToEx(dc, x, data->ruler.y, NULL);
				LineTo(dc, x, data->ruler.rect.bottom);
			}
		}

		// Paint the X cursor.

		if (data->x.enabled)
		{
			SetBkMode(dc, TRANSPARENT);
			SetTextColor(dc, data->device->paint.x_cursor.color);
			x	= (int) INDEX_TO_PIXEL(data->x.index, data);

			ExtTextOut(	dc,
						x,
						data->ruler.rect.top,
						ETO_CLIPPED,
						&data->ruler.rect,
						"X",
						1,
						NULL);
		}

		// Paint the Y cursor.

		if (data->y.enabled)
		{
			SetBkMode(dc, TRANSPARENT);
			SetTextColor(dc, data->device->paint.y_cursor.color);
			x	= (int) INDEX_TO_PIXEL(data->y.index, data);

			ExtTextOut(	dc,
						x,
						data->ruler.rect.top,
						ETO_CLIPPED,
						&data->ruler.rect,
						"Y",
						1,
						NULL);
		}
	}

	SelectObject(dc, brush_old);
	SelectObject(dc, font_old);
	SelectObject(dc, pen_old);
	SetBkMode(dc, bkmode);
	SetTextAlign(dc, align);
	SetTextColor(dc, color);
	ReleaseDC(data->ruler.control, dc);
}



/*****************************************************************************
*
*	Function:	_set_window_status
*
*	Purpose:
*
*		Update the window's status text.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		msg		The message to display.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _set_window_status(HWND dlg, const char* msg)
{
	ctrl_menu_set_text(dlg, IDC_GPIO_XFER_STATUS, msg);
}



/*****************************************************************************
*
*	Function:	_thread_cursor
*
*	Purpose:
*
*		Implement the service that updates the display as data is being
*		transferred.
*
*	Arguments:
*
*		pvArg	The thread's entry arguemtn.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static unsigned WINAPI _thread_cursor(LPVOID pvArg)
{
	gpio_xfer_t*	data	= (gpio_xfer_t*) pvArg;
	DWORD			thread_id;

	thread_id	= GetCurrentThreadId();
	AttachThreadInput(thread_id, data->thread.id, TRUE);

	_cursor_begin(data->dlg, data);

	for (;;)
	{
		if ((data->abort) || (data->active == FALSE))
			break;

		_cursor_update(data);
		Sleep(10);
	}

	_cursor_end(data);
	return(0);
}



/*****************************************************************************
*
*	Function:	_thread_xfer
*
*	Purpose:
*
*		Implement the data transfer service.
*
*	Arguments:
*
*		pvArg	The thread's entry arguemtn.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static unsigned WINAPI _thread_xfer(LPVOID pvArg)
{
	gpio_xfer_t*	data			= (gpio_xfer_t*) pvArg;
	long			elapsed			= 0;
	long			elapsed_last	= 0;
	int				i;
	unsigned		id;
	char			msg[128]		= "";
	long			remaining;
	U32				status;
	DWORD			thread_id;

	thread_id		= GetCurrentThreadId();
	AttachThreadInput(thread_id, data->thread.id, TRUE);

	data->abort			= FALSE;
	data->active		= TRUE;
	data->scale			= _compute_scale(data, data->capacity);
	data->x.enabled		= FALSE;
	data->x.selected	= FALSE;
	data->y.enabled		= FALSE;
	data->y.selected	= FALSE;
	_refresh_window(data->dlg, data->device, NULL);
	_data_scroll_pos_update(data->dlg, data);
	_interval_get(data->dlg, data);
	_wm_paint(data->dlg, data, TRUE);

	timeBeginPeriod(1);
	_beginthreadex(NULL, 0, _thread_cursor, data, 0, &id);

	for (i = 0;; i++)
	{
		status	= (data->xfer_data)(data);

		if (status != GSC_SUCCESS)
		{
			sprintf(msg,
					"Transfer error 0x%lX. Transferred %d values in %d.%03d s.",
					(long) status,
					data->values,
					(int) (elapsed / 1000),
					(int) (elapsed % 1000));
			_set_window_status(data->dlg, msg);
			break;
		}

		if (data->abort)
		{
			sprintf(msg,
					"Transfer aborted. Transferred %d values in %d.%03d s.",
					data->values,
					(int) (elapsed / 1000),
					(int) (elapsed % 1000));
			_set_window_status(data->dlg, msg);
			break;
		}

		if ((elapsed == 0) || (elapsed >= (elapsed_last + 1000)))
		{
			elapsed_last	= elapsed;
			remaining		= (data->transfer - i) * data->interval;

			if (remaining <= 0)
				remaining	= 0;

			sprintf(msg,
					"%d.%03d second(s) remaining ...",
					(int) (remaining / 1000),
					(int) (remaining % 1000));
			_set_window_status(data->dlg, msg);
		}

		elapsed	+= data->interval;
		data->values++;

		if (data->values >= data->transfer)
		{
			sprintf(msg,
					"Transferred %d values in %d.%03d s.",
					data->values,
					(int) (elapsed / 1000),
					(int) (elapsed % 1000));
			_set_window_status(data->dlg, msg);
			break;
		}

		Sleep((DWORD) data->interval);
	}

	timeEndPeriod(1);
	data->active	= FALSE;
	(data->xfer_end)(data);
	_x_y_settings_load(data);
	_refresh_window(data->dlg, data->device, msg);
	ctrl_menu_set(data->dlg, IDC_GPIO_XFER_START, CTRL_ENABLE | CTRL_FOCUS);
	_wm_paint(data->dlg, data, TRUE);
	return(0);
}



/*****************************************************************************
*
*	Function:	_x_y_settings_load
*
*	Purpose:
*
*		Retrieve the .ini settings.
*
*	Arguments:
*
*		data		The dialog specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _x_y_settings_load(gpio_xfer_t* data)
{
	if ((data->values) && (!data->active))
	{
		ini_bool_get(NULL, data->section, KEY_X_ENABLED, &data->x.enabled, FALSE);
		ini_int_get(NULL, data->section, KEY_X_INDEX, &data->x.index, 0);
		ini_bool_get(NULL, data->section, KEY_X_SELECTED, &data->x.selected, FALSE);
		ini_bool_get(NULL, data->section, KEY_Y_ENABLED, &data->y.enabled, FALSE);
		ini_int_get(NULL, data->section, KEY_Y_INDEX, &data->y.index, 0);
		ini_bool_get(NULL, data->section, KEY_Y_SELECTED, &data->y.selected, FALSE);

		if (!data->x.enabled)
			data->x.selected	= FALSE;

		if (data->x.index >= data->capacity)
			data->x.index	= data->capacity - 1;

		if (!data->y.enabled)
			data->y.selected	= FALSE;

		if (data->y.index >= data->capacity)
			data->y.index	= data->capacity - 1;
	}
	else if (data->values == 0)
	{
		data->x.enabled		= FALSE;
		data->x.index		= 0;
		data->x.selected	= FALSE;
		data->y.enabled		= FALSE;
		data->y.index		= 0;
		data->y.selected	= FALSE;
	}
}



/*****************************************************************************
*
*	Function:	_x_y_settings_save
*
*	Purpose:
*
*		Save the .ini settings.
*
*	Arguments:
*
*		data		The dialog specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _x_y_settings_save(gpio_xfer_t* data)
{
	if ((data->values) && (!data->active))
	{
		ini_bool_set(NULL, data->section, KEY_X_ENABLED, data->x.enabled);
		ini_int_set(NULL, data->section, KEY_X_INDEX, data->x.index);
		ini_bool_set(NULL, data->section, KEY_X_SELECTED, data->x.selected);
		ini_bool_set(NULL, data->section, KEY_Y_ENABLED, data->y.enabled);
		ini_int_set(NULL, data->section, KEY_Y_INDEX, data->y.index);
		ini_bool_set(NULL, data->section, KEY_Y_SELECTED, data->y.selected);
	}
}



/*****************************************************************************
*
*	Function:	_idc_gpio_xfer_file
*
*	Purpose:
*
*		Prompt for a file name to save or load data.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_gpio_xfer_file(HWND dlg, gpio_xfer_t* data)
{
	int	interval;

	interval	= (data->xfer_file)(dlg, data);

	if (interval)
	{
		_interval_get(dlg, data);
		_refresh_window(dlg, data->device, NULL);
		_wm_paint(dlg, data, TRUE);
	}
}



/*****************************************************************************
*
*	Function:	_idc_gpio_xfer_interval_list
*
*	Purpose:
*
*		Respond to a Interval List Box control.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's WORD parameter.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_gpio_xfer_interval_list(HWND dlg, WPARAM wParam)
{
	gpio_xfer_t*	data	= (gpio_xfer_t*) GET_WINDOW_USER_DATA(dlg);

	if ((HIWORD(wParam) == LBN_SELCHANGE) && (data->idc_interval))
	{
		_interval_get(dlg, data);
		_wm_paint(dlg, data, FALSE);
	}
}



/*****************************************************************************
*
*	Function:	_idc_gpio_xfer_start
*
*	Purpose:
*
*		Read in GPIO data.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_gpio_xfer_start(HWND dlg, gpio_xfer_t* data)
{
	unsigned	id;
	int			procede;
	long		thread;

	procede	= (data->xfer_begin)(data);

	if (procede)
	{
		data->thread.id		= GetCurrentThreadId();
		thread				= (long) _beginthreadex(NULL, 0, _thread_xfer, data, 0, &id);

		if (thread == -1L)
		{
			_set_window_status(dlg, "Unable to initiate operation.");
			(data->xfer_end)(data);
		}
	}
}



/*****************************************************************************
*
*	Function:	_idc_gpio_xfer_x_button
*
*	Purpose:
*
*		Respond to activity on the X button.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_gpio_xfer_x_button(HWND dlg, gpio_xfer_t* data)
{
	data->x.selected	= TRUE;
	data->y.selected	= FALSE;
	ctrl_menu_set_text(dlg, IDC_GPIO_XFER_STATUS, "Select X");
	_x_y_settings_save(data);
}



/*****************************************************************************
*
*	Function:	_idc_gpio_xfer_x_check
*
*	Purpose:
*
*		Respond to activity on the X check box.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_gpio_xfer_x_check(HWND dlg, gpio_xfer_t* data)
{
	HWND	control;
	int		state;

	control	= GetDlgItem(dlg, IDC_GPIO_XFER_X_CHECK);
	state	= (int) SendMessage(control, BM_GETCHECK, 0, 0);

	if (state == BST_CHECKED)
	{
		data->x.enabled	= TRUE;
		ctrl_menu_set(dlg, IDC_GPIO_XFER_X_BUTTON, CTRL_ENABLE);
		_idc_gpio_xfer_x_button(dlg, data);
	}
	else
	{
		data->x.enabled		= FALSE;
		data->x.selected	= FALSE;
		ctrl_menu_set(dlg, IDC_GPIO_XFER_X_BUTTON, CTRL_DISABLE);
		ctrl_menu_set_text(dlg, IDC_GPIO_XFER_STATUS, "Deselect X");
	}

	_wm_paint(dlg, data, TRUE);
	_x_y_settings_save(data);
}



/*****************************************************************************
*
*	Function:	_idc_gpio_xfer_y_button
*
*	Purpose:
*
*		Respond to activity on the Y button.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_gpio_xfer_y_button(HWND dlg, gpio_xfer_t* data)
{
	data->x.selected	= FALSE;
	data->y.selected	= TRUE;
	ctrl_menu_set_text(dlg, IDC_GPIO_XFER_STATUS, "Select Y");
	_x_y_settings_save(data);
}



/*****************************************************************************
*
*	Function:	_idc_gpio_xfer_y_check
*
*	Purpose:
*
*		Respond to activity on the Y check box.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_gpio_xfer_y_check(HWND dlg, gpio_xfer_t* data)
{
	HWND	control;
	int		state;

	control	= GetDlgItem(dlg, IDC_GPIO_XFER_Y_CHECK);
	state	= (int) SendMessage(control, BM_GETCHECK, 0, 0);

	if (state == BST_CHECKED)
	{
		data->y.enabled	= TRUE;
		ctrl_menu_set(dlg, IDC_GPIO_XFER_Y_BUTTON, CTRL_ENABLE);
		_idc_gpio_xfer_y_button(dlg, data);
	}
	else
	{
		data->y.enabled		= FALSE;
		data->y.selected	= FALSE;
		ctrl_menu_set(dlg, IDC_GPIO_XFER_Y_BUTTON, CTRL_DISABLE);
		ctrl_menu_set_text(dlg, IDC_GPIO_XFER_STATUS, "Deselect Y");
	}

	_wm_paint(dlg, data, TRUE);
	_x_y_settings_save(data);
}



/*****************************************************************************
*
*	Function:	_idcancel
*
*	Purpose:
*
*		Respond to a request to close the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idcancel(HWND dlg)
{
	gpio_xfer_t*	data;

	main_refresh_add(dlg, NULL);
	data	= (gpio_xfer_t*) GET_WINDOW_USER_DATA(dlg);

	if (data)
	{
		if (data->active)
			SendMessage(dlg, WM_COMMAND, IDC_GPIO_XFER_STOP, 0);

		_interval_get(dlg, data);
		win_pos_save(dlg, data->section);
		data->dlg	= NULL;
	}

	DestroyWindow(dlg);
}



/*****************************************************************************
*
*	Function:	_wm_command
*
*	Purpose:
*
*		Process WM_COMMAND messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		A service dependent value.
*
*		lParam		A service dependent value.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_command(HWND dlg, WPARAM wParam, LPARAM lParam)
{
	gpio_xfer_t*	data	= (gpio_xfer_t*) GET_WINDOW_USER_DATA(dlg);
	int				id		= LOWORD(wParam);
	LRESULT			ret		= TRUE;

	switch (id)
	{
		default:

			ret	= FALSE;
			break;

		case IDC_GPIO_XFER_COMMAND_0:

			dlg	= GetParent(dlg);
			cable_command_0_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_GPIO_XFER_COMMAND_1:

			dlg	= GetParent(dlg);
			cable_command_1_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_GPIO_XFER_COMMAND_2:

			dlg	= GetParent(dlg);
			cable_command_2_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_GPIO_XFER_COMMAND_3:

			dlg	= GetParent(dlg);
			cable_command_3_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_GPIO_XFER_COMMAND_4:

			dlg	= GetParent(dlg);
			cable_command_4_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_GPIO_XFER_COMMAND_5:

			dlg	= GetParent(dlg);
			cable_command_5_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_GPIO_XFER_COMMAND_6:

			dlg	= GetParent(dlg);
			cable_command_6_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_GPIO_XFER_FILE:

			_idc_gpio_xfer_file(dlg, data);
			break;

		case IDC_GPIO_XFER_INTERVAL_LIST:

			_idc_gpio_xfer_interval_list(dlg, wParam);
			break;

		case IDC_GPIO_XFER_START:

			_idc_gpio_xfer_start(dlg, data);
			break;

		case IDC_GPIO_XFER_STOP:

			data->abort	= TRUE;
			break;

		case IDC_GPIO_XFER_X_BUTTON:

			_idc_gpio_xfer_x_button(dlg, data);
			break;

		case IDC_GPIO_XFER_X_CHECK:

			_idc_gpio_xfer_x_check(dlg, data);
			break;

		case IDC_GPIO_XFER_Y_BUTTON:

			_idc_gpio_xfer_y_button(dlg, data);
			break;

		case IDC_GPIO_XFER_Y_CHECK:

			_idc_gpio_xfer_y_check(dlg, data);
			break;

		case IDCANCEL:

			_idcancel(dlg);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_wm_destroy
*
*	Purpose:
*
*		Process WM_DESTROY messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_destroy(HWND dlg)
{
	_idcancel(dlg);
	DestroyWindow(dlg);
	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_getminmaxinfo
*
*	Purpose:
*
*		Process a WM_GETMINMAXINFO message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		lParam		The messages's long size parameter.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_getminmaxinfo(HWND dlg, LPARAM lParam)
{
	HWND			control;
	gpio_xfer_t*	data	= (gpio_xfer_t*) GET_WINDOW_USER_DATA(dlg);
	RECT			inside;
	MINMAXINFO*		mmi		= (MINMAXINFO*) lParam;
	RECT			outside;
	RECT			rect;

	_align_controls(dlg, data, FALSE);

	// Height
	GetWindowRect(dlg, &outside);
	GetClientRect(dlg, &inside);
	control	= GetDlgItem(dlg, IDC_GPIO_XFER_FRAME);
	GetWindowRect(control, &rect);
	ScreenToClient(dlg, (POINT*) &rect.left);
	ScreenToClient(dlg, (POINT*) &rect.right);
	mmi->ptMaxTrackSize.y	= rect.bottom
							- 1
							+ RECT_HEIGHT(outside)
							- RECT_HEIGHT(inside);
	mmi->ptMinTrackSize.y	= mmi->ptMaxTrackSize.y;

	// Width
	control	= GetDlgItem(dlg, IDCANCEL);
	GetWindowRect(control, &rect);
	ScreenToClient(dlg, (POINT*) &rect.left);
	ScreenToClient(dlg, (POINT*) &rect.right);
	mmi->ptMinTrackSize.x	= rect.right
							+ data->device->margin
							+ RECT_WIDTH(outside)
							- RECT_WIDTH(inside);
	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_hscroll
*
*	Purpose:
*
*		Process WM_HSCROLL messages for the dialog, which is for horizontal
*		positioning.
*
*	Arguments:
*
*		dlg			The dialog in use.
*
*		data		The data for the work we're to do.
*
*		wParam		The wParam parameter for the message.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not processed the message.
*
*****************************************************************************/

static LRESULT _wm_hscroll(HWND dlg, WPARAM wParam)
{
	int				code	= LOWORD(wParam);
	gpio_xfer_t*	data	= (gpio_xfer_t*) GET_WINDOW_USER_DATA(dlg);
	int				line;
	int				offset	= data->offset;
	int				page;

	page	= (int) (((data->graph.width + 1) / data->scale) + 0.5);
	line	= page / 20;

	if (line < 1)
		line	= 1;

	switch (code)
	{
		default:											break;
		case SB_BOTTOM:			offset	= data->capacity;	break;
		case SB_LINEDOWN:		offset	+= line;			break;
		case SB_LINEUP:			offset	-= line;			break;
		case SB_PAGEDOWN:		offset	+= page;			break;
		case SB_PAGEUP:			offset	-= page;			break;
		case SB_THUMBPOSITION:	offset	= HIWORD(wParam);	break;
		case SB_THUMBTRACK:		offset	= HIWORD(wParam);	break;
		case SB_TOP:			offset	= 0;				break;
	}

	if (offset >= (data->capacity - page))
		offset	= data->capacity - page;
	else if (offset < 0)
		offset	= 0;

	if (offset != data->offset)
	{
		data->offset	= offset;
		_wm_paint(dlg, data, TRUE);
		_data_scroll_pos_update(dlg, data);
	}

	return(TRUE);
}



/*****************************************************************************
*
*	Function:	_wm_initdialog
*
*	Purpose:
*
*		Process a WM_INITDIALOG message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		lParam		The message's long sized argument.
*
*	Returned:
*
*		TRUE		Windows should set the initial control focus.
*		FALSE		Windows should NOT set the initial control focus.
*
*****************************************************************************/

static LRESULT _wm_initdialog(HWND dlg, LPARAM lParam)
{
	gpio_xfer_t*	data;
	HDC				dc;
	RECT			rect;
	U32				status;
	TEXTMETRIC		tm;

	SET_WINDOW_USER_DATA(dlg, lParam);
	data	= (gpio_xfer_t*) GET_WINDOW_USER_DATA(dlg);

	SendMessage(dlg, WM_SETICON, ICON_SMALL, (LPARAM) data->device->icon.little);
	SendMessage(dlg, WM_SETICON, ICON_BIG, (LPARAM) data->device->icon.big);

	_interval_init(dlg, data);
	dc	= GetDC(dlg);
	GetTextMetrics(dc, &tm);
	ReleaseDC(dlg, dc);
	status				= _buttons_update(dlg, data);
	error_status(dlg, IDC_GPIO_XFER_STATUS, status);
	data->abort			= FALSE;
	data->active		= FALSE;
	data->interval		= 5;
	data->offset		= 0;
	data->values		= 0;
	data->graph.control	= GetDlgItem(dlg, IDC_GPIO_XFER_DATA);
	data->ruler.control	= GetDlgItem(dlg, IDC_GPIO_XFER_RULER);
	data->ruler.y		= tm.tmAscent;
	_x_y_settings_load(data);

	GetClientRect(data->graph.control, &rect);
	data->graph.width			= RECT_WIDTH(rect);
	data->scale					= _compute_scale(data, data->capacity);
	data->cursor.x_right		= data->graph.rect.left - 1;
	data->cursor.x				= data->graph.rect.left - 1;
	data->cursor.values_last	= 0;

	_align_controls(dlg, data, FALSE);

	win_pos_restore(dlg, data->section, TRUE);
	(data->wm_initdialog)(dlg, data);

	_set_window_status(dlg, "");
	_refresh_window(dlg, data->device, NULL);
	main_refresh_add(dlg, _refresh_window);
	return(TRUE);
}



/*****************************************************************************
*
*	Function:	_wm_lbuttondown
*
*	Purpose:
*
*		Process a WM_LBUTTONDOWN message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		lParam		The message's LONG parameter.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_lbuttondown(HWND dlg, LPARAM lParam)
{
	gpio_xfer_t*	data	= (gpio_xfer_t*) GET_WINDOW_USER_DATA(dlg);
	POINT			point;

	point.x	= LOWORD(lParam);
	point.y	= HIWORD(lParam);

	_ruler_wm_lbuttondown(dlg, data, point);
	_graph_wm_lbuttondown(dlg, data, point);
	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_lbuttonup
*
*	Purpose:
*
*		Process a WM_LBUTTONUP message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_lbuttonup(HWND dlg)
{
	gpio_xfer_t*	data	= (gpio_xfer_t*) GET_WINDOW_USER_DATA(dlg);

	_ruler_wm_lbuttonup(data);
	_graph_wm_lbuttonup(data);
	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_mousemove
*
*	Purpose:
*
*		Process a WM_MOUSEMOVE message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		lParam		The message's LONG parameter.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_mousemove(HWND dlg, LPARAM lParam)
{
	gpio_xfer_t*	data	= (gpio_xfer_t*) GET_WINDOW_USER_DATA(dlg);
	POINT			point;

	point.x	= LOWORD(lParam);
	point.y	= HIWORD(lParam);

	_ruler_wm_mousemove(dlg, data, point);
	_graph_wm_mousemove(dlg, data, point);
	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_paint
*
*	Purpose:
*
*		Process a WM_PAINT message.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The data we're to display.
*
*		invalidate	Invalidate the entire data area?
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_paint(HWND dlg, gpio_xfer_t* data, int invalidate)
{
	_ruler_wm_paint(dlg, data, invalidate);
	_graph_wm_paint(data, invalidate);
	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_rbuttondown
*
*	Purpose:
*
*		Process a WM_RBUTTONDOWN message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		lParam		The message's LONG parameter.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_rbuttondown(HWND dlg, LPARAM lParam)
{
	gpio_xfer_t*	data	= (gpio_xfer_t*) GET_WINDOW_USER_DATA(dlg);
	POINT			point;

	point.x	= LOWORD(lParam);
	point.y	= HIWORD(lParam);

	_graph_wm_rbuttondown(dlg, data, point);
	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_rbuttonup
*
*	Purpose:
*
*		Process a WM_RBUTTONUP message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_rbuttonup(HWND dlg)
{
	gpio_xfer_t*	data	= (gpio_xfer_t*) GET_WINDOW_USER_DATA(dlg);

	_graph_wm_rbuttonup(data);
	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_size
*
*	Purpose:
*
*		Process a WM_SIZE message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		lParam		The message's LONG sized parameter.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_size(HWND dlg, LPARAM lParam)
{
	HWND			control;
	gpio_xfer_t*	data;
	int				i;
	RECT			rect;
	float			values;
	int				width_old;

	if ((lParam != SIZE_MINIMIZED) && (lParam != SIZE_MAXHIDE))
	{
		data	= (gpio_xfer_t*) GET_WINDOW_USER_DATA(dlg);
		_align_controls(dlg, data, FALSE);
		width_old	= data->graph.width;

		GetClientRect(data->graph.control, &data->graph.rect);
		data->graph.rect.left		+= 1;
		data->graph.rect.right		-= 1;
		data->graph.rect.top		+= 1;
		data->graph.rect.bottom		-= 1;
		data->graph.width			= RECT_WIDTH(data->graph.rect);

		GetClientRect(data->ruler.control, &data->ruler.rect);
		data->ruler.rect.left		+= 1;
		data->ruler.rect.right		-= 1;
		data->ruler.rect.top		+= 1;
		data->ruler.rect.bottom		-= 1;
		data->ruler.width			= RECT_WIDTH(data->ruler.rect);

		for (i = 0; i < (int) ARRAY_ELEMENTS(_command_list); i++)
		{
			control	= GetDlgItem(dlg, _command_list[i].ctrl_id);
			GetWindowRect(control, &rect);
			ScreenToClient(data->graph.control, (POINT*) &rect.left);
			ScreenToClient(data->graph.control, (POINT*) &rect.right);
			data->signals[i].y_top	= rect.top + 1;
			data->signals[i].y_bot	= rect.bottom - 1;
		}

		values			= ((float) width_old) / data->scale;
		data->scale		= _compute_scale(data, (int) values);
		_wm_vscroll(dlg, (WPARAM) -1);
		InvalidateRect(dlg, NULL, TRUE);
		UpdateWindow(dlg);
		_wm_paint(dlg, data, TRUE);
		_data_scroll_pos_update(dlg, data);
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_vscroll
*
*	Purpose:
*
*		Process a WM_VSCROLL message for the dialog, which is for scaling.
*
*	Arguments:
*
*		dlg			The dialog in use.
*
*		wParam		The wParam parameter for the message.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not processed the message.
*
*****************************************************************************/

static LRESULT _wm_vscroll(HWND dlg, WPARAM wParam)
{
	int				code	= LOWORD(wParam);
	gpio_xfer_t*	data	= (gpio_xfer_t*) GET_WINDOW_USER_DATA(dlg);
	int				delta;
	int				offset	= data->offset;
	float			scale	= data->scale;
	int				values;

	values	= (int) (((data->graph.width + 1) / scale) + 0.5);

	if ((code == SB_LINEUP) || (code == SB_LINEDOWN))
	{
		delta	= values /20;

		if (delta == 0)
			delta	= 1;

		if (code == SB_LINEUP)
			values	-= delta;
		else
			values	+= delta;
	}

	if (values < VALUES_MIN)
		values	= VALUES_MIN;

	if (values > data->capacity)
		values	= data->capacity;

	scale	= _compute_scale(data, values);

	if ((offset + values) > data->capacity)
		offset	= data->capacity - values;

	if ((scale != data->scale) || (offset != data->offset))
	{
		data->offset	= offset;
		data->scale		= scale;
		_wm_paint(dlg, data, TRUE);
		_data_scroll_pos_update(dlg, data);
	}

	return(TRUE);
}



/*****************************************************************************
*
*	Function:	_dlg_proc
*
*	Purpose:
*
*		Processes messages for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		msg			The message to process.
*
*		wParam		The message's word sized parameter.
*
*		lParam		The message's long sized parameter.
*
*	Returned:
*
*		A message specific value.
*
*****************************************************************************/

static INT_PTR CALLBACK _dlg_proc(	HWND	dlg,
									UINT	msg,
									WPARAM	wParam,
									LPARAM	lParam)
{
	gpio_xfer_t*	data;
	LRESULT			ret;

	switch (msg)
	{
		default:

			ret	= FALSE;
			break;

		case WM_COMMAND:

			ret	= _wm_command(dlg, wParam, lParam);
			break;

		case WM_DESTROY:

			ret	= _wm_destroy(dlg);
			break;

		case WM_GETMINMAXINFO:

			ret		= _wm_getminmaxinfo(dlg, lParam);
			break;

		case WM_HSCROLL:

			ret		= _wm_hscroll(dlg, wParam);
			break;

		case WM_INITDIALOG:

			ret	= _wm_initdialog(dlg, lParam);
			break;

		case WM_LBUTTONDOWN:

			ret		= _wm_lbuttondown(dlg, lParam);
			break;

		case WM_LBUTTONUP:

			ret		= _wm_lbuttonup(dlg);
			break;

		case WM_MOUSEMOVE:

			ret		= _wm_mousemove(dlg, lParam);
			break;

		case WM_PAINT:

			data	= (gpio_xfer_t*) GET_WINDOW_USER_DATA(dlg);
			ret		= _wm_paint(dlg, data, FALSE);
			break;

		case WM_RBUTTONDOWN:

			ret		= _wm_rbuttondown(dlg, lParam);
			break;

		case WM_RBUTTONUP:

			ret		= _wm_rbuttonup(dlg);
			break;

		case WM_SIZE:

			ret	= _wm_size(dlg, lParam);
			break;

		case WM_VSCROLL:

			ret		= _wm_vscroll(dlg, wParam);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_service_wm_command
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_WM_COMMAND service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_wm_command(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	gpio_xfer_t*	data	= (gpio_xfer_t*) wParam;

	if (data->dlg)
	{
		SetFocus(data->dlg);
	}
	else
	{
		data->device	= device;
		data->dlg		= CreateDialogParam(device->instance,
											(LPCTSTR) IDD_GPIO_XFER,
											dlg,
											(DLGPROC) _dlg_proc,
											(LPARAM) data);
		ShowWindow(data->dlg, SW_SHOWNORMAL);
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	gpio_xfer_service
*
*	Purpose:
*
*		Implement the common service entry point for the GPIO Read/Write
*		services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT gpio_xfer_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service				function			*/
		{ SERVICE_WM_COMMAND,	_service_wm_command	},
		{ SERVICE_END_LIST,		NULL				}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}


