// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/misc_txrx_tri.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement support for the Miscellaneous Tx/Rx Tri-State option.
*
*****************************************************************************/

#include <windows.h>

#include "cable.h"
#include "resource.h"
#include "util_ctrl.h"



/* variables	*************************************************************/

static const value_list_entry_t	_tx_rx_tri_state_value_list[]	=
{
	{
	/* user_text			*/	"No (Drive Tx Enable and Rx Enable at all times.)",
	/* value + value_text	*/	VAR_STR_MACRO(HPDI32_MISC_TX_RX_TRI_STATE_DISABLE),
	/* suffix				*/	"_NO"
	},
	{
	/* user_text			*/	"Yes (Drive Tx Enable when the Transmitter is"
								" enabled, and Rx Enable when the Receiver is"
								" enabled.)",
	/* value + value_text	*/	VAR_STR_MACRO(HPDI32_MISC_TX_RX_TRI_STATE_ENABLE),
	/* suffix				*/	"_YES"
	}
};

static const value_list_t	_tx_rx_tri_state_values[]	=
{
	{
	/* list	*/	_tx_rx_tri_state_value_list,
	/* size	*/	ARRAY_ELEMENTS(_tx_rx_tri_state_value_list)
	}
};

static const value_t	_tx_rx_tri_state_value	=
{
	/* type	*/	VALUE_TYPE_LIST,
	{
	/* u.vp	*/	_tx_rx_tri_state_values
	}
};

const parm_entry_t	misc_parm_tx_rx_tri_state	=
{
	/* title			*/	"Tx Enable/Rx Enable Tri-State",
	/* parm + parm_text	*/	VAR_STR_MACRO(HPDI32_MISC_TX_RX_TRI_STATE),
	/* suffix			*/	"",
	/* which			*/	0,
	/* set_arg			*/	"",
	/* get_arg			*/	"",
	/* values			*/	&_tx_rx_tri_state_value,
	/* cfg_wiz			*/	NULL,
	/* desc				*/
	"This is intended primarily to permit two HPDI32 boards to be connected"
	" back-to-back for testing purposes. In the hardware manual this is"
	" referred to as Test Mode. Enabling this parameter permits two such"
	" boards to perform data transfer without driving the Tx Enable and Rx"
	" Enable signals to opposite states, which would likely result in"
	" transceiver failure. If these signals are not used by the customer's"
	" hardware, this parameter can otherwise be ignored."
	" NOTE: Applications can generally ignore this parameter if using a single"
	" HPDI32 only, the remote device doesn't use the respective Tx/Rx Enable"
	" signal, and transmit only or receive only operations are performed."
};



/*****************************************************************************
*
*	Function:	_service_refresh
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_REFRESH service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_refresh(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	U32		state;
	U32		status;

	if (device->handle)
	{
		status	= HPDI32_MISC_TX_RX_TRI_STATE__GET(device->handle, &state);
		error_status(dlg, 0, status);

		if (state == HPDI32_MISC_TX_RX_TRI_STATE_ENABLE)
			ctrl_menu_set(dlg, IDC_CABLE_TRI_STATE, CTRL_ENABLE | CTRL_BM_CHECK);
		else
			ctrl_menu_set(dlg, IDC_CABLE_TRI_STATE, CTRL_ENABLE | CTRL_BM_UNCHECK);
	}
	else
	{
		ctrl_menu_set(dlg, IDC_CABLE_TRI_STATE, CTRL_DISABLE | CTRL_BM_UNCHECK);
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	_service_wm_command
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_WM_COMMAND service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_wm_command(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	U32		stat;
	U32		state;
	U32		status;

	status	= HPDI32_MISC_TX_RX_TRI_STATE__GET(device->handle, &state);
	error_status(dlg, 0, status);

	if (state == HPDI32_MISC_TX_RX_TRI_STATE_DISABLE)
		stat	= HPDI32_MISC_TX_RX_TRI_STATE__YES(device->handle);
	else
		stat	= HPDI32_MISC_TX_RX_TRI_STATE__NO(device->handle);

	status	= (status == GSC_SUCCESS) ? stat : status;
	main_refresh(device, NULL);
	error_status(dlg, 0, status);
	return(0);
}



/*****************************************************************************
*
*	Function:	misc_tx_rx_tri_state_service
*
*	Purpose:
*
*		Implement the common service entry point for the Miscellaneous Tx
*		Enabled/Rx Enabled Tri-State services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT misc_tx_rx_tri_state_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service				function			*/
		{ SERVICE_REFRESH,		_service_refresh	},
		{ SERVICE_WM_COMMAND,	_service_wm_command	},
		{ SERVICE_END_LIST,		NULL				}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}


