// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/rx_receive.c $
// $Rev: 31452 $
// $Date: 2014-11-13 09:58:59 -0600 (Thu, 13 Nov 2014) $

/*****************************************************************************
*
*	Implement support for the receiver's Rx Receive option.
*
*****************************************************************************/

#include <windows.h>
#include <stdio.h>

#include "gui.h"
#include "rx.h"
#include "align.h"
#include "arrow.h"
#include "button.h"
#include "data_xfer.h"
#include "ini.h"
#include "resource.h"
#include "util_ctrl.h"



/*****************************************************************************
*
*	Function:	_get_state_info
*
*	Purpose:
*
*		Get the state info we need about the device.
*
*	Arguments:
*
*		device		The device data.
*
*		enabled		Tell the caller if the receiver is enable.
*
*		fifo		Tell the caller if the receiver has FIFOs.
*
*		width		Tell the caller of the device's data width in bits.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _get_state_info(device_t* device, int* enabled, int* fifo, int* width)
{
	U32	enable;
	U32	feature;
	U32	size;
	U32	stat;
	U32	status	= GSC_SUCCESS;

	stat		= HPDI32_IO_DATA_SIZE__RX_GET(device->handle, &size);
	status		= (status == GSC_SUCCESS) ? stat : status;

	switch (size)
	{
		default:
		case HPDI32_IO_DATA_SIZE_8_BITS:

			width[0]	= 8;
			break;

		case HPDI32_IO_DATA_SIZE_16_BITS:

			width[0]	= 16;
			break;

		case HPDI32_IO_DATA_SIZE_32_BITS:

			width[0]	= 32;
			break;
	}

	stat		= HPDI32_RX_ENABLE__GET(device->handle, &enable);
	status		= (status == GSC_SUCCESS) ? stat : status;
	enabled[0]	= (enable == HPDI32_RX_ENABLE_YES) ? TRUE : FALSE;
	stat		= HPDI32_FIFO_SIZE__RX_GET(device->handle, &size);
	status		= (status == GSC_SUCCESS) ? stat : status;
	stat		= HPDI32_MISC_FEATURES__FIFO_SIZE(device->handle, &feature);
	status		= (status == GSC_SUCCESS) ? stat : status;

	if ((feature == HPDI32_MISC_FEATURES_ABSENT) || (size))
		fifo[0]	= TRUE;
	else
		fifo[0]	= FALSE;

	return(status);
}



/*****************************************************************************
*
*	Function:	_refresh_window
*
*	Purpose:
*
*		Update the content of the window per the state of the system.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None;
*
*****************************************************************************/

static void _refresh_window(HWND dlg, data_xfer_t* data)
{
	HWND	control;
	U32		enabled;
	U32		fifos;
	BOOL	inactive	= data->active == FALSE;
	BOOL	idle_data	= (data->data.count) && (inactive);
	U32		size;
	U32		stat;
	U32		status		= GSC_SUCCESS;

	if (data->device->handle)
	{
		stat	= HPDI32_RX_ENABLE__GET(data->device->handle, &enabled);
		status	= (status == GSC_SUCCESS) ? stat : status;
		enabled	= (enabled == HPDI32_RX_ENABLE_YES) ? 1 : 0;
		stat	= HPDI32_FIFO_SIZE__RX_GET(data->device->handle, &size);
		status	= (status == GSC_SUCCESS) ? stat : status;
		stat	= HPDI32_MISC_FEATURES__FIFO_SIZE(data->device->handle, &fifos);
		status	= (status == GSC_SUCCESS) ? stat : status;
		fifos	= ((fifos == HPDI32_MISC_FEATURES_PRESENT) && (size)) ? 1 : 0;
	}
	else
	{
		enabled	= 0;
		fifos	= 0;
		SendMessage(dlg, WM_COMMAND, IDC_DATA_XFER_STOP, 0);
		ctrl_menu_set(dlg, IDCANCEL, CTRL_ENABLE | CTRL_FOCUS);
	}

	control	= GetDlgItem(dlg, IDC_DATA_XFER_START);
	EnableWindow(control, (BOOL) ((fifos) && (inactive)));
	control	= GetDlgItem(dlg, IDC_DATA_XFER_FILE);
	EnableWindow(control, idle_data);
	control	= GetDlgItem(dlg, IDC_DATA_XFER_AMOUNT_TEXT);
	EnableWindow(control, (BOOL) ((fifos) && (inactive)));
	control	= GetDlgItem(dlg, IDC_DATA_XFER_AMOUNT_EDIT);
	EnableWindow(control, (BOOL) ((fifos) && (inactive)));

	if (data->device->handle == NULL)
		;
	else if (fifos == 0)
		strcpy(data->status_local, "No FIFOs installed.");
	else if (!enabled)
		strcpy(data->status_local, "The receiver is disabled.");

	if (status != GSC_SUCCESS)
		data->status	= status;
}



/*****************************************************************************
*
*	Function:	_wm_initdialog
*
*	Purpose:
*
*		Process a WM_INITDIALOG message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The dialog specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _wm_initdialog(HWND dlg, data_xfer_t* data)
{
	U32		amount;
	char	buf[16];

	ctrl_menu_set_text(dlg, IDC_DATA_XFER_START, "Read");
	ctrl_menu_set_text(dlg, IDC_DATA_XFER_FILE, "Save");

	ini_u32_get(NULL, data->section, data->amount_key, &amount, 1UL * 1024 * 1024);
	sprintf(buf, "%lu", (long) amount);
	ctrl_menu_set_text(dlg, IDC_DATA_XFER_AMOUNT_EDIT, buf);
}



/*****************************************************************************
*
*	Function:	_xfer_begin
*
*	Purpose:
*
*		Prepare for the transfer process to commense.
*
*	Arguments:
*
*		data		Our local data set.
*
*	Returned:
*
*		TRUE		Procede with the transfer.
*		FALSE		Do not procede with the transfer.
*
*****************************************************************************/

static int _xfer_begin(data_xfer_t* data)
{
	HWND	control;
	int		procede;

	for (;;)	// A convenience loop.
	{
		// Check the data size.
		data->status	= HPDI32_IO_DATA_SIZE__RX_GET(	data->device->handle,
														&data->data.size);

		// Make sure we've got an Application Buffer.

		if ((data->amount != data->app_buf_size) && (data->app_buf))
		{
			free(data->app_buf);
			data->app_buf		= NULL;
			data->app_buf_size	= 0;
		}

		if (data->app_buf == NULL)
		{
			data->app_buf	= malloc(data->amount);

			if (data->app_buf == NULL)
			{
				control	= GetDlgItem(data->dlg, IDC_DATA_XFER_AMOUNT_EDIT);
				SetFocus(control);
				error_status(	data->dlg,
								IDC_DATA_XFER_STATUS,
								GSC_INSUFFICIENT_RESOURCES);
				procede	= FALSE;
				break;
			}

			data->app_buf_size	= data->amount;
		}

		// Clear the API Buffer info if we're not using it.

		if (data->device->rx.app_buf)
		{
			data->api_buf		= 0;
			data->api_buf_size	= 0;
		}
		else if (data->api_buf == 0)
		{
			ctrl_menu_set_text(	data->dlg,
								IDC_DATA_XFER_STATUS,
								"Couldn't get API Buffer pointer.");
			procede	= FALSE;
			break;
		}

		data->repeat		= 0;
		data->xfer_tot		= data->amount;
		procede				= TRUE;
		break;
	}

	return(procede);
}



/*****************************************************************************
*
*	Function:	_xfer_data
*
*	Purpose:
*
*		Prepare for the transfer process to commense.
*
*	Arguments:
*
*		data		The dialog specific data we track.
*
*		xfer		The number of bytes to transfer.
*
*		xfered		The number of bytes actually transferred.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _xfer_data(data_xfer_t* data, U32 xfer, U32* xfered)
{
	U32	status;

	if (data->api_buf_ptr)
	{
		status	= hpdi32_read(data->device->handle, data->api_buf_ptr, xfer, xfered);

		if ((status == GSC_SUCCESS) && (xfered[0]))
			memcpy(data->app_buf_ptr, data->api_buf_ptr, xfered[0]);
	}
	else
	{
		status	= hpdi32_read(data->device->handle, data->app_buf_ptr, xfer, xfered);
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_xfer_end
*
*	Purpose:
*
*		Prepare for the transfer process to commense.
*
*	Arguments:
*
*		data		Our local data set.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _xfer_end(data_xfer_t* data)
{
	data->data.count	= data->xfered_tot;
}



/*****************************************************************************
*
*	Function:	_xfer_file
*
*	Purpose:
*
*		Prompt for a file name and save the data.
*
*	Arguments:
*
*		data		The dialog specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _xfer_file(data_xfer_t* data)
{
	HCURSOR			cursor;
	char			dir[_MAX_PATH];
	FILE*			file;
	TCHAR			name[_MAX_PATH];
	OPENFILENAME	ofn;
	U32				u32;

	name[0]	= 0;
	GetCurrentDirectory(sizeof(dir), dir);

	// Initialize OPENFILENAME
	memset(&ofn, 0, sizeof(OPENFILENAME));
	ofn.lStructSize		= sizeof(OPENFILENAME);
	ofn.hwndOwner		= data->dlg;
	ofn.lpstrFilter		= "Binary Files (*.bin)\0*.bin\0"
							"All (*.*)\0*.*\0";
	ofn.nFilterIndex	= 1;
	ofn.lpstrFile		= name;
	ofn.nMaxFile		= sizeof(name);
	ofn.lpstrInitialDir	= dir;
	ofn.lpstrTitle		= "Save Receive Data";
	ofn.Flags			= OFN_PATHMUSTEXIST
						| OFN_OVERWRITEPROMPT
						| OFN_HIDEREADONLY;
	u32					= (U32) GetSaveFileName(&ofn);

	for (; u32;)
	{
		cursor	= SetCursor(data->device->cursor.wait);
		file	= fopen(name, "wb");

		if (file == NULL)
		{
			sprintf(data->status_local, "Unable to create '%s'.", name);
			SetCursor(cursor);
			break;
		}

		u32		= (U32) fwrite(data->app_buf, 1, data->data.count, file);

		if (u32 != data->data.count)
		{
			sprintf(data->status_local, "Write failure to '%s'.", name);
		}
		else
		{
			sprintf(data->status_local,
					"Saved %ld bytes to '%s'.",
					(long) data->data.count,
					name);
		}

		SetCursor(cursor);
		fclose(file);
		break;
	}
}



/*****************************************************************************
*
*	Function:	_xfer_progress
*
*	Purpose:
*
*		Update the progress of the read operation.
*
*	Arguments:
*
*		data	The dialog specific data we track.
*
*	Returned:
*
*		TRUE	Stop updating the progress.
*		FALSE	Continue updating the progress.
*
*****************************************************************************/

static int _xfer_progress(data_xfer_t* data)
{
	char		buf[64];
	char		buf1[32];
	char		buf2[32];
	float		bytes;
	long		elapsed;
	const char*	msg;
	float		rate;
	int			stop;
	data_xfer_t	tmp	= data[0];

	if (tmp.status != GSC_SUCCESS)
	{
		elapsed	= (long) tmp.time_stop - (long) tmp.time_start;
		msg		= buf;
		stop	= TRUE;
		sprintf(buf, "Error 0x%lX. ", (long) tmp.status);
	}
	else if (tmp.abort)
	{
		elapsed	= (long) tmp.time_stop - (long) tmp.time_start;
		msg		= "Read aborted. ";
		stop	= TRUE;
	}
	else if (tmp.active == FALSE)
	{
		elapsed	= (long) tmp.time_stop - (long) tmp.time_start;
		msg		= "";
		stop	= TRUE;
	}
	else
	{
		elapsed	= (long) timeGetTime() - (long) tmp.time_start;
		msg		= "Reading ... ";
		stop	= FALSE;
	}

	bytes	= (float) tmp.xfered_tot;
	rate	= (float) (1000.0 * bytes / elapsed);
	format_float(bytes, buf1);
	format_float(rate, buf2);
	sprintf(data->status_local,
			"%sRead %lu bytes in %ld.%03ld s => %sB at %sB/S.",
			msg,
			(long) tmp.xfered_tot,
			elapsed / 1000,
			elapsed % 1000,
			buf1,
			buf2);
	return(stop);
}



/*****************************************************************************
*
*	Function:	_service_init
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_INIT service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_init(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static const align_t list[]	=
	{
		{
		/* id		*/	IDC_RX_RECEIVE,
		/* ref		*/	IDC_TX_TRANSMIT,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{
		/* id		*/	IDC_RX_RECEIVE,
		/* ref		*/	IDC_RX_CLOCK,
		/* flags	*/	ALIGN_T_B | ALIGN_H,
		/* value	*/	MARGIN
		},
		{
		/* id		*/	IDC_RX_RECEIVE_ARROW,
		/* ref		*/	IDC_TX_TRANSMIT_ARROW,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{
		/* id		*/	IDC_RX_RECEIVE_ARROW,
		/* ref		*/	IDC_RX_RECEIVE,
		/* flags	*/	ALIGN_T_T | ALIGN_B_B,
		/* value	*/	0
		},
		{ 0, 0, 0, 0 }
	};

	align_control_list(dlg, list);
	button_init(dlg, device, IDC_RX_RECEIVE);
	arrow_init(dlg, device, IDC_RX_RECEIVE_ARROW);
	arrow_type_set(dlg, IDC_RX_RECEIVE_ARROW, ARROW_LEFT_2);
	return(0);
}



/*****************************************************************************
*
*	Function:	_service_refresh
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_REFRESH service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_refresh(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	char	buf[16];
	int		enabled	= FALSE;
	int		fifo	= FALSE;
	U32		status;
	int		width;

	for (;;)	// A convenience loop.
	{
		if (device->handle == NULL)
		{
			// The device is closed.
			ctrl_menu_set(dlg, IDC_RX_RECEIVE, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_RX_RECEIVE_ARROW, CTRL_DISABLE | CTRL_NULL);
			break;
		}

		// The device is open.
		status	= _get_state_info(device, &enabled, &fifo, &width);
		error_status(dlg, IDC_INFO_TEXT, status);

		if ((device->handle) && (fifo))
		{
			// The device has Rx FIFOs.
			sprintf(buf, "%d-bits", width);
			ctrl_menu_set(dlg, IDC_RX_RECEIVE, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_RX_RECEIVE_ARROW, CTRL_ENABLE);
			ctrl_menu_set_text(dlg, IDC_RX_RECEIVE_ARROW, buf);
			break;
		}

		// The device has no Rx FIFOs.
		ctrl_menu_set(dlg, IDC_RX_RECEIVE, CTRL_DISABLE);
		ctrl_menu_set(dlg, IDC_RX_RECEIVE_ARROW, CTRL_DISABLE | CTRL_NULL);
		break;
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	_service_wm_command
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_WM_COMMAND service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_wm_command(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static	data_xfer_t	data;

	LRESULT	ret;

	data.amount_key		= "amount";
	data.section		= "rx receive";
	data.title			= "Data Read";
	data.which			= HPDI32_WHICH_RX;

	data.refresh_window	= _refresh_window;
	data.wm_initdialog	= _wm_initdialog;
	data.xfer_begin		= _xfer_begin;
	data.xfer_data		= _xfer_data;
	data.xfer_end		= _xfer_end;
	data.xfer_file		= _xfer_file;
	data.xfer_progress	= _xfer_progress;

	ret	= data_xfer_service(dlg, device, SERVICE_WM_COMMAND, (ULONG) &data, lParam);
	return(ret);
}



/*****************************************************************************
*
*	Function:	rx_receive_service
*
*	Purpose:
*
*		Implement the common service entry point for the Rx Receive services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT rx_receive_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service				function			*/
		{ SERVICE_INIT,			_service_init		},
		{ SERVICE_REFRESH,		_service_refresh	},
		{ SERVICE_WM_COMMAND,	_service_wm_command	},
		{ SERVICE_END_LIST,		NULL				}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}


