// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/tx_cd.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement support for the Tx Clock Divider configuration options.
*
*****************************************************************************/

#include <windows.h>
#include <stdio.h>

#include "tx.h"
#include "align.h"
#include "arrow.h"
#include "button.h"
#include "copy.h"
#include "gui.h"
#include "preference.h"
#include "resource.h"
#include "util_ctrl.h"



/* #defines	*****************************************************************/

#define	SECTION_NAME	"tx clock divider"



/* prototypes	*************************************************************/

static U32	_cfg_wiz_tx_clock_divider(
	HWND				dlg,
	device_t*			device,
	const parm_entry_t*	entry,
	U32					value);





/* variables	*************************************************************/

static const value_range_t	_clock_divider_value_range	=
{
	/* min	*/	0,
	/* max	*/	HPDI32_TX_CLOCK_DIVIDER_MAX,
	/* k	*/	1000
};

static const value_t	_clock_divider_value	=
{
	/* type	*/	VALUE_TYPE_RANGE,
	{
	/* u.vp	*/	&_clock_divider_value_range
	}
};

const parm_entry_t	tx_parm_clock_divider	=
{
	/* title			*/	"Clock Divider",
	/* parm + parm_text	*/	VAR_STR_MACRO(HPDI32_TX_CLOCK_DIVIDER),
	/* suffix			*/	"",
	/* which			*/	0,
	/* set_arg			*/	"",
	/* get_arg			*/	"",
	/* values			*/	&_clock_divider_value,
	/* cfg_wiz			*/	_cfg_wiz_tx_clock_divider,
	/* desc				*/
	"This parameter controls the degree to which the onboard oscillator is"
	" divided before being used by the transmitter and fed to the Cable Clock"
	" signal. A divisor of zero (0) or one (1) produces no division. Any"
	" other divisor produces a 2X reduction."
};



/*****************************************************************************
*
*	Function:	_cfg_wiz_tx_clock_divider
*
*	Purpose:
*
*		Provide customized processing for the Configuration Wizards processing
*		of the Tx Clock Divider configuration option.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		device		The device structure to access.
*
*		entry		The structure for the parameter to process.
*
*		value		The current value for the parameter.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _cfg_wiz_tx_clock_divider(
	HWND				dlg,
	device_t*			device,
	const parm_entry_t*	entry,
	U32					value)
{
	char	buf_1[128];
	char	buf_2[128];
	HWND	control;

	control	= GetDlgItem(dlg, IDC_CFG_WIZ_DATA_EDIT_STATUS);
	SendMessage(control, WM_GETTEXT, sizeof(buf_1), (LPARAM) buf_1);

	if (value < 1)
		strcpy(buf_2, "/1 ");
	else
		sprintf(buf_2, "/%lu ", (long) value * 2);

	strcat(buf_2, buf_1);
	SetWindowText(control, buf_2);
	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	_divider_device_read
*
*	Purpose:
*
*		Read the parameter from the device into memory.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _divider_device_read(tx_cd_t* data)
{
	U32	status;

	status	= HPDI32_TX_CLOCK_DIVIDER__GET(	data->device->handle,
											&data->divider.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_divider_device_write
*
*	Purpose:
*
*		Write the parameter held in memory to the device.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _divider_device_write(tx_cd_t* data)
{
	U32	status;

	status	= HPDI32_TX_CLOCK_DIVIDER__SET(	data->device->handle,
											data->divider.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_divider_dialog_read
*
*	Purpose:
*
*		Read the parameter from the dialog into memory.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _divider_dialog_read(HWND dlg, tx_cd_t* data)
{
	U32	status;

	status	= ctrl_text_2_u32(	dlg,
								IDC_CD_DIVIDER_EDIT,
								_clock_divider_value_range.k,
								&data->divider.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_divider_dialog_write
*
*	Purpose:
*
*		Write the parameter held in memory to the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _divider_dialog_write(HWND dlg, tx_cd_t* data)
{
	char			edit[32];
	U32				divider;
	unsigned int	flags;
	char			info[32];

	if (data->device->handle)
	{
		if (data->divider.setting < 1)
			divider	= 1;
		else
			divider	= data->divider.setting * 2;

		flags	= CTRL_ENABLE;
		sprintf(edit, "0x%lX", (long) data->divider.setting);
		sprintf(info, "(/%lu)", (long) divider);
	}
	else
	{
		flags	= CTRL_DISABLE;
		edit[0]	= 0;
		info[0]	= 0;
	}

	ctrl_menu_set(dlg, IDC_CD_DIVIDER_TITLE, flags);
	ctrl_menu_set(dlg, IDC_CD_DIVIDER_EDIT, flags);
	ctrl_menu_set(dlg, IDC_CD_DIVIDER_INFO, flags);
	ctrl_menu_set_text(dlg, IDC_CD_DIVIDER_INFO, info);

	if (data->divider.ignore == FALSE)
	{
		data->divider.ignore	= TRUE;
		ctrl_menu_set_text(dlg, IDC_CD_DIVIDER_EDIT, edit);
		data->divider.ignore	= FALSE;
	}

	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	tx_clock_divider_rate_dialog_write
*
*	Purpose:
*
*		Write the parameter held in memory to the dialog.
*		In this case, we merely inform the user of the expected clock rate.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 tx_clock_divider_rate_dialog_write(HWND dlg, tx_cd_t* data)
{
	char			buf[32];
	U32				divider;
	unsigned int	flags;
	U32				rate;

	if (data->device->handle)
	{
		if (data->divider.setting < 1)
			divider	= 1;
		else
			divider	= data->divider.setting * 2;

		flags	= CTRL_ENABLE;
		rate	= data->oscillator.setting / divider;
		format_u32(rate, TX_RATE_K, buf);
		strcat(buf, "Hz");
	}
	else
	{
		flags	= CTRL_DISABLE;
		buf[0]	= 0;
	}

	ctrl_menu_set(dlg, IDC_CD_RATE_TITLE, flags);
	ctrl_menu_set(dlg, IDC_CD_RATE_TEXT, flags);
	ctrl_menu_set(dlg, IDC_CD_RATE_INFO, flags);
	ctrl_menu_set_text(dlg, IDC_CD_RATE_TEXT, buf);

	return(GSC_SUCCESS);
}


/*****************************************************************************
*
*	Function:	_device_read
*
*	Purpose:
*
*		Read the parameters from the device into memory.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _device_read(tx_cd_t* data)
{
	U32	stat;
	U32	status	= GSC_SUCCESS;

	stat	= tx_oscillator_cd_device_read(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _divider_device_read(data);
	status	= (status == GSC_SUCCESS) ? stat : status;

	return(status);
}



/*****************************************************************************
*
*	Function:	_device_write
*
*	Purpose:
*
*		Write the parameters held in memory to the device.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _device_write(tx_cd_t* data)
{
	U32	stat;
	U32	status	= GSC_SUCCESS;

	stat	= tx_oscillator_cd_device_write(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _divider_device_write(data);
	status	= (status == GSC_SUCCESS) ? stat : status;

	return(status);
}



/*****************************************************************************
*
*	Function:	_dialog_read
*
*	Purpose:
*
*		Read the parameters from the dialog into memory.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _dialog_read(HWND dlg, tx_cd_t* data)
{
	U32	stat;
	U32	status	= GSC_SUCCESS;

	stat	= tx_oscillator_cd_dialog_read(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _divider_dialog_read(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;

	return(status);
}



/*****************************************************************************
*
*	Function:	_dialog_write
*
*	Purpose:
*
*		Write the parameters held in memory to the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _dialog_write(HWND dlg, tx_cd_t* data)
{
	U32	stat;
	U32	status	= GSC_SUCCESS;

	stat	= tx_oscillator_cd_dialog_write(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _divider_dialog_write(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= tx_clock_divider_rate_dialog_write(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;

	return(status);
}



/*****************************************************************************
*
*	Function:	_refresh_window
*
*	Purpose:
*
*		Update the state of the window.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		msg		A status message we're to report.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _refresh_window(HWND dlg, device_t* device, const char* msg)
{
	tx_cd_t*		data;
	unsigned int	flags;
	HMENU			menu	= GetMenu(dlg);
	U32				stat;
	U32				status	= GSC_SUCCESS;
	char			title[64];

	data	= (tx_cd_t*) GET_WINDOW_USER_DATA(dlg);

	data->changes	= device->handle ? data->changes : FALSE;
	strcpy(title, "Tx Clock Divider");
	strcat(title, data->changes ? " *" : "");
	set_win_title(dlg, device, FALSE, TRUE, title);

	if (data->changes)
	{
		ModifyMenu(menu, IDCANCEL, MF_BYCOMMAND | MF_STRING, IDCANCEL, "&Cancel");
		SetDlgItemText(dlg, IDCANCEL, "Cancel");
	}
	else
	{
		ModifyMenu(menu, IDCANCEL, MF_BYCOMMAND | MF_STRING, IDCANCEL, "&Close");
		SetDlgItemText(dlg, IDCANCEL, "Close");
	}

	if (device->handle)
	{
		if ((data->was_open == FALSE) || (data->changes == FALSE))
		{
			// Update everything from the device.
			stat	= _device_read(data);
			status	= (status == GSC_SUCCESS) ? stat : status;
			stat	= _dialog_write(dlg, data);
			status	= (status == GSC_SUCCESS) ? stat : status;
		}

		flags			= CTRL_ENABLE;
		data->was_open	= TRUE;
	}
	else
	{
		// Everything is essentially disabled.
		flags			= CTRL_DISABLE;
		data->changes	= FALSE;
		data->was_open	= FALSE;
		stat	= _dialog_write(dlg, data);
		status	= (status == GSC_SUCCESS) ? stat : status;
		ctrl_menu_set(dlg, IDCANCEL, CTRL_FOCUS);
	}

	ctrl_menu_set(dlg, IDOK, flags);
	ctrl_menu_set(dlg, IDC_APPLY, flags);
	ctrl_menu_set(dlg, IDC_DEFAULT, flags);
	error_status(dlg, 0, status);
}



/*****************************************************************************
*
*	Function:	_idc_apply
*
*	Purpose:
*
*		Respond to an APPLY request.
*
*	Arguments:
*
*		dlg			The dalog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_apply(HWND dlg, tx_cd_t* data)
{
	U32	stat;
	U32	status	= GSC_SUCCESS;

	data->changes	= FALSE;

	stat	= _dialog_read(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _device_write(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _device_read(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _dialog_write(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;

	main_refresh(data->device, NULL);
	error_status(dlg, 0, status);
}



/*****************************************************************************
*
*	Function:	_idc_copy
*
*	Purpose:
*
*		Implement a COPY function for the dialog content.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_copy(HWND dlg)
{
	static const copy_t	list[]	=
	{
		/* id						flags									*/
		{ -1,						COPY_EOL | COPY_UNDERLINE				},

		{ IDC_CD_OSCILLATOR_TITLE,	COPY_WIDTH | 12							},
		{ IDC_CD_OSCILLATOR_EDIT,	0										},
		{ IDC_CD_OSCILLATOR_INFO,	COPY_EOL | COPY_DISABLED				},

		{ IDC_CD_DIVIDER_TITLE,		COPY_WIDTH | 12							},
		{ IDC_CD_DIVIDER_EDIT,		0										},
		{ IDC_CD_DIVIDER_INFO,		COPY_INDENT | COPY_EOL | COPY_DISABLED	},

		{ IDC_CD_RATE_TITLE,		COPY_WIDTH | 12							},
		{ IDC_CD_RATE_TEXT,			0										},
		{ IDC_CD_RATE_INFO,			COPY_INDENT | COPY_EOL | COPY_DISABLED	},

		{ 0,			COPY_UNDERLINE										},
		{ 0,			COPY_EOL											},

		{ IDOK,			COPY_DISABLED | COPY_EOL | COPY_WIDTH | 8			},
		{ IDCANCEL,		COPY_DISABLED | COPY_EOL | COPY_WIDTH | 8			},
		{ IDC_APPLY,	COPY_DISABLED | COPY_EOL | COPY_WIDTH | 8			},
		{ IDC_DEFAULT,	COPY_DISABLED | COPY_EOL | COPY_WIDTH | 8			}
	};

	copy_list_to_clipboard(dlg, list, ARRAY_ELEMENTS(list));
}



/*****************************************************************************
*
*	Function:	_idc_default
*
*	Purpose:
*
*		Process clicking of the DEFAULT button.
*
*	Arguments:
*
*		dlg			The dalog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_default(HWND dlg, tx_cd_t* data)
{
	U32	status;

	data->changes	= TRUE;

	data->oscillator.setting	= data->oscillator.defalt;
	data->divider.setting		= data->divider.defalt;

	status	= _dialog_write(dlg, data);

	_refresh_window(dlg, data->device, NULL);
	error_status(dlg, 0, status);
}



/*****************************************************************************
*
*	Function:	_idcancel
*
*	Purpose:
*
*		Respond to a request to close the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idcancel(HWND dlg)
{
	tx_cd_t*	data;

	main_refresh_add(dlg, NULL);
	data	= (tx_cd_t*) GET_WINDOW_USER_DATA(dlg);

	if (data)
		data->dlg	= NULL;

	win_pos_save(dlg, SECTION_NAME);
	DestroyWindow(dlg);
}



/*****************************************************************************
*
*	Function:	_idok
*
*	Purpose:
*
*		Process clicking of the OK button.
*
*	Arguments:
*
*		dlg			The dalog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idok(HWND dlg, tx_cd_t* data)
{
	_idc_apply(dlg, data);
	_idcancel(dlg);
}



/*****************************************************************************
*
*	Function:	_idc_cd_divisor
*
*	Purpose:
*
*		Respond to activity in the DIVISOR edit control.
*
*	Arguments:
*
*		dlg			The dalog handle.
*
*		wParam		The message's WORD sized parameter.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_cd_divisor(HWND dlg, WPARAM wParam, tx_cd_t* data)
{
	int	code	= HIWORD(wParam);
	U32	stat;
	U32	status;

	if ((code == EN_CHANGE) && (data->divider.ignore == FALSE))
	{
		status			= GSC_SUCCESS;
		data->changes	= TRUE;

		stat	= _divider_dialog_read(dlg, data);
		status	= (status == GSC_SUCCESS) ? stat : status;

		data->divider.ignore	= TRUE;
		stat	= _divider_dialog_write(dlg, data);
		status	= (status == GSC_SUCCESS) ? stat : status;
		data->divider.ignore	= FALSE;

		stat	= tx_clock_divider_rate_dialog_write(dlg, data);
		status	= (status == GSC_SUCCESS) ? stat : status;

		_refresh_window(dlg, data->device, NULL);
		error_status(dlg, 0, status);
	}
}



/*****************************************************************************
*
*	Function:	_wm_command
*
*	Purpose:
*
*		Process WM_COMMAND messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's WORD parameter.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_command(HWND dlg, WPARAM wParam)
{
	tx_cd_t*	data	= (tx_cd_t*) GET_WINDOW_USER_DATA(dlg);
	int			id		= LOWORD(wParam);
	LRESULT		ret		= TRUE;

	switch (id)
	{
		default:

			ret	= FALSE;
			break;

		case IDC_APPLY:

			_idc_apply(dlg, data);
			break;

		case IDC_CD_DIVIDER_EDIT:

			_idc_cd_divisor(dlg, wParam, data);
			break;

		case IDC_CD_OSCILLATOR_EDIT:

			tx_oscillator_cd_idc_cd_oscillator(dlg, wParam, data);
			_refresh_window(dlg, data->device, NULL);
			break;

		case IDC_COPY:

			_idc_copy(dlg);
			break;

		case IDC_DEFAULT:

			_idc_default(dlg, data);
			break;

		case IDCANCEL:

			_idcancel(dlg);
			break;

		case IDOK:

			_idok(dlg, data);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_wm_destroy
*
*	Purpose:
*
*		Process WM_DESTROY messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_destroy(HWND dlg)
{
	_idcancel(dlg);
	DestroyWindow(dlg);
	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_initdialog
*
*	Purpose:
*
*		Process a WM_INITDIALOG message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		lParam		The message's long sized argument.
*
*	Returned:
*
*		TRUE		Windows should set the initial control focus.
*		FALSE		Windows should NOT set the initial control focus.
*
*****************************************************************************/

static LRESULT _wm_initdialog(HWND dlg, LPARAM lParam)
{
	char		buf[32];
	HWND		control;
	tx_cd_t*	data	= (tx_cd_t*) lParam;

	SET_WINDOW_USER_DATA(dlg, lParam);

	SendMessage(dlg, WM_SETICON, ICON_SMALL, (LPARAM) data->device->icon.little);
	SendMessage(dlg, WM_SETICON, ICON_BIG, (LPARAM) data->device->icon.big);

	control	= GetDlgItem(dlg, IDC_CD_DIVIDER_EDIT);
	SendMessage(control, EM_LIMITTEXT, 12, 0);

	sprintf(buf, "(1K = %d)", (int) TX_RATE_K);
	ctrl_menu_set_text(dlg, IDC_CD_RATE_INFO, buf);

	win_pos_restore(dlg, SECTION_NAME, FALSE);
	main_refresh_add(dlg, _refresh_window);
	tx_oscillator_cd_wm_initdialog(dlg);
	return(TRUE);
}



/*****************************************************************************
*
*	Function:	_wm_syscommand
*
*	Purpose:
*
*		Process a WM_SYSCOMMAND message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's word sized parameter.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_syscommand(HWND dlg, WPARAM wParam)
{
	LRESULT	ret;

	switch (wParam)
	{
		default:

			ret	= FALSE;
			break;

		case SC_CLOSE:

			_idcancel(dlg);
			ret	= TRUE;
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_dlg_proc
*
*	Purpose:
*
*		Processes messages for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		msg			The message to process.
*
*		wParam		The message's word sized parameter.
*
*		lParam		The message's long sized parameter.
*
*	Returned:
*
*		A message specific value.
*
*****************************************************************************/

static INT_PTR CALLBACK _dlg_proc(	HWND	dlg,
									UINT	msg,
									WPARAM	wParam,
									LPARAM	lParam)
{
	LRESULT	ret;

	switch (msg)
	{
		default:

			ret	= FALSE;
			break;

		case WM_COMMAND:

			ret	= _wm_command(dlg, wParam);
			break;

		case WM_DESTROY:

			ret	= _wm_destroy(dlg);
			break;

		case WM_INITDIALOG:

			ret	= _wm_initdialog(dlg, lParam);
			break;

		case WM_SYSCOMMAND:

			ret	= _wm_syscommand(dlg, wParam);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_service_init
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_INIT service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_init(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static const align_t list[]	=
	{
		{
		/* id		*/	IDC_TX_CD,
		/* ref		*/	IDC_TX_OSCILLATOR_ARROW,
		/* flags	*/	ALIGN_T_T | ALIGN_B_B | ALIGN_L_R | ALIGN_W,
		/* value	*/	0
		},
		{
		/* id		*/	IDC_TX_CD_ARROW,
		/* ref		*/	IDC_TX_CD,
		/* flags	*/	ALIGN_T_T | ALIGN_B_B | ALIGN_L_R | ALIGN_W,
		/* value	*/	0
		},
		{ 0, 0, 0, 0 }
	};

	align_control_list(dlg, list);
	button_init(dlg, device, IDC_TX_CD);
	arrow_init(dlg, device, IDC_TX_CD_ARROW);
	arrow_type_set(dlg, IDC_TX_CD_ARROW, ARROW_RIGHT);
	return(0);
}



/*****************************************************************************
*
*	Function:	_service_refresh
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_REFRESH service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_refresh(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	char	buf[16];
	size_t	len;
	char*	psz;
	U32		status;
	U32		u32;

	for (;;)	// A convenience loop.
	{
		if (device->handle == NULL)
		{
			// The device is closed.
			ctrl_menu_set(dlg, IDC_TX_CD, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_TX_CD_ARROW, CTRL_DISABLE | CTRL_NULL);
			break;
		}

		// The device is open.
		status	= HPDI32_TX_CLOCK_DIVIDER__GET(device->handle, &u32);
		error_status(dlg, IDC_INFO_TEXT, status);

		if (u32 < 1)
			u32	= device->oscillator;
		else
			u32	= device->oscillator / (u32 * 2);

		format_u32(u32, TX_RATE_K, buf);
		len	= strlen(buf);

		if (len > 5)
		{
			format_float((float) u32, buf);
			psz	= strchr(buf, ' ');

			if (psz)
			{
				psz[0]	= psz[1];
				psz[1]	= 0;
			}
		}

		strcat(buf, "Hz");
		ctrl_menu_set(dlg, IDC_TX_CD, CTRL_ENABLE);
		ctrl_menu_set(dlg, IDC_TX_CD_ARROW, CTRL_ENABLE);
		ctrl_menu_set_text(dlg, IDC_TX_CD_ARROW, buf);
		break;
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	_service_wm_command
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_WM_COMMAND service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_wm_command(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static	tx_cd_t	data;

	if (data.dlg)	/*lint !e727 */
	{
		SetFocus(data.dlg);
	}
	else
	{
		data.changes			= FALSE;
		data.device				= device;
		data.divider.defalt		= HPDI32_TX_CLOCK_DIVIDER_DEFAULT;
		data.divider.ignore		= TRUE;
		data.oscillator.defalt	= PREF_OSCILLATOR_RATE__GET();
		data.oscillator.ignore	= TRUE;
		data.was_open			= FALSE;
		data.dlg	= CreateDialogParam(device->instance,
										(LPCTSTR) IDD_CD,
										dlg,
										(DLGPROC) _dlg_proc,
										(LPARAM) &data);
		ShowWindow(data.dlg, SW_SHOWNORMAL);
		data.divider.ignore		= FALSE;
		data.oscillator.ignore	= FALSE;
		_refresh_window(data.dlg, device, NULL);
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	tx_clock_divider_service
*
*	Purpose:
*
*		Implement the common service entry point for the Tx Clock Divider
*		services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT tx_clock_divider_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service				function			*/
		{ SERVICE_INIT,			_service_init		},
		{ SERVICE_REFRESH,		_service_refresh	},
		{ SERVICE_WM_COMMAND,	_service_wm_command	},
		{ SERVICE_END_LIST,		NULL				}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}


