// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/tx_df.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement support for the Tx Data Flow configuration options.
*
*****************************************************************************/

#include <windows.h>

#include "tx.h"
#include "arrow.h"
#include "cable.h"
#include "copy.h"
#include "gui.h"
#include "resource.h"
#include "util_ctrl.h"



/* #defines	*****************************************************************/

#define	SECTION_NAME		"tx data flow"



/* prototypes	*************************************************************/

static	U32	_auto_stop_cfg_wiz(	HWND				dlg,
								device_t*			device,
								const parm_entry_t*	entry,
								U32					value);



/* typedefs	*****************************************************************/

typedef struct
{
	U32		defalt;		// MUST BE SET BY CODE WANTING THE DIALOG!
	U32		setting;	// The device/dialog setting is recorded here.
} _df_t;

typedef struct
{
	_df_t		auto_start;
	_df_t		auto_stop;
	U32			auto_stop_available;
	int			changes;	// Are there changes to apply?
	device_t*	device;
	HWND		dlg;
	_df_t		mode;
	_df_t		rt_enable;
	int			was_open;	// Was the device already open?
} df_t;



/* variables	*************************************************************/

static const value_list_entry_t	_auto_start_value_list[]	=
{
	{
	/* user_text			*/	"Yes (The API will initiate data transmission."
								" This offers better performance.)",
	/* value + value_text	*/	VAR_STR_MACRO(HPDI32_TX_AUTO_START_YES),
	/* suffix				*/	"_YES"
	},
	{
	/* user_text			*/	"No (The application must initiate data"
								" transmission. This offers more control.)",
	/* value + value_text	*/	VAR_STR_MACRO(HPDI32_TX_AUTO_START_NO),
	/* suffix				*/	"_NO"
	}
};

static const value_list_t	_auto_start_values[]	=
{
	{
	/* list	*/	_auto_start_value_list,
	/* size	*/	ARRAY_ELEMENTS(_auto_start_value_list)
	}
};

static const value_t	_auto_start_value	=
{
	/* type	*/	VALUE_TYPE_LIST,
	{
	/* u.vp	*/	_auto_start_values
	}
};

const parm_entry_t	tx_parm_auto_start	=
{
	/* title			*/	"Auto Start",
	/* parm + parm_text	*/	VAR_STR_MACRO(HPDI32_TX_AUTO_START),
	/* suffix			*/	"",
	/* which			*/	0,
	/* set_arg			*/	"",
	/* get_arg			*/	"",
	/* values			*/	&_auto_start_value,
	/* cfg_wiz			*/	NULL,
	/* desc				*/
	"This parameter controls who initiates data transmission from the Tx"
	" FIFO. If Auto Start is enabled, then the API will initiate"
	" transmission during the hpdi32_write() call. If disabled, then the"
	" application is responsible for initiating transmission by setting the"
	" Board Control Register's Tx Start bit. If disabled, then hpdi32_write()"
	" calls may timeout if the Tx FIFO become full during the call."
	" NOTE: Enabling this parameter generally provides better throughput"
	" performance."
};

static const value_list_entry_t	_auto_stop_value_list[]	=
{
	{
	/* user_text			*/	"No (This offers the best throughput as"
								" transmission resumes automatically as more"
								" data enters the Tx FIFO.)",
	/* value + value_text	*/	VAR_STR_MACRO(HPDI32_TX_AUTO_STOP_NO),
	/* suffix				*/	"_NO"
	},
	{
	/* user_text			*/	"Yes (Transmission will halt as soon as the"
								" Tx FIFO empties, even if only momentarily.)",
	/* value + value_text	*/	VAR_STR_MACRO(HPDI32_TX_AUTO_STOP_YES),
	/* suffix				*/	"_YES"
	}
};

static const value_list_t	_auto_stop_values[]	=
{
	{
	/* list	*/	_auto_stop_value_list,
	/* size	*/	ARRAY_ELEMENTS(_auto_stop_value_list)
	}
};

static const value_t	_auto_stop_value	=
{
	/* type	*/	VALUE_TYPE_LIST,
	{
	/* u.vp	*/	_auto_stop_values
	}
};

const parm_entry_t	tx_parm_auto_stop	=
{
	/* title			*/	"Auto Stop",
	/* parm + parm_text	*/	VAR_STR_MACRO(HPDI32_TX_AUTO_STOP),
	/* suffix			*/	"",
	/* which			*/	0,
	/* set_arg			*/	"",
	/* get_arg			*/	"",
	/* values			*/	&_auto_stop_value,
	/* cfg_wiz			*/	_auto_stop_cfg_wiz,
	/* desc				*/
	"This parameter controls how ongoing data transmission is affected by the"
	" Tx FIFO becoming empty. If Auto Stop is disabled when the Tx FIFO"
	" becomes empty, then transmission will resume as soon as more data"
	" enters the FIFO. This option generally offers the best throughput"
	" performance. If enabled, then transmission will halt, even if the FIFO"
	" is empty only momentarily. When this occurs, transmission must be"
	" reinitiated. (This is handled automatically by the API when Auto Start"
	" is enabled.)"
	" NOTE: Demand Mode DMA writes may unexpectedly timeout if Auto Stop is"
	" enabled."
};

static const value_list_entry_t	_remote_throttle_value_list[]	=
{
	{
	/* user_text			*/	"Disable (The Rx Ready cable signal does not control data transmission.)",
	/* value + value_text	*/	VAR_STR_MACRO(HPDI32_TX_REMOTE_THROTTLE_DISABLE),
	/* suffix				*/	"_DISABLE"
	},
	{
	/* user_text			*/	"Enable (The Rx Ready cable signal controls data transmission.)",
	/* value + value_text	*/	VAR_STR_MACRO(HPDI32_TX_REMOTE_THROTTLE_ENABLE),
	/* suffix				*/	"_ENABLE"
	}
};

static const value_list_t	_remote_throttle_values[]	=
{
	{
	/* list	*/	_remote_throttle_value_list,
	/* size	*/	ARRAY_ELEMENTS(_remote_throttle_value_list)
	}
};

static const value_t	_remote_throttle_value	=
{
	/* type	*/	VALUE_TYPE_LIST,
	{
	/* u.vp	*/	_remote_throttle_values
	}
};

const parm_entry_t	tx_parm_remote_throttle	=
{
	/* title			*/	"Remote Throttle",
	/* parm + parm_text	*/	VAR_STR_MACRO(HPDI32_TX_REMOTE_THROTTLE),
	/* suffix			*/	"",
	/* which			*/	0,
	/* set_arg			*/	"",
	/* get_arg			*/	"",
	/* values			*/	&_remote_throttle_value,
	/* cfg_wiz			*/	NULL,
	/* desc				*/
	"This parameter controls the transmitter's use of the Rx Ready cable"
	" signal. If Remote Throttle is enabled, then the remote device can drive"
	" this signal to pause or permit data flow. If enabled, then transmission"
	" is paused while the signal is negated, whereas transmission is permitted"
	" while the signal is asserted. This permits the remote device to pause"
	" data flow on an as needed basis. This parameter should be configured"
	" according to the interface needs of the remote device."
};



/*****************************************************************************
*
*	Function:	_auto_start_device_read
*
*	Purpose:
*
*		Read the parameter from the device into memory.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _auto_start_device_read(df_t* data)
{
	U32	status;

	status	= HPDI32_TX_AUTO_START__GET(data->device->handle,
										&data->auto_start.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_auto_start_device_write
*
*	Purpose:
*
*		Apply the current setting to the device.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _auto_start_device_write(df_t* data)
{
	U32	status;

	status	= HPDI32_TX_AUTO_START__SET(data->device->handle,
										data->auto_start.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_auto_start_dialog_read
*
*	Purpose:
*
*		Retrieve the current setting from the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _auto_start_dialog_read(HWND dlg, df_t* data)
{
	HWND	control;
	int		state;

	control	= GetDlgItem(dlg, IDC_DF_AUTO_START);
	state	= (int) SendMessage(control, BM_GETCHECK, 0, 0);

	if (state == BST_CHECKED)
		data->auto_start.setting	= HPDI32_TX_AUTO_START_YES;
	else
		data->auto_start.setting	= HPDI32_TX_AUTO_START_NO;

	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	_auto_start_dialog_write
*
*	Purpose:
*
*		Apply the Remote Throttle Enable setting to the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _auto_start_dialog_write(HWND dlg, df_t* data)
{
	unsigned int	flags;

	if (data->device->handle == NULL)
		flags	= CTRL_DISABLE | CTRL_BM_UNCHECK;
	else if (data->auto_start.setting == HPDI32_TX_AUTO_START_YES)
		flags	= CTRL_ENABLE | CTRL_BM_CHECK;
	else
		flags	= CTRL_ENABLE | CTRL_BM_UNCHECK;

	ctrl_menu_set(dlg, IDC_DF_AUTO_START, flags);
	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	_auto_stop_device_read
*
*	Purpose:
*
*		Read the parameter from the device into memory.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _auto_stop_device_read(df_t* data)
{
	U32	stat;
	U32	status	= GSC_SUCCESS;

	stat	= HPDI32_TX_AUTO_STOP__GET(	data->device->handle,
										&data->auto_stop.setting);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= HPDI32_MISC_FEATURES__TX_AUTO_STOP(	data->device->handle,
													&data->auto_stop_available);
	status	= (status == GSC_SUCCESS) ? stat : status;

	return(status);
}



/*****************************************************************************
*
*	Function:	_auto_stop_device_write
*
*	Purpose:
*
*		Apply the current setting to the device.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _auto_stop_device_write(df_t* data)
{
	U32	status;

	status	= HPDI32_TX_AUTO_STOP__SET(	data->device->handle,
										data->auto_stop.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_auto_stop_dialog_read
*
*	Purpose:
*
*		Retrieve the current setting from the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _auto_stop_dialog_read(HWND dlg, df_t* data)
{
	HWND	control;
	int		state;

	control	= GetDlgItem(dlg, IDC_DF_AUTO_STOP);
	state	= (int) SendMessage(control, BM_GETCHECK, 0, 0);

	if (state == BST_CHECKED)
		data->auto_stop.setting	= HPDI32_TX_AUTO_STOP_YES;
	else
		data->auto_stop.setting	= HPDI32_TX_AUTO_STOP_NO;

	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	_auto_stop_dialog_write
*
*	Purpose:
*
*		Apply the Remote Throttle Enable setting to the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _auto_stop_dialog_write(HWND dlg, df_t* data)
{
	unsigned int	flags;

	if (data->device->handle == NULL)
	{
		flags	= CTRL_DISABLE | CTRL_BM_UNCHECK;
	}
	else
	{
		if (data->auto_stop_available == HPDI32_MISC_FEATURES_ABSENT)
			flags	= CTRL_DISABLE;
		else
			flags	= CTRL_ENABLE;

		if (data->auto_stop.setting == HPDI32_TX_AUTO_STOP_YES)
			flags	|= CTRL_BM_CHECK;
		else
			flags	|= CTRL_BM_UNCHECK;
	}

	ctrl_menu_set(dlg, IDC_DF_AUTO_STOP, flags);
	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	_mode_device_read
*
*	Purpose:
*
*		Read the parameter from the device into memory.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _mode_device_read(df_t* data)
{
	U32	status;

	status	= HPDI32_CABLE_COMMAND_MODE__RR_GET(data->device->handle,
												&data->mode.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_mode_dialog_write
*
*	Purpose:
*
*		Write the parameter held in memory to the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _mode_dialog_write(HWND dlg, df_t* data)
{
	unsigned int	flags;
	const char*		psz;

	if (data->device->handle)
	{
		flags	= CTRL_ENABLE;

		if (data->mode.setting == HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL)
			psz	= "Flow Control";
		else
			psz	= "GPIO";
	}
	else
	{
		flags	= CTRL_DISABLE;
		psz		= "";
	}

	ctrl_menu_set(dlg, IDC_DF_RT_MODE_BUTTON, flags);
	ctrl_menu_set(dlg, IDC_DF_RT_MODE_TEXT, flags);
	ctrl_menu_set_text(dlg, IDC_DF_RT_MODE_TEXT, psz);
	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	_rt_enable_device_read
*
*	Purpose:
*
*		Read the parameter from the device into memory.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _rt_enable_device_read(df_t* data)
{
	U32	status;

	status	= HPDI32_TX_REMOTE_THROTTLE__GET(	data->device->handle,
												&data->rt_enable.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_rt_enable_device_write
*
*	Purpose:
*
*		Apply the current Remote Throttle Enable setting to the device.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _rt_enable_device_write(df_t* data)
{
	U32	status;

	status	= HPDI32_TX_REMOTE_THROTTLE__SET(	data->device->handle,
												data->rt_enable.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_rt_enable_dialog_read
*
*	Purpose:
*
*		Retrieve the current Remote Throttle Enable setting from the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _rt_enable_dialog_read(HWND dlg, df_t* data)
{
	HWND	control;
	int		state;

	control	= GetDlgItem(dlg, IDC_DF_RT_ENABLE_BUTTON);
	state	= (int) SendMessage(control, BM_GETCHECK, 0, 0);

	if (state == BST_CHECKED)
		data->rt_enable.setting	= HPDI32_TX_REMOTE_THROTTLE_ENABLE;
	else
		data->rt_enable.setting	= HPDI32_TX_REMOTE_THROTTLE_DISABLE;

	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	_rt_enable_dialog_write
*
*	Purpose:
*
*		Apply the Remote Throttle Enable setting to the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _rt_enable_dialog_write(HWND dlg, df_t* data)
{
	unsigned int	flags;

	if (data->device->handle == NULL)
		flags	= CTRL_DISABLE | CTRL_BM_UNCHECK;
	else if (data->rt_enable.setting == HPDI32_TX_REMOTE_THROTTLE_ENABLE)
		flags	= CTRL_ENABLE | CTRL_BM_CHECK;
	else
		flags	= CTRL_ENABLE | CTRL_BM_UNCHECK;

	ctrl_menu_set(dlg, IDC_DF_RT_ENABLE_BUTTON, flags);
	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	_device_read
*
*	Purpose:
*
*		Read the parameters from the device into memory.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _device_read(df_t* data)
{
	U32	stat;
	U32	status	= GSC_SUCCESS;

	stat	= _auto_start_device_read(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _auto_stop_device_read(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _mode_device_read(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _rt_enable_device_read(data);
	status	= (status == GSC_SUCCESS) ? stat : status;

	return(status);
}



/*****************************************************************************
*
*	Function:	_device_write
*
*	Purpose:
*
*		Write the parameters held in memory to the device.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _device_write(df_t* data)
{
	U32	stat;
	U32	status	= GSC_SUCCESS;

	stat	= _auto_start_device_write(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _auto_stop_device_write(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _rt_enable_device_write(data);
	status	= (status == GSC_SUCCESS) ? stat : status;

	return(status);
}



/*****************************************************************************
*
*	Function:	_dialog_read
*
*	Purpose:
*
*		Read the parameters from the dialog into memory.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _dialog_read(HWND dlg, df_t* data)
{
	U32	stat;
	U32	status	= GSC_SUCCESS;

	stat	= _auto_start_dialog_read(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _auto_stop_dialog_read(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _rt_enable_dialog_read(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;

	return(status);
}



/*****************************************************************************
*
*	Function:	_dialog_write
*
*	Purpose:
*
*		Write the parameters held in memory to the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _dialog_write(HWND dlg, df_t* data)
{
	U32	stat;
	U32	status	= GSC_SUCCESS;

	stat	= _auto_start_dialog_write(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _auto_stop_dialog_write(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _mode_dialog_write(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _rt_enable_dialog_write(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;

	return(status);
}



/*****************************************************************************
*
*	Function:	_refresh_window
*
*	Purpose:
*
*		Update the state of the window.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		msg		A status message we're to report.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _refresh_window(HWND dlg, device_t* device, const char* msg)
{
	df_t*			data;
	unsigned int	flags;
	HMENU			menu	= GetMenu(dlg);
	U32				stat;
	U32				status	= GSC_SUCCESS;
	char			title[64];

	data	= (df_t*) GET_WINDOW_USER_DATA(dlg);

	data->changes	= device->handle ? data->changes : FALSE;
	strcpy(title, "Tx Data Flow");
	strcat(title, data->changes ? " *" : "");
	set_win_title(dlg, data->device, FALSE, TRUE, title);

	if (data->changes)
	{
		ModifyMenu(menu, IDCANCEL, MF_BYCOMMAND | MF_STRING, IDCANCEL, "&Cancel");
		SetDlgItemText(dlg, IDCANCEL, "Cancel");
	}
	else
	{
		ModifyMenu(menu, IDCANCEL, MF_BYCOMMAND | MF_STRING, IDCANCEL, "&Close");
		SetDlgItemText(dlg, IDCANCEL, "Close");
	}

	if (device->handle)
	{
		if ((data->was_open == FALSE) || (data->changes == FALSE))
		{
			stat	= _device_read(data);
			status	= (status == GSC_SUCCESS) ? stat : status;
			stat	= _dialog_write(dlg, data);
			status	= (status == GSC_SUCCESS) ? stat : status;
		}

		data->was_open	= TRUE;
		flags			= CTRL_ENABLE;
	}
	else
	{
		data->changes	= FALSE;
		data->was_open	= FALSE;
		flags			= CTRL_DISABLE;
		stat	= _dialog_write(dlg, data);
		status	= (status == GSC_SUCCESS) ? stat : status;
	}

	ctrl_menu_set(dlg, IDC_DF_START_STOP, flags);
	ctrl_menu_set(dlg, IDC_DF_REMOTE_THROTTLE, flags);
	ctrl_menu_set(dlg, IDOK, flags);
	ctrl_menu_set(dlg, IDC_APPLY, flags);
	ctrl_menu_set(dlg, IDC_DEFAULT, flags);
	error_status(dlg, 0, status);
}



/*****************************************************************************
*
*	Function:	_idc_apply
*
*	Purpose:
*
*		Respond to an APPLY request.
*
*	Arguments:
*
*		dlg			The dalog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_apply(HWND dlg, df_t* data)
{
	U32	stat;
	U32	status	= GSC_SUCCESS;

	data->changes	= FALSE;

	stat	= _dialog_read(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _device_write(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _device_read(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _dialog_write(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;

	main_refresh(data->device, NULL);
	error_status(dlg, 0, status);
}



/*****************************************************************************
*
*	Function:	_idc_copy
*
*	Purpose:
*
*		Implement a COPY function for the dialog content.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_copy(HWND dlg)
{
	static const copy_t	list[]	=
	{
		/* id						flags														*/
		{ -1,						COPY_EOL | COPY_UNDERLINE									},

		{ IDC_DF_START_STOP,		COPY_EOL | COPY_DISABLED									},
		{ IDC_DF_AUTO_START,		COPY_EOL | COPY_DISABLED | COPY_BM_GETCHECK | COPY_INDENT	},
		{ IDC_DF_AUTO_STOP,			COPY_EOL | COPY_DISABLED | COPY_BM_GETCHECK | COPY_INDENT	},

		{ IDC_DF_REMOTE_THROTTLE,	COPY_EOL | COPY_DISABLED									},
		{ IDC_DF_RT_MODE_BUTTON,	COPY_WIDTH | 8 | COPY_INDENT								},
		{ IDC_DF_RT_MODE_TEXT,		COPY_EOL | COPY_DISABLED									},
		{ IDC_DF_RT_ENABLE_BUTTON,	COPY_EOL | COPY_DISABLED | COPY_BM_GETCHECK | COPY_INDENT	},

		{ 0,						COPY_UNDERLINE												},
		{ 0,						COPY_EOL													},

		{ IDOK,						COPY_DISABLED | COPY_EOL | COPY_WIDTH | 8					},
		{ IDCANCEL,					COPY_DISABLED | COPY_EOL | COPY_WIDTH | 8					},
		{ IDC_APPLY,				COPY_DISABLED | COPY_EOL | COPY_WIDTH | 8					},
		{ IDC_DEFAULT,				COPY_DISABLED | COPY_EOL | COPY_WIDTH | 8					}
	};

	copy_list_to_clipboard(dlg, list, ARRAY_ELEMENTS(list));
}



/*****************************************************************************
*
*	Function:	_idc_default
*
*	Purpose:
*
*		Respond to a DEFAULT request.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_default(HWND dlg, df_t* data)
{
	U32	status;

	data->changes	= TRUE;

	data->auto_start.setting	= data->auto_start.defalt;
	data->auto_stop.setting		= data->auto_stop.defalt;
	data->rt_enable.setting		= data->rt_enable.defalt;

	status	= _dialog_write(dlg, data);

	_refresh_window(dlg, data->device, NULL);
	error_status(dlg, 0, status);
}



/*****************************************************************************
*
*	Function:	_idc_rt_auto_start
*
*	Purpose:
*
*		Respond to activity on the Auto Start control.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_rt_auto_start(HWND dlg, df_t* data)
{
	HWND	control;
	int		state;

	control	= GetDlgItem(dlg, IDC_DF_AUTO_START);
	state	= (int) SendMessage(control, BM_GETCHECK, 0, 0);

	if (state == BST_CHECKED)
		ctrl_menu_set(dlg, IDC_DF_RT_ENABLE_BUTTON, CTRL_BM_UNCHECK);

	data->changes	= TRUE;
	_refresh_window(dlg, data->device, NULL);
}



/*****************************************************************************
*
*	Function:	_idc_rt_auto_stop
*
*	Purpose:
*
*		Respond to activity on the Auto Stop control.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_rt_auto_stop(HWND dlg, df_t* data)
{
	data->changes	= TRUE;
	_refresh_window(dlg, data->device, NULL);
}



/*****************************************************************************
*
*	Function:	_idc_rt_enable_button
*
*	Purpose:
*
*		Respond to activity on the Remote Throttle Enable control.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_rt_enable_button(HWND dlg, df_t* data)
{
	HWND	control;
	int		state;

	control	= GetDlgItem(dlg, IDC_DF_RT_ENABLE_BUTTON);
	state	= (int) SendMessage(control, BM_GETCHECK, 0, 0);

	if (state == BST_CHECKED)
		ctrl_menu_set(dlg, IDC_DF_AUTO_START, CTRL_BM_UNCHECK);

	data->changes	= TRUE;
	_refresh_window(dlg, data->device, NULL);
}



/*****************************************************************************
*
*	Function:	_idcancel
*
*	Purpose:
*
*		Respond to a request to close the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idcancel(HWND dlg)
{
	df_t*	data;

	main_refresh_add(dlg, NULL);
	data	= (df_t*) GET_WINDOW_USER_DATA(dlg);

	if (data)
		data->dlg	= NULL;

	win_pos_save(dlg, SECTION_NAME);
	DestroyWindow(dlg);
}



/*****************************************************************************
*
*	Function:	_idok
*
*	Purpose:
*
*		Process clicking of the OK button.
*
*	Arguments:
*
*		dlg			The dalog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idok(HWND dlg, df_t* data)
{
	_idc_apply(dlg, data);
	_idcancel(dlg);
}



/*****************************************************************************
*
*	Function:	_wm_command
*
*	Purpose:
*
*		Process WM_COMMAND messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's WORD parameter.
*
*		lParam		The message's LONG parameter.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_command(HWND dlg, WPARAM wParam, LPARAM lParam)
{
	df_t*	data	= (df_t*) GET_WINDOW_USER_DATA(dlg);
	int		id		= LOWORD(wParam);
	LRESULT	ret		= TRUE;

	switch (id)
	{
		default:

			ret	= FALSE;
			break;

		case IDC_APPLY:

			_idc_apply(dlg, data);
			break;

		case IDC_COPY:

			_idc_copy(dlg);
			break;

		case IDC_DEFAULT:

			_idc_default(dlg, data);
			break;

		case IDC_DF_AUTO_START:

			_idc_rt_auto_start(dlg, data);
			break;

		case IDC_DF_AUTO_STOP:

			_idc_rt_auto_stop(dlg, data);
			break;

		case IDC_DF_RT_ENABLE_BUTTON:

			_idc_rt_enable_button(dlg, data);
			break;

		case IDC_DF_RT_MODE_BUTTON:

			dlg	= GetParent(dlg);
			cable_command_3_service(dlg, data->device, SERVICE_WM_COMMAND, (ULONG) wParam, (ULONG) lParam);
			break;

		case IDCANCEL:

			_idcancel(dlg);
			break;

		case IDOK:

			_idok(dlg, data);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_wm_destroy
*
*	Purpose:
*
*		Process WM_DESTROY messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_destroy(HWND dlg)
{
	_idcancel(dlg);
	DestroyWindow(dlg);
	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_initdialog
*
*	Purpose:
*
*		Process a WM_INITDIALOG message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		lParam		The message's long sized argument.
*
*	Returned:
*
*		TRUE		Windows should set the initial control focus.
*		FALSE		Windows should NOT set the initial control focus.
*
*****************************************************************************/

static LRESULT _wm_initdialog(HWND dlg, LPARAM lParam)
{
	df_t*	data	= (df_t*) lParam;

	SET_WINDOW_USER_DATA(dlg, lParam);

	SendMessage(dlg, WM_SETICON, ICON_SMALL, (LPARAM) data->device->icon.little);
	SendMessage(dlg, WM_SETICON, ICON_BIG, (LPARAM) data->device->icon.big);

	win_pos_restore(dlg, SECTION_NAME, FALSE);
	main_refresh_add(dlg, _refresh_window);
	return(TRUE);
}



/*****************************************************************************
*
*	Function:	_wm_syscommand
*
*	Purpose:
*
*		Process a WM_SYSCOMMAND message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's word sized parameter.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_syscommand(HWND dlg, WPARAM wParam)
{
	LRESULT	ret;

	switch (wParam)
	{
		default:

			ret	= FALSE;
			break;

		case SC_CLOSE:

			_idcancel(dlg);
			ret	= TRUE;
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_dlg_proc
*
*	Purpose:
*
*		Processes messages for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		msg			The message to process.
*
*		wParam		The message's word sized parameter.
*
*		lParam		The message's long sized parameter.
*
*	Returned:
*
*		A message specific value.
*
*****************************************************************************/

static INT_PTR CALLBACK _dlg_proc(	HWND	dlg,
									UINT	msg,
									WPARAM	wParam,
									LPARAM	lParam)
{
	LRESULT	ret;

	switch (msg)
	{
		default:

			ret	= FALSE;
			break;

		case WM_COMMAND:

			ret	= _wm_command(dlg, wParam, lParam);
			break;

		case WM_DESTROY:

			ret	= _wm_destroy(dlg);
			break;

		case WM_INITDIALOG:

			ret	= _wm_initdialog(dlg, lParam);
			break;

		case WM_SYSCOMMAND:

			ret	= _wm_syscommand(dlg, wParam);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_auto_stop_cfg_wiz
*
*	Purpose:
*
*		Provide customized processing for the Configuration Wizards processing
*		of the Auto Stop configuration option.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		device		The device structure to access.
*
*		entry		The structure for the parameter to process.
*
*		value		The current argument value.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _auto_stop_cfg_wiz(	HWND				dlg,
								device_t*			device,
								const parm_entry_t*	entry,
								U32					value)
{
	char	buf[4096];
	HWND	control;
	U32		feature;
	int		id;
	char*	psz;
	U32		status;

	status	= HPDI32_MISC_FEATURES__TX_AUTO_STOP(device->handle, &feature);

	if (feature == HPDI32_MISC_FEATURES_ABSENT)
	{
		control	= GetDlgItem(dlg, IDC_CFG_WIZ_DATA_EDIT);
		SendMessage(control, WM_GETTEXT, (WPARAM) sizeof(buf), (LPARAM) buf);
		strcat(	buf,
				" This HPDI32 does not support the Auto Stop feature.");
		SetWindowText(control, buf);

		if (_auto_stop_value_list[0].value == HPDI32_TX_AUTO_STOP_NO)
			id	= IDC_CFG_WIZ_DATA_RADIO_1;
		else
			id	= IDC_CFG_WIZ_DATA_RADIO_2;

		control	= GetDlgItem(dlg, id);
		SendMessage(control, WM_GETTEXT, (WPARAM) sizeof(buf), (LPARAM) buf);
		psz	= strchr(buf, '(');

		if (psz)
			psz[0]	= 0;

		strcat(buf, " (This option is not available on this HPDI32.)");
		SetWindowText(control, buf);
		EnableWindow(control, FALSE);
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_service_refresh
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_REFRESH service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_refresh(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	char	buf[128];
	U32		mode;
	U32		start;
	U32		stat;
	U32		status	= GSC_SUCCESS;
	U32		stop;
	U32		rt;

	if (device->handle)
	{
		stat	= HPDI32_TX_AUTO_START__GET(device->handle, &start);
		status	= (status == GSC_SUCCESS) ? stat : status;
		stat	= HPDI32_TX_AUTO_STOP__GET(device->handle, &stop);
		status	= (status == GSC_SUCCESS) ? stat : status;
		stat	= HPDI32_CABLE_COMMAND_MODE__RR_GET(device->handle, &mode);
		status	= (status == GSC_SUCCESS) ? stat : status;
		stat	= HPDI32_TX_REMOTE_THROTTLE__GET(device->handle, &rt);
		status	= (status == GSC_SUCCESS) ? stat : status;

		strcpy(buf, "Auto Start ");

		if (start == HPDI32_TX_AUTO_START_YES)
			strcat(buf, "Enabled");
		else
			strcat(buf, "Disabled");

		strcat(buf, ", Auto Stop ");

		if (stop == HPDI32_TX_AUTO_STOP_YES)
			strcat(buf, "Enabled");
		else
			strcat(buf, "Disabled");

		strcat(buf, ", Remote Throttle ");

		if ((mode == HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL) &&
			(rt == HPDI32_TX_REMOTE_THROTTLE_ENABLE))
		{
			strcat(buf, "Enabled");
		}
		else if (rt == HPDI32_TX_REMOTE_THROTTLE_DISABLE)
		{
			strcat(buf, "Disabled");
		}
		else
		{
			strcat(buf, "Disabled (GPIO)");
		}

		ctrl_menu_set(dlg, IDC_TX_DF, CTRL_ENABLE);
	}
	else
	{
		ctrl_menu_set(dlg, IDC_TX_DF, CTRL_DISABLE);
	}

	error_status(dlg, IDC_INFO_TEXT, status);
	return(0);
}



/*****************************************************************************
*
*	Function:	_service_wm_command
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_WM_COMMAND service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_wm_command(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static	df_t	data;

	if (data.dlg)	/*lint !e727 */
	{
		SetFocus(data.dlg);
	}
	else
	{
		data.auto_start.defalt	= HPDI32_TX_AUTO_START_DEFAULT;
		data.auto_stop.defalt	= HPDI32_TX_AUTO_STOP_DEFAULT;
		data.changes			= FALSE;
		data.device				= device;
		data.rt_enable.defalt	= HPDI32_TX_REMOTE_THROTTLE_DEFAULT;
		data.was_open			= FALSE;
		data.dlg				= CreateDialogParam(device->instance,
													(LPCTSTR) IDD_DF,
													dlg,
													(DLGPROC) _dlg_proc,
													(LPARAM) &data);
		ShowWindow(data.dlg, SW_SHOWNORMAL);
		_refresh_window(data.dlg, device, NULL);
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	tx_data_flow_service
*
*	Purpose:
*
*		Implement the common service entry point for the Tx Data Flow services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT tx_data_flow_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service				function			*/
		{ SERVICE_REFRESH,		_service_refresh	},
		{ SERVICE_WM_COMMAND,	_service_wm_command	},
		{ SERVICE_END_LIST,		NULL				}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}


