// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/tx_sv.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement support for the Tx Status Valid configuration options.
*
*****************************************************************************/

#include <windows.h>
#include <stdio.h>

#include "tx.h"
#include "align.h"
#include "arrow.h"
#include "button.h"
#include "cable.h"
#include "copy.h"
#include "gui.h"
#include "resource.h"
#include "util_ctrl.h"



/* #defines	*****************************************************************/

#define	SECTION_NAME	"tx status valid"



/* typedefs	*****************************************************************/

typedef struct
{
	U32		defalt;		// MUST BE SET BY CODE WANTING THE DIALOG!
	int		ignore;		// Ignore certain activities.
	U32		setting;	// The device/dialog setting is recorded here.
} _sv_t;

typedef struct
{
	int			changes;	// Are there changes to apply?
	device_t*	device;
	HWND		dlg;
	_sv_t		mode;
	_sv_t		on;
	int			was_open;	// Was the device already open?
} sv_t;



/* variables	*************************************************************/

static const value_range_t	_status_valid_value_range	=
{
	/* min	*/	0,
	/* max	*/	HPDI32_TX_STATUS_VALID_COUNT_MAX,
	/* k	*/	1024
};

static const value_t	_status_valid_value	=
{
	/* type	*/	VALUE_TYPE_RANGE,
	{
	/* u.vp	*/	&_status_valid_value_range
	}
};

const parm_entry_t	tx_parm_status_valid	=
{
	/* title			*/	"Status Valid Count",
	/* parm + parm_text	*/	VAR_STR_MACRO(HPDI32_TX_STATUS_VALID_COUNT),
	/* suffix			*/	"",
	/* which			*/	0,
	/* set_arg			*/	"",
	/* get_arg			*/	"",
	/* values			*/	&_status_valid_value,
	/* cfg_wiz			*/	NULL,
	/* desc				*/
	"The parameter controls the length of the Status Valid period. The"
	" Status Valid period appears at the start of each frame. The value"
	" specified refers to the number of Cable Clock cycles that Status Valid"
	" will remain active during the Status Valid period. Data is transferred"
	" during the Status Valid period. If the Tx FIFO goes empty during the"
	" Status Valid period, then the frame is ended and Status Valid becomes"
	" inactive."
};



/*****************************************************************************
*
*	Function:	_mode_device_read
*
*	Purpose:
*
*		Read the parameter from the device into memory.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _mode_device_read(sv_t* data)
{
	U32	status;

	status	= HPDI32_CABLE_COMMAND_MODE__SV_GET(data->device->handle,
												&data->mode.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_mode_dialog_write
*
*	Purpose:
*
*		Write the parameter held in memory to the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _mode_dialog_write(HWND dlg, sv_t* data)
{
	unsigned int	flags;
	const char*		psz;

	if (data->device->handle)
	{
		flags	= CTRL_ENABLE;

		if (data->mode.setting == HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL)
			psz	= "Flow Control";
		else
			psz	= "GPIO";
	}
	else
	{
		flags	= CTRL_DISABLE;
		psz		= "";
	}

	ctrl_menu_set(dlg, IDC_SV_MODE_BUTTON, flags);
	ctrl_menu_set(dlg, IDC_SV_MODE_TEXT, flags);
	ctrl_menu_set_text(dlg, IDC_SV_MODE_TEXT, psz);
	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	_on_count_device_read
*
*	Purpose:
*
*		Read the parameter from the device into memory.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _on_count_device_read(sv_t* data)
{
	U32	status;

	status	= HPDI32_TX_STATUS_VALID_COUNT__GET(	data->device->handle,
													&data->on.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_on_count_device_write
*
*	Purpose:
*
*		Write the parameter held in memory to the device.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _on_count_device_write(sv_t* data)
{
	U32	status;

	status	= HPDI32_TX_STATUS_VALID_COUNT__SET(	data->device->handle,
													data->on.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_on_count_dialog_read
*
*	Purpose:
*
*		Read the parameter from the dialog into memory.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _on_count_dialog_read(HWND dlg, sv_t* data)
{
	U32	status;

	status	= ctrl_text_2_u32(	dlg,
								IDC_SV_COUNT_EDIT,
								_status_valid_value_range.k,
								&data->on.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_on_count_dialog_write
*
*	Purpose:
*
*		Apply the current Status Valid On Count setting to the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _on_count_dialog_write(HWND dlg, sv_t* data)
{
	char			buf[32];
	unsigned int	flags;

	if (data->device->handle)
	{
		flags	= CTRL_ENABLE;
		format_u32(data->on.setting, _status_valid_value_range.k, buf);
	}
	else
	{
		flags	= CTRL_DISABLE;
		buf[0]	= 0;
	}

	ctrl_menu_set(dlg, IDC_SV_COUNT_TITLE, flags);
	ctrl_menu_set(dlg, IDC_SV_COUNT_EDIT, flags);
	ctrl_menu_set(dlg, IDC_SV_COUNT_INFO, flags);

	if (data->on.ignore == FALSE)
	{
		data->on.ignore	= TRUE;
		ctrl_menu_set_text(dlg, IDC_SV_COUNT_EDIT, buf);
		data->on.ignore	= FALSE;
	}

	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	_device_read
*
*	Purpose:
*
*		Read the parameters from the device into memory.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _device_read(sv_t* data)
{
	U32	stat;
	U32	status	= GSC_SUCCESS;

	stat	= _mode_device_read(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _on_count_device_read(data);
	status	= (status == GSC_SUCCESS) ? stat : status;

	return(status);
}



/*****************************************************************************
*
*	Function:	_device_write
*
*	Purpose:
*
*		Write the parameters held in memory to the device.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _device_write(sv_t* data)
{
	U32	stat;
	U32	status	= GSC_SUCCESS;

	stat	= _on_count_device_write(data);
	status	= (status == GSC_SUCCESS) ? stat : status;

	return(status);
}



/*****************************************************************************
*
*	Function:	_dialog_read
*
*	Purpose:
*
*		Read the parameters from the dialog into memory.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _dialog_read(HWND dlg, sv_t* data)
{
	U32	stat;
	U32	status	= GSC_SUCCESS;

	stat	= _on_count_dialog_read(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;

	return(status);
}



/*****************************************************************************
*
*	Function:	_dialog_write
*
*	Purpose:
*
*		Write the parameters held in memory to the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _dialog_write(HWND dlg, sv_t* data)
{
	U32	stat;
	U32	status	= GSC_SUCCESS;

	stat	= _mode_dialog_write(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _on_count_dialog_write(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;

	return(status);
}



/*****************************************************************************
*
*	Function:	_refresh_window
*
*	Purpose:
*
*		Update the state of the window.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		msg		A status message we're to report.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _refresh_window(HWND dlg, device_t* device, const char* msg)
{
	sv_t*			data;
	unsigned int	flags;
	HMENU			menu	= GetMenu(dlg);
	U32				stat;
	U32				status	= GSC_SUCCESS;
	char			title[64];

	data	= (sv_t*) GET_WINDOW_USER_DATA(dlg);

	data->changes	= device->handle ? data->changes : FALSE;
	strcpy(title, "Tx Status Valid");
	strcat(title, data->changes ? " *" : "");
	set_win_title(dlg, data->device, FALSE, TRUE, title);

	if (data->changes)
	{
		ModifyMenu(menu, IDCANCEL, MF_BYCOMMAND | MF_STRING, IDCANCEL, "&Cancel");
		SetDlgItemText(dlg, IDCANCEL, "Cancel");
	}
	else
	{
		ModifyMenu(menu, IDCANCEL, MF_BYCOMMAND | MF_STRING, IDCANCEL, "&Close");
		SetDlgItemText(dlg, IDCANCEL, "Close");
	}

	if (device->handle)
	{
		if ((data->was_open == FALSE) || (data->changes == FALSE))
		{
			stat	= _device_read(data);
			status	= (status == GSC_SUCCESS) ? stat : status;
			stat	= _dialog_write(dlg, data);
			status	= (status == GSC_SUCCESS) ? stat : status;
		}

		data->was_open	= TRUE;
		flags			= CTRL_ENABLE;
	}
	else
	{
		// Everything is essentially disabled.
		data->changes	= FALSE;
		data->was_open	= FALSE;
		flags	= CTRL_DISABLE;
		stat	= _dialog_write(dlg, data);
		status	= (status == GSC_SUCCESS) ? stat : status;
		ctrl_menu_set(dlg, IDCANCEL, CTRL_FOCUS);
	}

	ctrl_menu_set(dlg, IDOK, flags);
	ctrl_menu_set(dlg, IDC_APPLY, flags);
	ctrl_menu_set(dlg, IDC_DEFAULT, flags);
	error_status(dlg, 0, status);
}



/*****************************************************************************
*
*	Function:	_idc_apply
*
*	Purpose:
*
*		Respond to an APPLY request.
*
*	Arguments:
*
*		dlg			The dalog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_apply(HWND dlg, sv_t* data)
{
	U32	stat;
	U32	status	= GSC_SUCCESS;

	data->changes	= FALSE;

	stat	= _dialog_read(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _device_write(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _device_read(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _dialog_write(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;

	main_refresh(data->device, NULL);
	error_status(dlg, 0, status);
}



/*****************************************************************************
*
*	Function:	_idc_copy
*
*	Purpose:
*
*		Implement a COPY function for the dialog content.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_copy(HWND dlg)
{
	static const copy_t	list[]	=
	{
		/* id					flags										*/
		{ -1,					COPY_EOL | COPY_UNDERLINE					},

		{ IDC_SV_MODE_BUTTON,	COPY_WIDTH | 11								},
		{ IDC_SV_MODE_TEXT,		COPY_EOL | COPY_DISABLED					},

		{ IDC_SV_COUNT_TITLE,	COPY_WIDTH | 11								},
		{ IDC_SV_COUNT_EDIT,	0											},
		{ IDC_SV_COUNT_INFO,	COPY_INDENT | COPY_EOL | COPY_DISABLED		},

		{ 0,					COPY_UNDERLINE								},
		{ 0,					COPY_EOL									},

		{ IDOK,					COPY_DISABLED | COPY_EOL | COPY_WIDTH | 8	},
		{ IDCANCEL,				COPY_DISABLED | COPY_EOL | COPY_WIDTH | 8	},
		{ IDC_APPLY,			COPY_DISABLED | COPY_EOL | COPY_WIDTH | 8	},
		{ IDC_DEFAULT,			COPY_DISABLED | COPY_EOL | COPY_WIDTH | 8	}
	};

	copy_list_to_clipboard(dlg, list, ARRAY_ELEMENTS(list));
}



/*****************************************************************************
*
*	Function:	_idc_default
*
*	Purpose:
*
*		Respond to a DEFAULT request.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_default(HWND dlg, sv_t* data)
{
	U32	status;

	data->changes	= TRUE;

	data->on.setting	= data->on.defalt;

	status	= _dialog_write(dlg, data);

	_refresh_window(dlg, data->device, NULL);
	error_status(dlg, 0, status);
}



/*****************************************************************************
*
*	Function:	_idc_sv_count_edit
*
*	Purpose:
*
*		Respond to activity in the Status Valid Count edit control.
*
*	Arguments:
*
*		dlg			The dalog handle.
*
*		wParam		The message's WORD sized parameter.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_sv_count_edit(HWND dlg, WPARAM wParam, sv_t* data)
{
	int	code	= HIWORD(wParam);

	if ((code == EN_CHANGE) && (data->on.ignore == FALSE))
	{
		data->changes	= TRUE;
		_refresh_window(dlg, data->device, NULL);
	}
}



/*****************************************************************************
*
*	Function:	_idcancel
*
*	Purpose:
*
*		Respond to a request to close the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idcancel(HWND dlg)
{
	sv_t*	data;

	main_refresh_add(dlg, NULL);
	data	= (sv_t*) GET_WINDOW_USER_DATA(dlg);

	if (data)
		data->dlg	= NULL;

	win_pos_save(dlg, SECTION_NAME);
	DestroyWindow(dlg);
}



/*****************************************************************************
*
*	Function:	_idok
*
*	Purpose:
*
*		Process clicking of the OK button.
*
*	Arguments:
*
*		dlg			The dalog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idok(HWND dlg, sv_t* data)
{
	_idc_apply(dlg, data);
	_idcancel(dlg);
}



/*****************************************************************************
*
*	Function:	_wm_command
*
*	Purpose:
*
*		Process WM_COMMAND messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's WORD parameter.
*
*		lParam		The message's LONG parameter.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_command(HWND dlg, WPARAM wParam, LPARAM lParam)
{
	sv_t*	data	= (sv_t*) GET_WINDOW_USER_DATA(dlg);
	int		id		= LOWORD(wParam);
	LRESULT	ret		= TRUE;

	switch (id)
	{
		default:

			ret	= FALSE;
			break;

		case IDC_APPLY:

			_idc_apply(dlg, data);
			break;

		case IDC_COPY:

			_idc_copy(dlg);
			break;

		case IDC_DEFAULT:

			_idc_default(dlg, data);
			break;

		case IDC_SV_COUNT_EDIT:

			_idc_sv_count_edit(dlg, wParam, data);
			break;

		case IDC_SV_MODE_BUTTON:

			dlg	= GetParent(dlg);
			cable_command_2_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDCANCEL:

			_idcancel(dlg);
			break;

		case IDOK:

			_idok(dlg, data);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_wm_destroy
*
*	Purpose:
*
*		Process WM_DESTROY messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_destroy(HWND dlg)
{
	_idcancel(dlg);
	DestroyWindow(dlg);
	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_initdialog
*
*	Purpose:
*
*		Process a WM_INITDIALOG message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		lParam		The message's long sized argument.
*
*	Returned:
*
*		TRUE		Windows should set the initial control focus.
*		FALSE		Windows should NOT set the initial control focus.
*
*****************************************************************************/

static LRESULT _wm_initdialog(HWND dlg, LPARAM lParam)
{
	char	buf[32];
	HWND	control;
	sv_t*	data	= (sv_t*) lParam;

	SET_WINDOW_USER_DATA(dlg, lParam);

	SendMessage(dlg, WM_SETICON, ICON_SMALL, (LPARAM) data->device->icon.little);
	SendMessage(dlg, WM_SETICON, ICON_BIG, (LPARAM) data->device->icon.big);

	control	= GetDlgItem(dlg, IDC_SV_COUNT_EDIT);
	SendMessage(control, EM_LIMITTEXT, 12, 0);

	sprintf(buf, "(1K = %d)", _status_valid_value_range.k);
	ctrl_menu_set_text(dlg, IDC_SV_COUNT_INFO, buf);

	win_pos_restore(dlg, SECTION_NAME, FALSE);
	main_refresh_add(dlg, _refresh_window);
	return(TRUE);
}



/*****************************************************************************
*
*	Function:	_wm_syscommand
*
*	Purpose:
*
*		Process a WM_SYSCOMMAND message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's word sized parameter.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_syscommand(HWND dlg, WPARAM wParam)
{
	LRESULT	ret;

	switch (wParam)
	{
		default:

			ret	= FALSE;
			break;

		case SC_CLOSE:

			_idcancel(dlg);
			ret	= TRUE;
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_dlg_proc
*
*	Purpose:
*
*		Processes messages for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		msg			The message to process.
*
*		wParam		The message's word sized parameter.
*
*		lParam		The message's long sized parameter.
*
*	Returned:
*
*		A message specific value.
*
*****************************************************************************/

static INT_PTR CALLBACK _dlg_proc(	HWND	dlg,
									UINT	msg,
									WPARAM	wParam,
									LPARAM	lParam)
{
	LRESULT	ret;

	switch (msg)
	{
		default:

			ret	= FALSE;
			break;

		case WM_COMMAND:

			ret	= _wm_command(dlg, wParam, lParam);
			break;

		case WM_DESTROY:

			ret	= _wm_destroy(dlg);
			break;

		case WM_INITDIALOG:

			ret	= _wm_initdialog(dlg, lParam);
			break;

		case WM_SYSCOMMAND:

			ret	= _wm_syscommand(dlg, wParam);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_gui_paint
*
*	Purpose:
*
*		Provide additional painting support for the GUI Tx Status Valid
*		control.
*
*	Arguments:
*
*		control		The handle to the control.
*
*		data		The control specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _gui_paint(HWND control, button_t* data)
{
	static const wave_t	off[]	=
	{
		{
		/* flags	*/	WAVE_BOTTOM,
		/* skip		*/	0,
		/* count	*/	1,
		/* w1		*/	47,
		/* w2		*/	0
		},
		{ 0 }
	};

	static const wave_t	on[]	=
	{
		{
		/* flags	*/	WAVE_BOTTOM,
		/* skip		*/	0,
		/* count	*/	1,
		/* w1		*/	2,
		/* w2		*/	1
		},
		{
		/* flags	*/	WAVE_BOTTOM,
		/* skip		*/	0,
		/* count	*/	1,
		/* w1		*/	44,
		/* w2		*/	0
		},
		{ 0 }
	};

	HWND	dlg;
	U32		status;
	U32		sv;
	int		x;
	int		y;

	x	= data->inside2.rect.left
		+ data->arrow1.offset
		+ data->arrow1.width
		+ MARGIN
		+ data->offset;
	y	= data->inside2.rect.top
		+ data->offset;

	x	+= button_text_paint(control, data, x, y, " ");

	if (data->device->handle)
	{
		status	= HPDI32_TX_STATUS_VALID_COUNT__GET(data->device->handle, &sv);
		dlg		= GetParent(control);
		error_status(dlg, IDC_INFO_TEXT, status);

		if (sv)
			button_wave_paint(data, x, y, on);
		else
			button_wave_paint(data, x, y, off);
	}
	else
	{
		button_wave_paint(data, x, y, off);
	}
}



/*****************************************************************************
*
*	Function:	_service_init
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_INIT service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_init(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static const align_t list[]	=
	{
		{
		/* id		*/	IDC_TX_SV,
		/* ref		*/	IDC_TX_LV,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{
		/* id		*/	IDC_TX_SV,
		/* ref		*/	IDC_TX_LV,
		/* flags	*/	ALIGN_T_B | ALIGN_H,
		/* value	*/	MARGIN
		},
		{
		/* id		*/	IDC_TX_SV_ARROW,
		/* ref		*/	IDC_TX_CLOCK_ARROW,
		/* flags	*/	ALIGN_L_L | ALIGN_R_R,
		/* value	*/	0
		},
		{
		/* id		*/	IDC_TX_SV_ARROW,
		/* ref		*/	IDC_TX_SV,
		/* flags	*/	ALIGN_T_T | ALIGN_B_B,
		/* value	*/	0
		},
		{ 0, 0, 0, 0 }
	};

	align_control_list(dlg, list);
	button_init(dlg, device, IDC_TX_SV);
	button_arrow_direction_set(dlg, IDC_TX_SV, +1);
	button_paint_function_set(dlg, IDC_TX_SV, _gui_paint);
	arrow_init(dlg, device, IDC_TX_SV_ARROW);
	arrow_type_set(dlg, IDC_TX_SV_ARROW, ARROW_RIGHT);
	return(0);
}



/*****************************************************************************
*
*	Function:	_service_refresh
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_REFRESH service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_refresh(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	HWND	control;
	U32		enable;
	U32		mode;
	U32		status;

	for (;;)	// A convenience loop.
	{
		if (device->handle == NULL)
		{
			// The device is closed.
			ctrl_menu_set(dlg, IDC_TX_SV, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_TX_SV_ARROW, CTRL_DISABLE);
			break;
		}

		// The device is open.
		status	= HPDI32_TX_ENABLE__GET(device->handle, &enable);
		error_status(dlg, IDC_INFO_TEXT, status);

		if (enable == HPDI32_TX_ENABLE_NO)
		{
			// The transmitter is disabled.
			ctrl_menu_set(dlg, IDC_TX_SV, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_TX_SV_ARROW, CTRL_DISABLE);
			break;
		}

		// The transmitter is enabled.
		status	= HPDI32_CABLE_COMMAND_MODE__SV_GET(device->handle, &mode);
		error_status(dlg, IDC_INFO_TEXT, status);

		if (mode == HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL)
		{
			// Status Valid is being driven on the signal.
			ctrl_menu_set(dlg, IDC_TX_SV, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_TX_SV_ARROW, CTRL_ENABLE);
			break;
		}

		// Status Valid is not in use.
		ctrl_menu_set(dlg, IDC_TX_SV, CTRL_ENABLE);
		ctrl_menu_set(dlg, IDC_TX_SV_ARROW, CTRL_DISABLE);
		break;
	}

	control	= GetDlgItem(dlg, IDC_TX_SV);
	InvalidateRect(control, NULL, TRUE);
	return(0);
}



/*****************************************************************************
*
*	Function:	_service_wm_command
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_WM_COMMAND service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_wm_command(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static	sv_t	data;

	if (data.dlg)	/*lint !e727 */
	{
		SetFocus(data.dlg);
	}
	else
	{
		data.changes	= FALSE;
		data.device		= device;
		data.on.defalt	= HPDI32_TX_STATUS_VALID_COUNT_DEFAULT;
		data.on.ignore	= TRUE;
		data.was_open	= FALSE;
		data.dlg		= CreateDialogParam(device->instance,
											(LPCTSTR) IDD_SV,
											dlg,
											(DLGPROC) _dlg_proc,
											(LPARAM) &data);
		ShowWindow(data.dlg, SW_SHOWNORMAL);
		data.on.ignore	= FALSE;
		_refresh_window(data.dlg, device, NULL);
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	tx_status_valid_service
*
*	Purpose:
*
*		Implement the common service entry point for the Tx Status Valid
*		services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT tx_status_valid_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service				function			*/
		{ SERVICE_INIT,			_service_init		},
		{ SERVICE_REFRESH,		_service_refresh	},
		{ SERVICE_WM_COMMAND,	_service_wm_command	},
		{ SERVICE_END_LIST,		NULL				}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}


