// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/tx_transmit.c $
// $Rev: 31452 $
// $Date: 2014-11-13 09:58:59 -0600 (Thu, 13 Nov 2014) $

/*****************************************************************************
*
*	Implement support for the transmitter's Tx Transmit option.
*
*****************************************************************************/

#include <windows.h>
#include <stdio.h>

#include "gui.h"
#include "tx.h"
#include "align.h"
#include "arrow.h"
#include "button.h"
#include "data_xfer.h"
#include "ini.h"
#include "resource.h"
#include "util_ctrl.h"



/*****************************************************************************
*
*	Function:	_get_state_info
*
*	Purpose:
*
*		Get the state info we need about the device.
*
*	Arguments:
*
*		device		The device data.
*
*		enabled		Tell the caller if the transmitter is enable.
*
*		fifo		Tell the caller if the transmitter has FIFOs.
*
*		width		Tell the caller of the device's data width in bits.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _get_state_info(device_t* device, int* enabled, int* fifo, int* width)
{
	U32	enable;
	U32	feature;
	U32	size;
	U32	stat;
	U32	status	= GSC_SUCCESS;

	stat		= HPDI32_IO_DATA_SIZE__TX_GET(device->handle, &size);
	status		= (status == GSC_SUCCESS) ? stat : status;

	switch (size)
	{
		default:
		case HPDI32_IO_DATA_SIZE_8_BITS:

			width[0]	= 8;
			break;

		case HPDI32_IO_DATA_SIZE_16_BITS:

			width[0]	= 16;
			break;

		case HPDI32_IO_DATA_SIZE_32_BITS:

			width[0]	= 32;
			break;
	}

	stat		= HPDI32_TX_ENABLE__GET(device->handle, &enable);
	status		= (status == GSC_SUCCESS) ? stat : status;
	enabled[0]	= (enable == HPDI32_TX_ENABLE_YES) ? TRUE : FALSE;
	stat		= HPDI32_FIFO_SIZE__TX_GET(device->handle, &size);
	status		= (status == GSC_SUCCESS) ? stat : status;
	stat		= HPDI32_MISC_FEATURES__FIFO_SIZE(device->handle, &feature);
	status		= (status == GSC_SUCCESS) ? stat : status;

	if ((feature == HPDI32_MISC_FEATURES_ABSENT) || (size))
		fifo[0]	= TRUE;
	else
		fifo[0]	= FALSE;

	return(status);
}



/*****************************************************************************
*
*	Function:	_refresh_window
*
*	Purpose:
*
*		Update the content of the window per the state of the system.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None;
*
*****************************************************************************/

static void _refresh_window(HWND dlg, data_xfer_t* data)
{
	HWND	control;
	U32		enabled;
	U32		fifos;
	BOOL	inactive	= data->active == FALSE;
	BOOL	idle_data	= (data->data.count) && (inactive);
	U32		size;
	U32		stat;
	U32		status	= GSC_SUCCESS;

	if (data->device->handle)
	{
		stat	= HPDI32_TX_ENABLE__GET(data->device->handle, &enabled);
		status	= (status == GSC_SUCCESS) ? stat : status;
		enabled	= (enabled == HPDI32_TX_ENABLE_YES) ? 1 : 0;
		stat	= HPDI32_FIFO_SIZE__TX_GET(data->device->handle, &size);
		status	= (status == GSC_SUCCESS) ? stat : status;
		stat	= HPDI32_MISC_FEATURES__FIFO_SIZE(data->device->handle, &fifos);
		status	= (status == GSC_SUCCESS) ? stat : status;
		fifos	= ((fifos == HPDI32_MISC_FEATURES_PRESENT) && (size)) ? 1 : 0;
		stat	= HPDI32_IO_DATA_SIZE__TX_GET(	data->device->handle,
												&data->data.size);
		status	= (status == GSC_SUCCESS) ? stat : status;
	}
	else
	{
		enabled			= 0;
		fifos			= 0;
		data->data.size	= HPDI32_IO_DATA_SIZE_32_BITS;
		SendMessage(dlg, WM_COMMAND, IDC_DATA_XFER_STOP, 0);
		ctrl_menu_set(dlg, IDCANCEL, CTRL_ENABLE | CTRL_FOCUS);
	}

	control	= GetDlgItem(dlg, IDC_DATA_XFER_START);
	EnableWindow(control, (BOOL) ((fifos) && (idle_data)));
	control	= GetDlgItem(dlg, IDC_DATA_XFER_FILE);
	EnableWindow(control, inactive);
	control	= GetDlgItem(dlg, IDC_DATA_XFER_AMOUNT_TEXT);
	EnableWindow(control, idle_data);
	control	= GetDlgItem(dlg, IDC_DATA_XFER_AMOUNT_EDIT);
	EnableWindow(control, idle_data);

	if (data->device->handle == NULL)
		;
	else if (fifos == 0)
		strcpy(data->status_local, "No FIFOs installed.");
	else if (!enabled)
		strcpy(data->status_local, "The transmitter is disabled.");

	if (status != GSC_SUCCESS)
		data->status	= status;
}



/*****************************************************************************
*
*	Function:	_wm_initdialog
*
*	Purpose:
*
*		Process a WM_INITDIALOG message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The dialog specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _wm_initdialog(HWND dlg, data_xfer_t* data)
{
	struct
	{
		int			old_id;
		HWND		control;
		int			new_id;
		const char*	new_title;
	} list[]	=
	{
		/* old_id				control	new_id					new_title	*/
		{ IDC_DATA_XFER_START,	0,		IDC_DATA_XFER_FILE,		"Load"		},
		{ IDC_DATA_XFER_STOP,	0,		IDC_DATA_XFER_START,	"Write"		},
		{ IDC_DATA_XFER_FILE,	0,		IDC_DATA_XFER_STOP,		"Stop"		}
	};

	char	buf[32];
	int		i;
	U32		repeat;

	for (i = 0; i < (int) ARRAY_ELEMENTS(list); i++)
		list[i].control	= GetDlgItem(dlg, list[i].old_id);

	for (i = 0; i < (int) ARRAY_ELEMENTS(list); i++)
		SetWindowLong(list[i].control, GWL_ID, list[i].new_id);

	for (i = 0; i < (int) ARRAY_ELEMENTS(list); i++)
		SetWindowText(list[i].control, list[i].new_title);

	ctrl_menu_set_text(dlg, IDC_DATA_XFER_AMOUNT_TEXT, "Repeat:");

	ini_u32_get(NULL, data->section, data->amount_key, &repeat, 1);
	sprintf(buf, "%lu", (long) repeat);
	ctrl_menu_set_text(dlg, IDC_DATA_XFER_AMOUNT_EDIT, buf);
}



/*****************************************************************************
*
*	Function:	_xfer_begin
*
*	Purpose:
*
*		Prepare for the transfer process to commense.
*
*	Arguments:
*
*		data		Our local data set.
*
*	Returned:
*
*		TRUE		Procede with the transfer.
*		FALSE		Do not procede with the transfer.
*
*****************************************************************************/

static int _xfer_begin(data_xfer_t* data)
{
	int	procede;

	for (;;)	// A convenience loop.
	{
		// Clear the API Buffer info if we're not using it.

		if (data->device->tx.app_buf)
		{
			data->api_buf		= 0;
			data->api_buf_size	= 0;
		}
		else if (data->api_buf == 0)
		{
			ctrl_menu_set_text(	data->dlg,
								IDC_DATA_XFER_STATUS,
								"Couldn't get API Buffer pointer.");
			procede	= FALSE;
			break;
		}

		data->repeat	= data->amount;
		data->xfer_tot	= data->data.count;
		procede			= TRUE;
		break;
	}

	return(procede);
}



/*****************************************************************************
*
*	Function:	_xfer_data
*
*	Purpose:
*
*		Prepare for the transfer process to commense.
*
*	Arguments:
*
*		data		The dialog specific data we track.
*
*		xfer		The number of bytes to transfer.
*
*		xfered		The number of bytes actually transferred.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _xfer_data(data_xfer_t* data, U32 xfer, U32* xfered)
{
	U32	status;

	if (data->api_buf_ptr)
	{
		memcpy(data->api_buf_ptr, data->app_buf_ptr, xfer);
		status	= hpdi32_write(data->device->handle, data->api_buf_ptr, xfer, xfered);

	}
	else
	{
		status	= hpdi32_write(data->device->handle, data->app_buf_ptr, xfer, xfered);
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_xfer_end
*
*	Purpose:
*
*		Prepare for the transfer process to commense.
*
*	Arguments:
*
*		data		Our local data set.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _xfer_end(data_xfer_t* data)
{
}



/*****************************************************************************
*
*	Function:	_xfer_file
*
*	Purpose:
*
*		Prompt for a file name and save the data.
*
*	Arguments:
*
*		data		The dialog specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _xfer_file(data_xfer_t* data)
{
	HCURSOR			cursor;
	char			dir[_MAX_PATH];
	FILE*			file;
	int				i;
	TCHAR			name[_MAX_PATH];
	OPENFILENAME	ofn;
	long			size;
	U32				u32;

	name[0]	= 0;
	GetCurrentDirectory(sizeof(dir), dir);

	// Initialize OPENFILENAME
	memset(&ofn, 0, sizeof(OPENFILENAME));
	ofn.lStructSize		= sizeof(OPENFILENAME);
	ofn.hwndOwner		= data->dlg;
	ofn.lpstrFilter		= "Binary Files (*.bin)\0*.bin\0"
							"All (*.*)\0*.*\0";
	ofn.nFilterIndex	= 1;
	ofn.lpstrFile		= name;
	ofn.nMaxFile		= sizeof(name);
	ofn.lpstrInitialDir	= dir;
	ofn.lpstrTitle		= "Load Transmit Data";
	ofn.Flags			= OFN_PATHMUSTEXIST
						| OFN_FILEMUSTEXIST
						| OFN_HIDEREADONLY;
	i					= GetOpenFileName(&ofn);

	for (;i;)
	{
		cursor	= SetCursor(data->device->cursor.wait);
		file	= fopen(name, "rb");

		if (file == NULL)
		{
			sprintf(data->status_local, "Unable to open '%s'.", name);
			SetCursor(cursor);
			break;
		}

		fseek(file, 0, SEEK_END);
		size	= ftell(file);
		fseek(file, 0, SEEK_SET);

		if (size == 0)
		{
			sprintf(data->status_local, "File '%s' contains no data.", name);
			fclose(file);
			SetCursor(cursor);
			break;
		}

		if (size > _1G)
		{
			sprintf(data->status_local,
					"File '%s' is too big (%lu bytes).",
					name,
					(long) size);
			fclose(file);
			SetCursor(cursor);
			break;
		}

		if ((size != (long) data->app_buf_size) && (data->app_buf))
		{
			free(data->app_buf);
			data->app_buf_size	= 0;
			data->app_buf		= NULL;
		}

		if (data->app_buf == NULL)
		{
			data->app_buf	= malloc((size_t) size);

			if (data->app_buf)
			{
				data->app_buf_size	= (U32) size;
			}
			else
			{
				sprintf(data->status_local, "A malloc(%lu) call failed.", (long) size);
				fclose(file);
				SetCursor(cursor);
				break;
			}
		}

		data->offset		= 0;
		u32	= (U32) fread(data->app_buf, 1, (size_t) size, file);

		if (u32 != (U32) size)
		{
			data->data.count	= 0;
			sprintf(data->status_local, "Read failure from '%s'.", name);
		}
		else
		{
			data->data.count	= (U32) size;
			sprintf(data->status_local,
					"Read %lu bytes from '%s'.",
					(long) data->data.count,
					name);
		}

		fclose(file);
		SetCursor(cursor);
		break;
	}
}



/*****************************************************************************
*
*	Function:	_xfer_progress
*
*	Purpose:
*
*		Update the progress of the read operation.
*
*	Arguments:
*
*		data	The dialog specific data we track.
*
*	Returned:
*
*		TRUE	Stop updating the progress.
*		FALSE	Continue updating the progress.
*
*****************************************************************************/

static int _xfer_progress(data_xfer_t* data)
{
	char		buf[64];
	char		buf1[32];
	char		buf2[32];
	float		bytes;
	long		elapsed;
	float		rate;
	int			stop;
	data_xfer_t	tmp	= data[0];

	if (tmp.status != GSC_SUCCESS)
	{
		bytes	= ((float) tmp.repeat * tmp.data.count) + tmp.xfered_tot;
		elapsed	= (long) tmp.time_stop - (long) tmp.time_start;
		stop	= TRUE;
		sprintf(buf,
				"Error 0x%lX. Repeat %lu, %lu bytes. ",
				(long) tmp.status,
				(long) tmp.repeat,
				(long) tmp.xfer_tot);
	}
	else if (tmp.abort)
	{
		bytes	= ((float) tmp.repeat * tmp.data.count) + tmp.xfered_tot;
		elapsed	= (long) tmp.time_stop - (long) tmp.time_start;
		stop	= TRUE;
		sprintf(buf,
				"Write aborted. Repeat %lu, %lu bytes. ",
				(long) tmp.repeat,
				(long) tmp.xfer_tot);
	}
	else if (tmp.active == FALSE)
	{
		bytes	= (float) tmp.repeat * tmp.data.count;
		elapsed	= (long) tmp.time_stop - (long) tmp.time_start;
		stop	= TRUE;
		sprintf(buf,
				"Wrote %lu x %lu bytes. ",
				(long) tmp.repeat,
				(long) tmp.xfer_tot);
	}
	else
	{
		bytes	= ((float) tmp.repeat * tmp.data.count) + tmp.xfered_tot;
		elapsed	= (long) timeGetTime() - (long) tmp.time_start;
		stop	= FALSE;
		sprintf(buf,
				"Writing ... repeat %lu, %lu bytes. ",
				(long) tmp.repeat,
				(long) tmp.xfer_tot);
	}

	rate	= (float) (1000.0 * bytes / elapsed);
	format_float(bytes, buf1);
	format_float(rate, buf2);
	sprintf(data->status_local,
			"%sWrote %s bytes in %ld.%03ld s => %sB at %sB/S.",
			buf,
			buf1,
			elapsed / 1000,
			elapsed % 1000,
			buf1,
			buf2);
	return(stop);
}



/*****************************************************************************
*
*	Function:	_service_init
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_INIT service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_init(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static const align_t list[]	=
	{
		{
		/* id		*/	IDC_TX_TRANSMIT,
		/* ref		*/	IDC_TX_OSCILLATOR,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{
		/* id		*/	IDC_TX_TRANSMIT,
		/* ref		*/	IDC_TX_OSCILLATOR,
		/* flags	*/	ALIGN_T_B | ALIGN_H,
		/* value	*/	MARGIN
		},
		{
		/* id		*/	IDC_TX_TRANSMIT_ARROW,
		/* ref		*/	IDC_TX_OSCILLATOR_ARROW,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{
		/* id		*/	IDC_TX_TRANSMIT_ARROW,
		/* ref		*/	IDC_TX_OSCILLATOR_ARROW,
		/* flags	*/	ALIGN_T_B | ALIGN_H,
		/* value	*/	MARGIN
		},
		{ 0, 0, 0, 0 }
	};

	align_control_list(dlg, list);
	button_init(dlg, device, IDC_TX_TRANSMIT);
	arrow_init(dlg, device, IDC_TX_TRANSMIT_ARROW);
	arrow_type_set(dlg, IDC_TX_TRANSMIT_ARROW, ARROW_RIGHT_2);
	return(0);
}



/*****************************************************************************
*
*	Function:	_service_refresh
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_REFRESH service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_refresh(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	char	buf[16];
	int		enabled;
	int		fifo;
	U32		status;
	int		width;

	for (;;)	// A convenience loop.
	{
		if (device->handle == NULL)
		{
			// The device is closed.
			ctrl_menu_set(dlg, IDC_TX_TRANSMIT, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_TX_TRANSMIT_ARROW, CTRL_DISABLE | CTRL_NULL);
			break;
		}

		// The device is open.
		status	= _get_state_info(device, &enabled, &fifo, &width);
		error_status(dlg, IDC_INFO_TEXT, status);

		if (fifo)
		{
			// The device has Tx FIFOs.
			sprintf(buf, "%d-bits", width);
			ctrl_menu_set(dlg, IDC_TX_TRANSMIT, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_TX_TRANSMIT_ARROW, CTRL_ENABLE);
			ctrl_menu_set_text(dlg, IDC_TX_TRANSMIT_ARROW, buf);
			break;
		}

		// The device has no Tx FIFOs.
		ctrl_menu_set(dlg, IDC_TX_TRANSMIT, CTRL_DISABLE);
		ctrl_menu_set(dlg, IDC_TX_TRANSMIT_ARROW, CTRL_DISABLE | CTRL_NULL);
		break;
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	_service_wm_command
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_WM_COMMAND service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_wm_command(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static	data_xfer_t	data;

	LRESULT				ret;

	data.amount_key		= "repeat";
	data.section		= "tx transmit";
	data.title			= "Data Write";
	data.which			= HPDI32_WHICH_TX;

	data.refresh_window	= _refresh_window;
	data.wm_initdialog	= _wm_initdialog;
	data.xfer_begin		= _xfer_begin;
	data.xfer_data		= _xfer_data;
	data.xfer_end		= _xfer_end;
	data.xfer_file		= _xfer_file;
	data.xfer_progress	= _xfer_progress;

	ret	= data_xfer_service(dlg, device, SERVICE_WM_COMMAND, (ULONG) &data, lParam);

	return(ret);
}



/*****************************************************************************
*
*	Function:	tx_transmit_service
*
*	Purpose:
*
*		Implement the common service entry point for the Tx Transmit services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT tx_transmit_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service				function			*/
		{ SERVICE_INIT,			_service_init		},
		{ SERVICE_REFRESH,		_service_refresh	},
		{ SERVICE_WM_COMMAND,	_service_wm_command	},
		{ SERVICE_END_LIST,		NULL				}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}


